module isrpia

  integer, parameter :: NCOMP  =   8
  integer, parameter :: NIONS  =  10
  integer, parameter :: NGASAQ =   3
  integer, parameter :: NSLDS  =  19
  integer, parameter :: NPAIR  =  23
  integer, parameter :: NZSR   = 100
! integer, parameter :: NERRMX =  25

  integer, parameter :: MAXIT  = 20

  integer, parameter :: NSWEEP = 4
  integer, parameter :: NDIV   = 5
  integer, parameter :: WFTYP  = 2
  integer, parameter :: NADJ   = 1

  real(8), parameter :: TINY   = 1.D-20
  real(8), parameter :: GREAT  = 1.D10
  real(8), parameter :: ZERO   = 0.D0
  real(8), parameter :: ONE    = 1.D0
  real(8), parameter :: TINY2  = 1.D-11
  real(8), parameter :: EPS    = 1.D-5
  real(8), parameter :: EPSACT = 5.D-2
  real(8), parameter :: R      = 82.0567D-6

  character(15), parameter :: version = '2.1 (07/19/09)'

! *** INPUT VARIABLES **************************************************

  integer :: METSTBL, IPROB
  real(8) :: W(NCOMP), WAER(NCOMP), TEMP, RH

  !$omp threadprivate( METSTBL, IPROB, W, WAER, TEMP, RH )

! *** DELIQUESCENCE RELATIVE HUMIDITIES ********************************

  real(8) :: DRH2SO4,  DRNH42S4, DRNAHSO4, DRNACL,   DRNANO3, &
             DRNA2SO4, DRNH4HS4, DRLC,     DRNH4NO3, DRNH4CL, &
             DRCASO4,  DRCANO32, DRCACL2,  DRK2SO4,  DRKHSO4, &
             DRKNO3,   DRKCL,    DRMGSO4,  DRMGNO32, DRMGCL2

  !$omp threadprivate( DRH2SO4,  DRNH42S4, DRNAHSO4, DRNACL,   DRNANO3, &
  !$omp                DRNA2SO4, DRNH4HS4, DRLC,     DRNH4NO3, DRNH4CL, &
  !$omp                DRCASO4,  DRCANO32, DRCACL2,  DRK2SO4,  DRKHSO4, &
  !$omp                DRKNO3,   DRKCL,    DRMGSO4,  DRMGNO32, DRMGCL2  )

  real(8) :: DRMLCAB,  DRMLCAS,  DRMASAN,  DRMG1,    DRMG2, &
             DRMG3,    DRMH1,    DRMH2,    DRMI1,    DRMI2, &
             DRMI3,    DRMQ1,    DRMR1,    DRMR2,    DRMR3, &
             DRMR4,    DRMR5,    DRMR6,    DRMR7,    DRMR8, &
             DRMR9,    DRMR10,   DRMR11,   DRMR12,   DRMR13

  !$omp threadprivate( DRMLCAB,  DRMLCAS,  DRMASAN,  DRMG1,    DRMG2, &
  !$omp                DRMG3,    DRMH1,    DRMH2,    DRMI1,    DRMI2, &
  !$omp                DRMI3,    DRMQ1,    DRMR1,    DRMR2,    DRMR3, &
  !$omp                DRMR4,    DRMR5,    DRMR6,    DRMR7,    DRMR8, &
  !$omp                DRMR9,    DRMR10,   DRMR11,   DRMR12,   DRMR13 )

  real(8) :: DRMO1,    DRMO2,    DRMO3,    DRML1,    DRML2, &
             DRML3,    DRMM1,    DRMM2,    DRMP1,    DRMP2, &
             DRMP3,    DRMP4,    DRMP5,    DRMV1

  !$omp threadprivate( DRMO1,    DRMO2,    DRMO3,    DRML1,    DRML2, &
  !$omp                DRML3,    DRMM1,    DRMM2,    DRMP1,    DRMP2, &
  !$omp                DRMP3,    DRMP4,    DRMP5,    DRMV1            )

! *** VARIABLES FOR LIQUID AEROSOL PHASE *******************************

  real(8) :: MOLAL(NIONS), MOLALR(NPAIR)
  real    :: IONIC
  real(8) :: WATER, M0I(NPAIR)
  real(8) :: GAMA(NPAIR), GAMIN(NPAIR), GAMOU(NPAIR)
  real(8) :: GASAQ(NGASAQ)
  real(8) :: COH, CHNO3, CHCL
  LOGICAL :: CALAOU, CALAIN, FRST, DRYF

  real, parameter :: ZZ(NPAIR) = (/ 1., 2., 1., 2., 1., 1., 2., 1., 1., 1., &
                                    1., 1., 2., 4., 2., 2., 2., 1., 1., 1., &
                                    4., 2., 2. /)

  real, parameter :: Z(NIONS) = (/ 1., 1., 1., 1., 2., 1., 1., 2., 1., 2. /)

  !$omp threadprivate( MOLAL, MOLALR, IONIC, WATER, M0I, GAMA, GAMIN, GAMOU, &
  !$omp                GASAQ, COH, CHNO3, CHCL, CALAOU, CALAIN, FRST, DRYF )

! *** VARIABLES FOR SOLID AEROSOL PHASE ********************************

      real(8) :: CH2SO4,  CNH42S4, CNH4HS4, CNACL,   CNA2SO4,         &
                 CNANO3,  CNH4NO3, CNH4CL,  CNAHSO4, CLC,     CCASO4, &
                 CCANO32, CCACL2,  CK2SO4,  CKHSO4,  CKNO3,   CKCL,   &
                 CMGSO4,  CMGNO32, CMGCL2

  !$omp threadprivate( CH2SO4,  CNH42S4, CNH4HS4, CNACL,   CNA2SO4,         &
  !$omp                CNANO3,  CNH4NO3, CNH4CL,  CNAHSO4, CLC,     CCASO4, &
  !$omp                CCANO32, CCACL2,  CK2SO4,  CKHSO4,  CKNO3,   CKCL,   &
  !$omp                CMGSO4,  CMGNO32, CMGCL2 )

! *** VARIABLES FOR GAS PHASE ******************************************

  real(8) :: GNH3, GHNO3, GHCL 

  !$omp threadprivate( GNH3, GHNO3, GHCL )

! *** EQUILIBRIUM CONSTANTS ********************************************

  real(8) :: XK1, XK2, XK3, XK4, XK5, XK6, XK7, XK8, XK9, XK10, &
             XK11,XK12,XK13,XK14,XKW, XK21,XK22,XK31,XK32,XK41, &
             XK42, XK15, XK16, XK17, XK18, XK19, XK20, XK23,    &
             XK24, XK25

  !$omp threadprivate( XK1, XK2, XK3, XK4, XK5, XK6, XK7, XK8, XK9, XK10, &
  !$omp                XK11,XK12,XK13,XK14,XKW, XK21,XK22,XK31,XK32,XK41, &
  !$omp                XK42, XK15, XK16, XK17, XK18, XK19, XK20, XK23,    &
  !$omp                XK24, XK25 )

! *** SOLUTION/INFO VARIABLES ********************************************

  character(15) :: SCASE
  real(8)       :: SULRATW, SULRAT, SODRAT, SO4RAT, CRNARAT, CRRAT
  integer       :: ICLACT

  !$omp threadprivate( SCASE, SULRATW, SULRAT, SODRAT, SO4RAT, CRNARAT, &
  !$omp                CRRAT, ICLACT )

! *** SOLUT COMMON *******************************************

  real(8) :: CHI1, CHI2,  CHI3,  CHI4,  CHI5,  CHI6,  CHI7,  CHI8,  &
             CHI9, CHI10, CHI11, CHI12, CHI13, CHI14, CHI15, CHI16, CHI17

  real(8) :: PSI1, PSI2,  PSI3,  PSI4,  PSI5,  PSI6,  PSI7,  PSI8,  &
             PSI9, PSI10, PSI11, PSI12, PSI13, PSI14, PSI15, PSI16, PSI17
  
  real(8) :: A1,  A2,  A3,  A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, &
             A14, A15, A16, A17

  real(8) :: LAMDA, KAPA

  !$omp threadprivate( CHI1, CHI2, CHI3, CHI4, CHI5, CHI6, CHI7, CHI8, &
  !$omp                CHI9, CHI10, CHI11, CHI12, CHI13, CHI14, CHI15, &
  !$omp                CHI16, CHI17, PSI1, PSI2, PSI3, PSI4, PSI5, PSI6, &
  !$omp                PSI7, PSI8, PSI9, PSI10, PSI11, PSI12, PSI13, &
  !$omp                PSI14, PSI15, PSI16, PSI17, A1, A2, A3, A4, A5, A6, &
  !$omp                A7, A8, A9, A10, A11, A12, A13, A14, A15, A16, A17, &
  !$omp                LAMDA, KAPA )

! *** WATER ACTIVITIES OF PURE SALT SOLUTIONS **************************

  ! ammonium sulfate
  real, parameter :: AWAS(NZSR) = (/ &
       187.72,187.72,187.72,187.72,187.72,187.72,187.72,187.72,187.72,187.72, &
       158.13,134.41,115.37,100.10, 87.86, 78.00, 70.00, 63.45, 58.02, 53.46, &
        49.59, 46.26, 43.37, 40.84, 38.59, 36.59, 34.79, 33.16, 31.67, 30.31, &
        29.07, 27.91, 26.84, 25.84, 24.91, 24.03, 23.21, 22.44, 21.70, 21.01, &
        20.34, 19.71, 19.11, 18.54, 17.99, 17.46, 16.95, 16.46, 15.99, 15.54, &
        15.10, 14.67, 14.26, 13.86, 13.47, 13.09, 12.72, 12.36, 12.01, 11.67, &
        11.33, 11.00, 10.68, 10.37, 10.06,  9.75,  9.45,  9.15,  8.86,  8.57, &
         8.29,  8.01,  7.73,  7.45,  7.18,  6.91,  6.64,  6.37,  6.10,  5.83, &
         5.56,  5.29,  5.02,  4.74,  4.47,  4.19,  3.91,  3.63,  3.34,  3.05, &
         2.75,  2.45,  2.14,  1.83,  1.51,  1.19,  0.87,  0.56,  0.26,  0.10 /)
  real, parameter :: AWASI(NZSR) = 1.0 / AWAS(1:NZSR)

  ! sodium nitrate
  real, parameter :: AWSN(NZSR) = (/ &
       394.54,394.54,394.54,394.54,394.54,394.54,394.54,394.54,394.54,394.54, &
       338.91,293.01,254.73,222.61,195.56,172.76,153.53,137.32,123.65,112.08, &
       102.26, 93.88, 86.68, 80.45, 75.02, 70.24, 66.02, 62.26, 58.89, 55.85, &
        53.09, 50.57, 48.26, 46.14, 44.17, 42.35, 40.65, 39.06, 37.57, 36.17, &
        34.85, 33.60, 32.42, 31.29, 30.22, 29.20, 28.22, 27.28, 26.39, 25.52, &
        24.69, 23.89, 23.12, 22.37, 21.65, 20.94, 20.26, 19.60, 18.96, 18.33, &
        17.72, 17.12, 16.53, 15.96, 15.40, 14.85, 14.31, 13.78, 13.26, 12.75, &
        12.25, 11.75, 11.26, 10.77, 10.29,  9.82,  9.35,  8.88,  8.42,  7.97, &
         7.52,  7.07,  6.62,  6.18,  5.75,  5.32,  4.89,  4.47,  4.05,  3.64, &
         3.24,  2.84,  2.45,  2.07,  1.70,  1.34,  0.99,  0.65,  0.31,  0.10 /)
  real, parameter :: AWSNI(NZSR) = 1.0 / AWSN(1:NZSR)

  ! sodium chloride
  real, parameter :: AWSC(NZSR) = (/ &
       28.16, 28.16, 28.16, 28.16, 28.16, 28.16, 28.16, 28.16, 28.16, 28.16, &
       27.17, 26.27, 25.45, 24.69, 23.98, 23.33, 22.72, 22.14, 21.59, 21.08, &
       20.58, 20.12, 19.67, 19.24, 18.82, 18.43, 18.04, 17.67, 17.32, 16.97, &
       16.63, 16.31, 15.99, 15.68, 15.38, 15.08, 14.79, 14.51, 14.24, 13.97, &
       13.70, 13.44, 13.18, 12.93, 12.68, 12.44, 12.20, 11.96, 11.73, 11.50, &
       11.27, 11.05, 10.82, 10.60, 10.38, 10.16,  9.95,  9.74,  9.52,  9.31, &
        9.10,  8.89,  8.69,  8.48,  8.27,  8.07,  7.86,  7.65,  7.45,  7.24, &
        7.04,  6.83,  6.62,  6.42,  6.21,  6.00,  5.79,  5.58,  5.36,  5.15, &
        4.93,  4.71,  4.48,  4.26,  4.03,  3.80,  3.56,  3.32,  3.07,  2.82, &
        2.57,  2.30,  2.04,  1.76,  1.48,  1.20,  0.91,  0.61,  0.30,  0.10 /)
  real, parameter :: AWSCI(NZSR) = 1.0 / AWSC(1:NZSR)

  ! ammonium chloride
  real, parameter :: AWAC(NZSR) = (/ &
       1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,1209.00,1209.00, &
       1067.60, 949.27, 848.62, 761.82, 686.04, 619.16, 559.55, 505.92, 457.25, 412.69, &
        371.55, 333.21, 297.13, 262.81, 229.78, 197.59, 165.98, 135.49, 108.57,  88.29, &
         74.40,  64.75,  57.69,  52.25,  47.90,  44.30,  41.27,  38.65,  36.36,  34.34, &
         32.52,  30.88,  29.39,  28.02,  26.76,  25.60,  24.51,  23.50,  22.55,  21.65, &
         20.80,  20.00,  19.24,  18.52,  17.83,  17.17,  16.54,  15.93,  15.35,  14.79, &
         14.25,  13.73,  13.22,  12.73,  12.26,  11.80,  11.35,  10.92,  10.49,  10.08, &
          9.67,   9.28,   8.89,   8.51,   8.14,   7.77,   7.42,   7.06,   6.72,   6.37, &
          6.03,   5.70,   5.37,   5.05,   4.72,   4.40,   4.08,   3.77,   3.45,   3.14, &
          2.82,   2.51,   2.20,   1.89,   1.57,   1.26,   0.94,   0.62,   0.31,   0.10 /)
  real, parameter :: AWACI(NZSR) = 1.0 / AWAC(1:NZSR)

! awss= sodium sulfate
  real, parameter :: AWSS(NZSR) = (/ &
       24.10, 24.10, 24.10, 24.10, 24.10, 24.10, 24.10, 24.10, 24.10, 24.10, &
       23.17, 22.34, 21.58, 20.90, 20.27, 19.69, 19.15, 18.64, 18.17, 17.72, &
       17.30, 16.90, 16.52, 16.16, 15.81, 15.48, 15.16, 14.85, 14.55, 14.27, &
       13.99, 13.73, 13.47, 13.21, 12.97, 12.73, 12.50, 12.27, 12.05, 11.84, &
       11.62, 11.42, 11.21, 11.01, 10.82, 10.63, 10.44, 10.25, 10.07,  9.89, &
        9.71,  9.53,  9.36,  9.19,  9.02,  8.85,  8.68,  8.51,  8.35,  8.19, &
        8.02,  7.86,  7.70,  7.54,  7.38,  7.22,  7.06,  6.90,  6.74,  6.58, &
        6.42,  6.26,  6.10,  5.94,  5.78,  5.61,  5.45,  5.28,  5.11,  4.93, &
        4.76,  4.58,  4.39,  4.20,  4.01,  3.81,  3.60,  3.39,  3.16,  2.93, &
        2.68,  2.41,  2.13,  1.83,  1.52,  1.19,  0.86,  0.54,  0.25,  0.10 /)
  real, parameter :: AWSSI(NZSR) = 1.0 / AWSS(1:NZSR)

  ! ammonium bisulfate
  real, parameter :: AWAB(NZSR) = (/ &
       312.84,312.84,312.84,312.84,312.84,312.84,312.84,312.84,312.84,312.84, &
       271.43,237.19,208.52,184.28,163.64,145.97,130.79,117.72,106.42, 96.64, &
        88.16, 80.77, 74.33, 68.67, 63.70, 59.30, 55.39, 51.89, 48.76, 45.93, &
        43.38, 41.05, 38.92, 36.97, 35.18, 33.52, 31.98, 30.55, 29.22, 27.98, &
        26.81, 25.71, 24.67, 23.70, 22.77, 21.90, 21.06, 20.27, 19.52, 18.80, &
        18.11, 17.45, 16.82, 16.21, 15.63, 15.07, 14.53, 14.01, 13.51, 13.02, &
        12.56, 12.10, 11.66, 11.24, 10.82, 10.42, 10.04,  9.66,  9.29,  8.93, &
         8.58,  8.24,  7.91,  7.58,  7.26,  6.95,  6.65,  6.35,  6.05,  5.76, &
         5.48,  5.20,  4.92,  4.64,  4.37,  4.09,  3.82,  3.54,  3.27,  2.99, &
         2.70,  2.42,  2.12,  1.83,  1.52,  1.22,  0.90,  0.59,  0.28,  0.10 /)
  real, parameter :: AWABI(NZSR) = 1.0 / AWAB(1:NZSR)

  ! sulfuric acid
  real, parameter :: AWSA(NZSR) = (/ &
       34.00, 33.56, 29.22, 26.55, 24.61, 23.11, 21.89, 20.87, 19.99, 18.45, &
       17.83, 17.26, 16.73, 16.25, 15.80, 15.38, 14.98, 14.61, 14.26, 13.93, &
       13.61, 13.30, 13.01, 12.73, 12.47, 12.21, 11.96, 11.72, 11.49, 11.26, &
       11.04, 10.83, 10.62, 10.42, 10.23, 10.03,  9.85,  9.67,  9.49,  9.31, &
        9.14,  8.97,  8.81,  8.65,  8.49,  8.33,  8.18,  8.02,  7.87,  7.73, &
        7.58,  7.44,  7.29,  7.15,  7.01,  6.88,  6.74,  6.61,  6.47,  6.34, &
        6.21,  6.07,  5.94,  5.81,  5.68,  5.55,  5.43,  5.30,  5.17,  5.04, &
        4.91,  4.78,  4.65,  4.52,  4.39,  4.26,  4.13,  4.00,  3.86,  3.73, &
        3.59,  3.45,  3.31,  3.17,  3.02,  2.87,  2.71,  2.56,  2.39,  2.22, &
        2.05,  1.87,  1.68,  1.48,  1.27,  1.04,  0.80,  0.55,  0.28,  0.10 /)
  real, parameter :: AWSAI(NZSR) = 1.0 / AWSA(1:NZSR)

  ! (NH4)3H(SO4)2
  real, parameter :: AWLC(NZSR) = (/ &
       125.37,125.37,125.37,125.37,125.37,125.37,125.37,125.37,125.37,125.37, &
       110.10, 97.50, 86.98, 78.08, 70.49, 63.97, 58.33, 53.43, 49.14, 45.36, &
        42.03, 39.07, 36.44, 34.08, 31.97, 30.06, 28.33, 26.76, 25.32, 24.01, &
        22.81, 21.70, 20.67, 19.71, 18.83, 18.00, 17.23, 16.50, 15.82, 15.18, &
        14.58, 14.01, 13.46, 12.95, 12.46, 11.99, 11.55, 11.13, 10.72, 10.33, &
         9.96,  9.60,  9.26,  8.93,  8.61,  8.30,  8.00,  7.72,  7.44,  7.17, &
         6.91,  6.66,  6.42,  6.19,  5.96,  5.74,  5.52,  5.31,  5.11,  4.91, &
         4.71,  4.53,  4.34,  4.16,  3.99,  3.81,  3.64,  3.48,  3.31,  3.15, &
         2.99,  2.84,  2.68,  2.53,  2.37,  2.22,  2.06,  1.91,  1.75,  1.60, &
         1.44,  1.28,  1.12,  0.95,  0.79,  0.62,  0.45,  0.29,  0.14,  0.10 /)
  real, parameter :: AWLCI(NZSR) = 1.0 / AWLC(1:NZSR)

  ! ammonium nitrate
  real, parameter :: AWAN(NZSR) = (/ &
       960.19,960.19,960.19,960.19,960.19,960.19,960.19,960.19,960.19,960.19, &
       853.15,763.85,688.20,623.27,566.92,517.54,473.91,435.06,400.26,368.89, &
       340.48,314.63,291.01,269.36,249.46,231.11,214.17,198.50,184.00,170.58, &
       158.15,146.66,136.04,126.25,117.24,108.97,101.39, 94.45, 88.11, 82.33, &
        77.06, 72.25, 67.85, 63.84, 60.16, 56.78, 53.68, 50.81, 48.17, 45.71, &
        43.43, 41.31, 39.32, 37.46, 35.71, 34.06, 32.50, 31.03, 29.63, 28.30, &
        27.03, 25.82, 24.67, 23.56, 22.49, 21.47, 20.48, 19.53, 18.61, 17.72, &
        16.86, 16.02, 15.20, 14.41, 13.64, 12.89, 12.15, 11.43, 10.73, 10.05, &
         9.38,  8.73,  8.09,  7.47,  6.86,  6.27,  5.70,  5.15,  4.61,  4.09, &
         3.60,  3.12,  2.66,  2.23,  1.81,  1.41,  1.03,  0.67,  0.32,  0.10 /)
  real, parameter :: AWANI(NZSR) = 1.0 / AWAN(1:NZSR)

  ! sodium bisulfate
  real, parameter :: AWSB(NZSR) = (/ &
       55.99, 55.99, 55.99, 55.99, 55.99, 55.99, 55.99, 55.99, 55.99, 55.99, &
       53.79, 51.81, 49.99, 48.31, 46.75, 45.28, 43.91, 42.62, 41.39, 40.22, &
       39.10, 38.02, 36.99, 36.00, 35.04, 34.11, 33.21, 32.34, 31.49, 30.65, &
       29.84, 29.04, 28.27, 27.50, 26.75, 26.01, 25.29, 24.57, 23.87, 23.17, &
       22.49, 21.81, 21.15, 20.49, 19.84, 19.21, 18.58, 17.97, 17.37, 16.77, &
       16.19, 15.63, 15.08, 14.54, 14.01, 13.51, 13.01, 12.53, 12.07, 11.62, &
       11.19, 10.77, 10.36,  9.97,  9.59,  9.23,  8.87,  8.53,  8.20,  7.88, &
        7.57,  7.27,  6.97,  6.69,  6.41,  6.14,  5.88,  5.62,  5.36,  5.11, &
        4.87,  4.63,  4.39,  4.15,  3.92,  3.68,  3.45,  3.21,  2.98,  2.74, &
        2.49,  2.24,  1.98,  1.72,  1.44,  1.16,  0.87,  0.57,  0.28,  0.10 /)
  real, parameter :: AWSBI(NZSR) = 1.0 / AWSB(1:NZSR)

! awpc= potassium chloride
  real, parameter :: AWPC(NZSR) = (/ &
       172.62, 165.75, 159.10, 152.67, 146.46, 140.45, 134.64, &
       129.03, 123.61, 118.38, 113.34, 108.48, 103.79, 99.27, &
       94.93, 90.74, 86.71, 82.84, 79.11, 75.53, 72.09, 68.79, &
       65.63, 62.59, 59.68, 56.90, 54.23, 51.68, 49.24, 46.91, &
       44.68, 42.56, 40.53, 38.60, 36.76, 35.00, 33.33, 31.75, &
       30.24, 28.81, 27.45, 26.16, 24.94, 23.78, 22.68, 21.64, &
       20.66, 19.74, 18.86, 18.03, 17.25, 16.51, 15.82, 15.16, &
       14.54, 13.96, 13.41, 12.89, 12.40, 11.94, 11.50, 11.08, &
       10.69, 10.32, 9.96, 9.62, 9.30, 8.99, 8.69, 8.40, 8.12, &
       7.85, 7.59, 7.33, 7.08, 6.83, 6.58, 6.33, 6.08, 5.84, &
       5.59, 5.34, 5.09, 4.83, 4.57, 4.31, 4.04, 3.76, 3.48, &
       3.19, 2.90, 2.60, 2.29, 1.98, 1.66, 1.33, 0.99, 0.65, &
       0.30, 0.1 /)
  real, parameter :: AWPCI(NZSR) = 1.0 / AWPC(1:NZSR)

  ! potassium sulfate
  real, parameter :: AWPS(NZSR) = (/ &
      1014.82, 969.72, 926.16, 884.11, 843.54, 804.41, 766.68, &
       730.32, 695.30, 661.58, 629.14, 597.93, 567.92, 539.09, &
       511.41, 484.83, 459.34, 434.89, 411.47, 389.04, 367.58, &
       347.05, 327.43, 308.69, 290.80, 273.73, 257.47, 241.98, &
       227.24, 213.22, 199.90, 187.26, 175.27, 163.91, 153.15, &
       142.97, 133.36, 124.28, 115.73, 107.66, 100.08, 92.95, &
       86.26, 79.99, 74.12, 68.63, 63.50, 58.73, 54.27, 50.14, &
       46.30, 42.74, 39.44, 36.40, 33.59, 31.00, 28.63, 26.45, &
       24.45, 22.62, 20.95, 19.43, 18.05, 16.79, 15.64, 14.61, &
       13.66,  12.81, 12.03, 11.33, 10.68, 10.09, 9.55, 9.06, &
       8.60, 8.17, 7.76, 7.38, 7.02, 6.66, 6.32, 5.98, 5.65, &
       5.31, 4.98, 4.64, 4.31, 3.96, 3.62, 3.27, 2.92, 2.57, &
       2.22, 1.87, 1.53, 1.20, 0.87, 0.57, 0.28, 0.1 /)
  real, parameter :: AWPSI(NZSR) = 1.0 / AWPS(1:NZSR)

  ! potassium nitrate
  real, parameter :: AWPN(NZSR) = (/ &
       (1000.00,i=1,44), 953.05, 881.09, 813.39, &
       749.78, 690.09, 634.14, 581.77, 532.83, 487.16, 444.61, &
       405.02, 368.26, 334.18, 302.64, 273.51, 246.67, 221.97, &
       199.31, 178.56, 159.60, 142.33, 126.63, 112.40, 99.54, &
       87.96, 77.55, 68.24, 59.92, 52.53, 45.98, 40.2, 35.11, &
       30.65, 26.75, 23.35, 20.40, 17.85, 15.63, 13.72, 12.06, &
       10.61, 9.35, 8.24, 7.25, 6.37, 5.56, 4.82, 4.12, 3.47, &
       2.86, 2.28, 1.74, 1.24, 0.79, 0.40, 0.1 /)
  real, parameter :: AWPNI(NZSR) = 1.0 / AWPN(1:NZSR)

! awpb= potassium bisulfate
!
  real, parameter :: AWPB(NZSR) = (/ (55.99,i=1,10), &
       53.79, 51.81, 49.99, 48.31, 46.75, 45.28, 43.91, 42.62, 41.39, 40.22, &
       39.10, 38.02, 36.99, 36.00, 35.04, 34.11, 33.21, 32.34, 31.49, 30.65, &
       29.84, 29.04, 28.27, 27.50, 26.75, 26.01, 25.29, 24.57, 23.87, 23.17, &
       22.49, 21.81, 21.15, 20.49, 19.84, 19.21, 18.58, 17.97, 17.37, 16.77, &
       16.19, 15.63, 15.08, 14.54, 14.01, 13.51, 13.01, 12.53, 12.07, 11.62, &
       11.19, 10.77, 10.36,  9.97,  9.59,  9.23,  8.87,  8.53,  8.20,  7.88, &
        7.57,  7.27,  6.97,  6.69,  6.41,  6.14,  5.88,  5.62,  5.36,  5.11, &
        4.87,  4.63,  4.39,  4.15,  3.92,  3.68,  3.45,  3.21,  2.98,  2.74, &
        2.49,  2.24,  1.98,  1.72,  1.44,  1.16,  0.87,  0.57,  0.28,  0.10 /)
  real, parameter :: AWPBI(NZSR) = 1.0 / AWPB(1:NZSR)

  ! calcium chloride
  real, parameter :: AWCC(NZSR) = (/ &
       19.90, 19.00, 18.15, 17.35, 16.6, 15.89, 15.22, 14.58, &
       13.99, 13.43, 12.90, 12.41, 11.94, 11.50, 11.09, 10.7, &
       10.34, 9.99, 9.67, 9.37, 9.09, 8.83, 8.57, 8.34, 8.12, &
        7.91, 7.71, 7.53, 7.35, 7.19, 7.03, 6.88, 6.74, 6.60, &
        6.47, 6.35, 6.23, 6.12, 6.01, 5.90, 5.80, 5.70, 5.61, &
        5.51, 5.42, 5.33, 5.24, 5.16, 5.07, 4.99, 4.91, 4.82, &
        4.74, 4.66, 4.58, 4.50, 4.42, 4.34, 4.26, 4.19, 4.11, &
        4.03, 3.95, 3.87, 3.79, 3.72, 3.64, 3.56, 3.48, 3.41, &
        3.33, 3.25, 3.17, 3.09, 3.01, 2.93, 2.85, 2.76, 2.68, &
        2.59, 2.50, 2.41, 2.32, 2.23, 2.13, 2.03, 1.93, 1.82, &
        1.71, 1.59, 1.47, 1.35, 1.22, 1.07, 0.93, 0.77, 0.61, &
        0.44, 0.25, 0.1 /)
  real, parameter :: AWCCI(NZSR) = 1.0 / AWCC(1:NZSR)

  ! calcium nitrate
  real, parameter :: AWCN(NZSR) = (/ &
       32.89, 31.46, 30.12, 28.84, 27.64, 26.51, 25.44, 24.44, &
       23.49, 22.59, 21.75, 20.96, 20.22, 19.51, 18.85, 18.23, &
       17.64, 17.09, 16.56, 16.07, 15.61, 15.17, 14.75, 14.36, &
       13.99, 13.63, 13.30, 12.98, 12.68, 12.39, 12.11, 11.84, &
       11.59, 11.35, 11.11, 10.88, 10.66, 10.45, 10.24, 10.04, &
       9.84, 9.65, 9.46, 9.28, 9.1, 8.92, 8.74, 8.57, 8.40, &
       8.23, 8.06, 7.90, 7.73, 7.57, 7.41, 7.25, 7.1,6.94, 6.79, &
       6.63, 6.48, 6.33, 6.18, 6.03, 5.89, 5.74, 5.60, 5.46, &
       5.32, 5.17, 5.04, 4.9, 4.76, 4.62, 4.49, 4.35, 4.22, &
       4.08, 3.94, 3.80, 3.66, 3.52, 3.38, 3.23, 3.08, 2.93, &
       2.77, 2.60, 2.43, 2.25, 2.07, 1.87, 1.67, 1.45, 1.22, &
       0.97, 0.72, 0.44, 0.14, 0.1 /)
  real, parameter :: AWCNI(NZSR) = 1.0 / AWCN(1:NZSR)

  ! magnesium chloride
  real, parameter :: AWMC(NZSR) = (/ &
       11.24, 10.99, 10.74, 10.5, 10.26, 10.03, 9.81, 9.59, &
       9.38, 9.18, 8.98, 8.79, 8.60, 8.42, 8.25, 8.07, 7.91, &
       7.75, 7.59, 7.44, 7.29, 7.15, 7.01, 6.88, 6.75, 6.62, &
       6.5, 6.38, 6.27, 6.16, 6.05, 5.94, 5.85, 5.75, 5.65, &
       5.56, 5.47, 5.38, 5.30, 5.22, 5.14, 5.06, 4.98, 4.91, &
       4.84, 4.77, 4.7, 4.63, 4.57, 4.5, 4.44, 4.37, 4.31, &
       4.25, 4.19, 4.13, 4.07, 4.01, 3.95, 3.89, 3.83, 3.77, &
       3.71, 3.65, 3.58, 3.52, 3.46, 3.39, 3.33, 3.26, 3.19, &
       3.12, 3.05, 2.98, 2.9, 2.82, 2.75, 2.67, 2.58, 2.49, &
       2.41, 2.32, 2.22, 2.13, 2.03, 1.92, 1.82, 1.71, 1.60, &
       1.48, 1.36, 1.24, 1.11, 0.98, 0.84, 0.70, 0.56, 0.41, &
       0.25, 0.1 /)
  real, parameter :: AWMCI(NZSR) = 1.0 / AWMC(1:NZSR)

  ! magnesium nitrate
  real, parameter :: AWMN(NZSR) = (/ &
       12.00, 11.84, 11.68, 11.52, 11.36, 11.2, 11.04, 10.88, &
       10.72, 10.56, 10.40, 10.25, 10.09, 9.93, 9.78, 9.63, &
       9.47, 9.32, 9.17, 9.02, 8.87, 8.72, 8.58, 8.43, 8.29, &
       8.15, 8.01, 7.87, 7.73, 7.59, 7.46, 7.33, 7.2, 7.07, &
       6.94, 6.82, 6.69, 6.57, 6.45, 6.33, 6.21, 6.01, 5.98, &
       5.87, 5.76, 5.65, 5.55, 5.44, 5.34, 5.24, 5.14, 5.04, &
       4.94, 4.84, 4.75, 4.66, 4.56, 4.47, 4.38, 4.29, 4.21, &
       4.12, 4.03, 3.95, 3.86, 3.78, 3.69, 3.61, 3.53, 3.45, &
       3.36, 3.28, 3.19, 3.11, 3.03, 2.94, 2.85, 2.76, 2.67, &
       2.58, 2.49, 2.39, 2.3, 2.2, 2.1, 1.99, 1.88, 1.77, 1.66, &
       1.54, 1.42, 1.29, 1.16, 1.02, 0.88, 0.73, 0.58, 0.42, &
       0.25, 0.1 /)
  real, parameter :: AWMNI(NZSR) = 1.0 / AWMN(1:NZSR)

  ! magnesium sulfate
  real, parameter :: AWMS(NZSR) = (/ &
       0.93, 2.5, 3.94, 5.25, 6.45, 7.54, 8.52, 9.40, 10.19, &
       10.89, 11.50, 12.04, 12.51, 12.90, 13.23, 13.50, 13.72, &
       13.88, 13.99, 14.07, 14.1, 14.09, 14.05, 13.98, 13.88, &
       13.75, 13.6, 13.43, 13.25, 13.05, 12.83, 12.61, 12.37, &
       12.13, 11.88, 11.63, 11.37, 11.12, 10.86, 10.60, 10.35, &
       10.09, 9.85, 9.6, 9.36, 9.13, 8.9, 8.68, 8.47, 8.26, &
       8.07, 7.87, 7.69, 7.52, 7.35, 7.19, 7.03, 6.89, 6.75, &
       6.62, 6.49, 6.37, 6.26, 6.15, 6.04, 5.94, 5.84, 5.75, &
       5.65, 5.56, 5.47, 5.38, 5.29, 5.20, 5.11, 5.01, 4.92, &
       4.82, 4.71, 4.60, 4.49, 4.36, 4.24, 4.10, 3.96, 3.81, &
       3.65, 3.48, 3.30, 3.11, 2.92, 2.71, 2.49, 2.26, 2.02, &
       1.76, 1.50, 1.22, 0.94, 0.64 /)
  real, parameter :: AWMSI(NZSR) = 1.0 / AWMS(1:NZSR)

end module isrpia
