!===============================================================================
! OLAM was originally developed at Duke University by Robert Walko, Martin Otte,
! and David Medvigy in the project group headed by Roni Avissar.  Development
! has continued by the same team working at other institutions (University of
! Miami (rwalko@rsmas.miami.edu), the Environmental Protection Agency, and
! Princeton University), with significant contributions from other people.

! Portions of this software are copied or derived from the RAMS software
! package.  The following copyright notice pertains to RAMS and its derivatives,
! including OLAM:  

   !----------------------------------------------------------------------------
   ! Copyright (C) 1991-2006  ; All Rights Reserved ; Colorado State University; 
   ! Colorado State University Research Foundation ; ATMET, LLC 

   ! This software is free software; you can redistribute it and/or modify it 
   ! under the terms of the GNU General Public License as published by the Free
   ! Software Foundation; either version 2 of the License, or (at your option)
   ! any later version. 

   ! This software is distributed in the hope that it will be useful, but
   ! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   ! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   ! for more details.
 
   ! You should have received a copy of the GNU General Public License along
   ! with this program; if not, write to the Free Software Foundation, Inc.,
   ! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA 
   ! (http://www.gnu.org/licenses/gpl.html) 
   !----------------------------------------------------------------------------

!===============================================================================

Module mem_flux_accum

  use consts_coms, only: r8

! Atmosphere grid arrays

  real(r8), allocatable, target :: rshort_accum      (:)
  real(r8), allocatable, target :: rshortup_accum    (:)
  real(r8), allocatable, target :: rlong_accum       (:)
  real(r8), allocatable, target :: rlongup_accum     (:)
  real(r8), allocatable, target :: rshort_top_accum  (:)
  real(r8), allocatable, target :: rshortup_top_accum(:)
  real(r8), allocatable, target :: rlongup_top_accum (:)

  real(r8), allocatable, target :: rshort_clr_accum      (:)
  real(r8), allocatable, target :: rshortup_clr_accum    (:)
  real(r8), allocatable, target :: rlong_clr_accum       (:)
  real(r8), allocatable, target :: rlongup_clr_accum     (:)
  real(r8), allocatable, target :: rshort_top_clr_accum  (:)
  real(r8), allocatable, target :: rshortup_top_clr_accum(:)
  real(r8), allocatable, target :: rlongup_top_clr_accum (:)

  real(r8), allocatable, target :: vc_accum   (:,:)
  real(r8), allocatable, target :: wc_accum   (:,:)
  real(r8), allocatable, target :: press_accum(:,:)
  real(r8), allocatable, target :: tair_accum (:,:)
  real(r8), allocatable, target :: sh_v_accum (:,:)

! Surface grid arrays

  real(r8), allocatable, target :: vels_accum    (:)
  real(r8), allocatable, target :: airtemp_accum (:)
  real(r8), allocatable, target :: airshv_accum  (:)
  real(r8), allocatable, target :: cantemp_accum (:)
  real(r8), allocatable, target :: canshv_accum  (:)
  real(r8), allocatable, target :: skintemp_accum(:)
  real(r8), allocatable, target :: sfluxt_accum  (:)
  real(r8), allocatable, target :: sfluxr_accum  (:)

  real(r8), allocatable, target ::  airtemp_dmin_accum (:)
  real(r8), allocatable, target ::  airtemp_dmax_accum (:)
  real(r8), allocatable, target ::  cantemp_dmin_accum (:)
  real(r8), allocatable, target ::  cantemp_dmax_accum (:)

! Land arrays

  real(r8), allocatable, target :: wxfer1_accum  (:) ! filled in leaf4_soil

  real(r8), allocatable, target ::  vegtemp_dmin_accum (:)
  real(r8), allocatable, target ::  vegtemp_dmax_accum (:)
  real(r8), allocatable, target :: soiltemp_dmin_accum (:)
  real(r8), allocatable, target :: soiltemp_dmax_accum (:)

Contains

!===============================================================================

subroutine alloc_flux_accum(mza,mva,mwa,mwsfc,mland)

  use misc_coms, only: ilwrtyp, iswrtyp
  use leaf_coms, only: isfcl
  use consts_coms, only: r8

  implicit none

  integer, intent(in) :: mza, mva, mwa, mwsfc, mland

! Allocate arrays for accumulated flux quantities
! Initialize arrays to zero

  if (iswrtyp > 0) then
     allocate (rshort_accum      (mwa)) ; rshort_accum       = 0._r8
     allocate (rshortup_accum    (mwa)) ; rshortup_accum     = 0._r8
     allocate (rshort_top_accum  (mwa)) ; rshort_top_accum   = 0._r8
     allocate (rshortup_top_accum(mwa)) ; rshortup_top_accum = 0._r8
  endif

  if (ilwrtyp > 0) then
     allocate (rlong_accum       (mwa)) ; rlong_accum        = 0._r8
     allocate (rlongup_accum     (mwa)) ; rlongup_accum      = 0._r8
     allocate (rlongup_top_accum (mwa)) ; rlongup_top_accum  = 0._r8
  endif

  if (.true.) then
     allocate (rshort_clr_accum      (mwa)) ; rshort_clr_accum       = 0._r8
     allocate (rshortup_clr_accum    (mwa)) ; rshortup_clr_accum     = 0._r8
     allocate (rlong_clr_accum       (mwa)) ; rlong_clr_accum        = 0._r8
     allocate (rlongup_clr_accum     (mwa)) ; rlongup_clr_accum      = 0._r8
     allocate (rshort_top_clr_accum  (mwa)) ; rshort_top_clr_accum   = 0._r8
     allocate (rshortup_top_clr_accum(mwa)) ; rshortup_top_clr_accum = 0._r8
     allocate (rlongup_top_clr_accum (mwa)) ; rlongup_top_clr_accum  = 0._r8
  endif

  allocate (vc_accum   (mza,mva)) ; vc_accum    = 0._r8
  allocate (wc_accum   (mza,mwa)) ; wc_accum    = 0._r8
  allocate (press_accum(mza,mwa)) ; press_accum = 0._r8
  allocate (tair_accum (mza,mwa)) ; tair_accum  = 0._r8
  allocate (sh_v_accum (mza,mwa)) ; sh_v_accum  = 0._r8

  if (isfcl > 0) then
     allocate (vels_accum    (mwsfc)) ; vels_accum     = 0._r8
     allocate (airtemp_accum (mwsfc)) ; airtemp_accum  = 0._r8
     allocate (airshv_accum  (mwsfc)) ; airshv_accum   = 0._r8
     allocate (cantemp_accum (mwsfc)) ; cantemp_accum  = 0._r8
     allocate (canshv_accum  (mwsfc)) ; canshv_accum   = 0._r8
     allocate (skintemp_accum(mwsfc)) ; skintemp_accum = 0._r8
     allocate (sfluxt_accum  (mwsfc)) ; sfluxt_accum   = 0._r8
     allocate (sfluxr_accum  (mwsfc)) ; sfluxr_accum   = 0._r8

     allocate (wxfer1_accum  (mland)) ; wxfer1_accum   = 0._r8

     allocate ( airtemp_dmin_accum (mwsfc)) ;  airtemp_dmin_accum = 0._r8
     allocate ( airtemp_dmax_accum (mwsfc)) ;  airtemp_dmax_accum = 0._r8
     allocate ( cantemp_dmin_accum (mwsfc)) ;  cantemp_dmin_accum = 0._r8
     allocate ( cantemp_dmax_accum (mwsfc)) ;  cantemp_dmax_accum = 0._r8

     allocate ( vegtemp_dmin_accum (mland)) ;  vegtemp_dmin_accum = 0._r8
     allocate ( vegtemp_dmax_accum (mland)) ;  vegtemp_dmax_accum = 0._r8
     allocate (soiltemp_dmin_accum (mland)) ; soiltemp_dmin_accum = 0._r8
     allocate (soiltemp_dmax_accum (mland)) ; soiltemp_dmax_accum = 0._r8
  endif
  
end subroutine alloc_flux_accum

!===============================================================================

subroutine filltab_flux_accum()

  use var_tables, only: increment_vtable
  implicit none

  if (allocated(      rshort_accum)) call increment_vtable(          'RSHORT_ACCUM','AW', dvar1=          rshort_accum)
  if (allocated(    rshortup_accum)) call increment_vtable(        'RSHORTUP_ACCUM','AW', dvar1=        rshortup_accum)
  if (allocated(       rlong_accum)) call increment_vtable(           'RLONG_ACCUM','AW', dvar1=           rlong_accum)
  if (allocated(     rlongup_accum)) call increment_vtable(         'RLONGUP_ACCUM','AW', dvar1=         rlongup_accum)
  if (allocated(  rshort_top_accum)) call increment_vtable(      'RSHORT_TOP_ACCUM','AW', dvar1=      rshort_top_accum)
  if (allocated(rshortup_top_accum)) call increment_vtable(    'RSHORTUP_TOP_ACCUM','AW', dvar1=    rshortup_top_accum)
  if (allocated( rlongup_top_accum)) call increment_vtable(     'RLONGUP_TOP_ACCUM','AW', dvar1=     rlongup_top_accum)
  if (allocated(      rshort_accum)) call increment_vtable(      'RSHORT_CLR_ACCUM','AW', dvar1=      rshort_clr_accum)
  if (allocated(    rshortup_accum)) call increment_vtable(    'RSHORTUP_CLR_ACCUM','AW', dvar1=    rshortup_clr_accum)
  if (allocated(       rlong_accum)) call increment_vtable(       'RLONG_CLR_ACCUM','AW', dvar1=       rlong_clr_accum)
  if (allocated(     rlongup_accum)) call increment_vtable(     'RLONGUP_CLR_ACCUM','AW', dvar1=     rlongup_clr_accum)
  if (allocated(  rshort_top_accum)) call increment_vtable(  'RSHORT_TOP_CLR_ACCUM','AW', dvar1=  rshort_top_clr_accum)
  if (allocated(rshortup_top_accum)) call increment_vtable('RSHORTUP_TOP_CLR_ACCUM','AW', dvar1=rshortup_top_clr_accum)
  if (allocated( rlongup_top_accum)) call increment_vtable( 'RLONGUP_TOP_CLR_ACCUM','AW', dvar1= rlongup_top_clr_accum)
  if (allocated(          vc_accum)) call increment_vtable(              'VC_ACCUM','AV', dvar2=              vc_accum)
  if (allocated(          wc_accum)) call increment_vtable(              'WC_ACCUM','AW', dvar2=              wc_accum)
  if (allocated(       press_accum)) call increment_vtable(           'PRESS_ACCUM','AW', dvar2=           press_accum)
  if (allocated(        tair_accum)) call increment_vtable(            'TAIR_ACCUM','AW', dvar2=            tair_accum)
  if (allocated(        sh_v_accum)) call increment_vtable(            'SH_V_ACCUM','AW', dvar2=            sh_v_accum)

  if (allocated(      vels_accum)) call increment_vtable(    'VELS_ACCUM','CW', dvar1=    vels_accum)
  if (allocated(   airtemp_accum)) call increment_vtable( 'AIRTEMP_ACCUM','CW', dvar1= airtemp_accum)
  if (allocated(    airshv_accum)) call increment_vtable(  'AIRSHV_ACCUM','CW', dvar1=  airshv_accum)
  if (allocated(   cantemp_accum)) call increment_vtable( 'CANTEMP_ACCUM','CW', dvar1= cantemp_accum)
  if (allocated(    canshv_accum)) call increment_vtable(  'CANSHV_ACCUM','CW', dvar1=  canshv_accum)
  if (allocated(  skintemp_accum)) call increment_vtable('SKINTEMP_ACCUM','CW', dvar1=skintemp_accum)
  if (allocated(    sfluxt_accum)) call increment_vtable(  'SFLUXT_ACCUM','CW', dvar1=  sfluxt_accum)
  if (allocated(    sfluxr_accum)) call increment_vtable(  'SFLUXR_ACCUM','CW', dvar1=  sfluxr_accum)

  if (allocated(    wxfer1_accum)) call increment_vtable(  'WXFER1_ACCUM','LW', dvar1=  wxfer1_accum)

  if (allocated( airtemp_dmin_accum)) call increment_vtable( 'AIRTEMP_DMIN_ACCUM','CW', dvar1= airtemp_dmin_accum)
  if (allocated( airtemp_dmax_accum)) call increment_vtable( 'AIRTEMP_DMAX_ACCUM','CW', dvar1= airtemp_dmax_accum)
  if (allocated( cantemp_dmin_accum)) call increment_vtable( 'CANTEMP_DMIN_ACCUM','CW', dvar1= cantemp_dmin_accum)
  if (allocated( cantemp_dmax_accum)) call increment_vtable( 'CANTEMP_DMAX_ACCUM','CW', dvar1= cantemp_dmax_accum)

  if (allocated( vegtemp_dmin_accum)) call increment_vtable( 'VEGTEMP_DMIN_ACCUM','LW', dvar1= vegtemp_dmin_accum)
  if (allocated( vegtemp_dmax_accum)) call increment_vtable( 'VEGTEMP_DMAX_ACCUM','LW', dvar1= vegtemp_dmax_accum)
  if (allocated(soiltemp_dmin_accum)) call increment_vtable('SOILTEMP_DMIN_ACCUM','LW', dvar1=soiltemp_dmin_accum)
  if (allocated(soiltemp_dmax_accum)) call increment_vtable('SOILTEMP_DMAX_ACCUM','LW', dvar1=soiltemp_dmax_accum)

end subroutine filltab_flux_accum

!===============================================================================

subroutine flux_accum()

  use misc_coms,   only: io6, time_istp8, time_istp8p, dtlm, dtsm, time_prevhist, &
                         ilwrtyp, iswrtyp, isubdomain

  use mem_ijtabs,  only: istp, itab_v, itab_w, itabg_w, jtab_v, jtab_w, &
                         jtv_prog, jtw_prog, mrl_begl, mrl_begs, mrl_endl, mrls

  use mem_basic,   only: vc, wc, press, tair, sh_v, vxe, vye, vze

  use mem_radiate, only: albedt, rshort, rlong, rlongup, &
                         rshort_top, rshortup_top, rlongup_top, &
                         rshort_clr, rshortup_clr, rshort_top_clr, &
                         rshortup_top_clr, rlong_clr, rlongup_clr, &
                         rlongup_top_clr

  use mem_average_vars, only:  airtempk_dmin,  airtempk_dmax, &
                               cantempk_dmin,  cantempk_dmax, &
                               vegtempk_dmin,  vegtempk_dmax, &
                              soiltempk_dmin, soiltempk_dmax

  use leaf_coms,   only: mrl_leaf, dt_leaf, isfcl

  use consts_coms, only: r8

  use mem_sfcg,    only: sfcg, mwsfc, itab_wsfc
  use mem_land,    only: land, mland, omland, nzg
  use mem_lake,    only: lake, mlake, omlake
  use mem_sea,     only: sea, msea, omsea
  use mem_grid,    only: mza, lpv, lpw
  use therm_lib,   only: qtk, qwtk
  use mem_para,    only: myrank

  implicit none

  integer :: mrl, j, iv, iw, iwsfc, iland, ilake, isea, k, nls

  real :: soiltempk, swtempk, fracliq, fldval

  real(r8) :: dta

! Update accumulations of ATM velocity and pressure

!----------------------------------------------------------------------
  mrl = mrl_begs(istp)
  if (mrl > 0) then

     !$omp parallel do private(iv,dta,k)
     do j = 1,jtab_v(jtv_prog)%jend(mrl); iv = jtab_v(jtv_prog)%iv(j)
!----------------------------------------------------------------------

! Timestep for accumulating velocity and pressure, DTA, is the small
! (acoustic) timestep. The frequency that each IV cell is processed in this
! loop should be consistent with (inversely proportional to) that DTA.

        dta = dtsm(itab_v(iv)%mrlv)

        do k = lpv(iv),mza
           vc_accum(k,iv) = vc_accum(k,iv) + dta * real(vc(k,iv),r8)
        enddo

     enddo
     !$omp end parallel do

  endif

!----------------------------------------------------------------------
  mrl = mrl_begs(istp)
  if (mrl > 0) then

     !$omp parallel do private(iw,dta,k)
     do j = 1,jtab_w(jtw_prog)%jend(mrl); iw = jtab_w(jtw_prog)%iw(j)
!----------------------------------------------------------------------

! Timestep for accumulating velocity and pressure, DTA, is the small
! (acoustic) timestep. The frequency that each IW cell is processed in this
! loop should be consistent with (inversely proportional to) that DTA.

        dta = dtsm(itab_w(iw)%mrlw)

        do k = lpw(iw),mza
              wc_accum(k,iw) =    wc_accum(k,iw) + dta * real(wc(k,iw),r8)
           press_accum(k,iw) = press_accum(k,iw) + dta *   press(k,iw)
        enddo

     enddo
     !$omp end parallel do

  endif

! Update accumulations of ATM temperature and specific humidity

!----------------------------------------------------------------------
  mrl = mrl_endl(istp)
  if (mrl > 0) then

     !$omp parallel do private(iw,dta,k)
     do j = 1,jtab_w(jtw_prog)%jend(mrl); iw = jtab_w(jtw_prog)%iw(j)
!----------------------------------------------------------------------

! Timestep for accumulating velocity and pressure, DTA, is the long
! timestep. The frequency that each IW cell is processed in this
! loop should be consistent with (inversely proportional to) that DTA.

        dta = dtlm(itab_w(iw)%mrlw)

        do k = lpw(iw),mza
           tair_accum(k,iw) = tair_accum(k,iw) + dta * real(tair(k,iw),r8)
           sh_v_accum(k,iw) = sh_v_accum(k,iw) + dta * real(sh_v(k,iw),r8)
        enddo

     enddo
     !$omp end parallel do

  endif

! Update accumulations of ATM SFC and TOP radiative fluxes

!----------------------------------------------------------------------
  mrl = mrl_begl(istp)
  if (mrl > 0 .and. ilwrtyp + iswrtyp > 0) then

     !$omp parallel do private(iw,dta)
     do j = 1,jtab_w(jtw_prog)%jend(mrl); iw = jtab_w(jtw_prog)%iw(j)
!----------------------------------------------------------------------

! Timestep for accumulating fluxes, DTA is dtlm for the given IW cell; 
! the frequency that each IW cell is processed in this loop should be 
! consistent with (inversely proportional to) that DTA.

        dta = dtlm(itab_w(iw)%mrlw)

        if (iswrtyp > 0) then
           rshort_accum      (iw) = rshort_accum      (iw) + dta * real(rshort(iw),r8)
           rshortup_accum    (iw) = rshortup_accum    (iw) + dta * real(rshort(iw) * albedt(iw),r8)
           rshort_top_accum  (iw) = rshort_top_accum  (iw) + dta * real(rshort_top(iw),r8)
           rshortup_top_accum(iw) = rshortup_top_accum(iw) + dta * real(rshortup_top(iw),r8)

           rshort_clr_accum      (iw) = rshort_clr_accum      (iw) + dta * real(rshort_clr(iw),r8)
           rshortup_clr_accum    (iw) = rshortup_clr_accum    (iw) + dta * real(rshortup_clr(iw),r8)
           rshort_top_clr_accum  (iw) = rshort_top_clr_accum  (iw) + dta * real(rshort_top_clr(iw),r8)
           rshortup_top_clr_accum(iw) = rshortup_top_clr_accum(iw) + dta * real(rshortup_top_clr(iw),r8)
        endif

        if (ilwrtyp > 0) then
           rlong_accum      (iw) = rlong_accum      (iw) + dta * real(rlong(iw),r8)
           rlongup_accum    (iw) = rlongup_accum    (iw) + dta * real(rlongup(iw),r8)
           rlongup_top_accum(iw) = rlongup_top_accum(iw) + dta * real(rlongup_top(iw),r8)

           rlong_clr_accum      (iw) = rlong_clr_accum      (iw) + dta * real(rlong_clr(iw),r8)
           rlongup_clr_accum    (iw) = rlongup_clr_accum    (iw) + dta * real(rlongup_clr(iw),r8)
           rlongup_top_clr_accum(iw) = rlongup_top_clr_accum(iw) + dta * real(rlongup_top_clr(iw),r8)
        endif

     enddo
     !$omp end parallel do
  endif

! Update accumulations of SFC grid cells

!----------------------------------------------------------------------
  mrl = mrl_begl(istp)
  if (mrl > 0 .and. isfcl > 0) then

     if (mrl <= mrl_leaf) mrl = 1  ! special mrl set for leaf

     dta  = dtlm(mrl)

     !$omp parallel do
     do iwsfc = 2,mwsfc

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

           vels_accum(iwsfc) =    vels_accum(iwsfc) + dta * real(sfcg%vels   (iwsfc),r8)
        airtemp_accum(iwsfc) = airtemp_accum(iwsfc) + dta * real(sfcg%airtemp(iwsfc),r8)
         airshv_accum(iwsfc) =  airshv_accum(iwsfc) + dta * real(sfcg%airshv (iwsfc),r8)

        cantemp_accum(iwsfc) = cantemp_accum(iwsfc) + dta * real(sfcg%cantemp(iwsfc),r8)
         canshv_accum(iwsfc) =  canshv_accum(iwsfc) + dta * real(sfcg%canshv (iwsfc),r8)
         sfluxt_accum(iwsfc) =  sfluxt_accum(iwsfc) + dta * real(sfcg%sfluxt (iwsfc),r8)
         sfluxr_accum(iwsfc) =  sfluxr_accum(iwsfc) + dta * real(sfcg%sfluxr (iwsfc),r8)

        if (airtempk_dmin(iwsfc) > sfcg%airtemp(iwsfc)) airtempk_dmin(iwsfc) = sfcg%airtemp(iwsfc)
        if (airtempk_dmax(iwsfc) < sfcg%airtemp(iwsfc)) airtempk_dmax(iwsfc) = sfcg%airtemp(iwsfc)

        if (cantempk_dmin(iwsfc) > sfcg%cantemp(iwsfc)) cantempk_dmin(iwsfc) = sfcg%cantemp(iwsfc)
        if (cantempk_dmax(iwsfc) < sfcg%cantemp(iwsfc)) cantempk_dmax(iwsfc) = sfcg%cantemp(iwsfc)

     enddo
     !$omp end parallel do 

  endif

! Update accumulations of LAND cells

!----------------------------------------------------------------------
  mrl = mrl_begl(istp)
  if (mrl > 0 .and. isfcl > 0) then

     if (mrl <= mrl_leaf) mrl = 1  ! special mrl set for leaf

     dta  = dtlm(mrl)

     !$omp parallel do private(iwsfc,nls,soiltempk,swtempk,fracliq,fldval)
     do iland = 2,mland
        iwsfc = iland + omland

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

        nls = land%nlev_sfcwater(iland)

        call qwtk(land%soil_energy(nzg,iland),        &
                  land%soil_water(nzg,iland)*1.e3,    &
                  land%specifheat_drysoil(nzg,iland), &
                  soiltempk, fracliq)
 
        if (nls > 0) then
           call qtk(land%sfcwater_energy(nls,iland),swtempk,fracliq)

           fldval = (1. - land%vf(iland)) * swtempk &
                        + land%vf(iland)  * land%veg_temp(iland)
        else
           fldval = (1. - land%vf(iland)) * soiltempk &
                        + land%vf(iland)  * land%veg_temp(iland)
        endif

        skintemp_accum(iwsfc) = skintemp_accum(iwsfc) + dta * real(fldval,r8)

        if (vegtempk_dmin(iland) > land%veg_temp(iland)) vegtempk_dmin(iland) = land%veg_temp(iland)
        if (vegtempk_dmax(iland) < land%veg_temp(iland)) vegtempk_dmax(iland) = land%veg_temp(iland)

        if (soiltempk_dmin(iland) > soiltempk) soiltempk_dmin(iland) = soiltempk
        if (soiltempk_dmax(iland) < soiltempk) soiltempk_dmax(iland) = soiltempk

     enddo
     !$omp end parallel do 

  endif

! Update accumulations of LAKE cells

!----------------------------------------------------------------------
  mrl = mrl_begl(istp)
  if (mrl > 0 .and. isfcl > 0) then

     if (mrl <= mrl_leaf) mrl = 1  ! special mrl set for leaf

     dta  = dtlm(mrl)

     !$omp parallel do private (iwsfc)
     do ilake = 2,mlake
        iwsfc = ilake + omlake

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

        skintemp_accum(iwsfc) =  skintemp_accum(iwsfc) + dta * real(lake%sfctemp(ilake),r8)

     enddo
     !$omp end parallel do 

  endif

! Update accumulations of SEA cells

!----------------------------------------------------------------------
  mrl = mrl_begl(istp)
  if (mrl > 0 .and. isfcl > 0) then

     if (mrl <= mrl_leaf) mrl = 1  ! special mrl set for leaf

     dta  = dtlm(mrl)

     !$omp parallel do private(iwsfc,nls,fldval)
     do isea = 2,msea
        iwsfc = isea + omsea

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

        nls = sea%nlev_seaice(isea)

        if (nls > 0) then
           fldval = (1.0 - sea%seaicec(isea)) * sea%seatc(isea) &
                  +        sea%seaicec(isea)  * sea%seaice_tempk(nls,isea)
        else
           fldval = sea%seatc(isea)
        endif

        skintemp_accum(iwsfc) =  skintemp_accum(iwsfc) + dta * real(fldval,r8)

     enddo
     !$omp end parallel do 

  endif

  ! Apply daily min/max accumulations

  if (mod(time_istp8p,86400.0_r8) < dtsm(mrls) .and. isfcl > 0) then

     ! SFC grid cells

     !$omp parallel do
     do iwsfc = 2,mwsfc

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

        airtemp_dmin_accum(iwsfc) = airtemp_dmin_accum(iwsfc) + real(airtempk_dmin(iwsfc),r8) * 86400._r8
        airtemp_dmax_accum(iwsfc) = airtemp_dmax_accum(iwsfc) + real(airtempk_dmax(iwsfc),r8) * 86400._r8

        cantemp_dmin_accum(iwsfc) = cantemp_dmin_accum(iwsfc) + real(cantempk_dmin(iwsfc),r8) * 86400._r8
        cantemp_dmax_accum(iwsfc) = cantemp_dmax_accum(iwsfc) + real(cantempk_dmax(iwsfc),r8) * 86400._r8

     enddo
     !$omp end parallel do

     ! LAND cells

     !$omp parallel do
     do iland = 2,mland
        iwsfc = iland + omland

        ! Skip this cell if running in parallel and cell rank is not MYRANK
        if (isubdomain == 1 .and. itab_wsfc(iwsfc)%irank /= myrank) cycle

        vegtemp_dmin_accum(iland) = vegtemp_dmin_accum(iland) + real(vegtempk_dmin(iland),r8) * 86400._r8
        vegtemp_dmax_accum(iland) = vegtemp_dmax_accum(iland) + real(vegtempk_dmax(iland),r8) * 86400._r8

        soiltemp_dmin_accum(iland) = soiltemp_dmin_accum(iland) + real(soiltempk_dmin(iland),r8) * 86400._r8
        soiltemp_dmax_accum(iland) = soiltemp_dmax_accum(iland) + real(soiltempk_dmax(iland),r8) * 86400._r8

     enddo
     !$omp end parallel do 

  endif

end subroutine flux_accum

!===============================================================================

End Module mem_flux_accum
