!===============================================================================
! OLAM was originally developed at Duke University by Robert Walko, Martin Otte,
! and David Medvigy in the project group headed by Roni Avissar.  Development
! has continued by the same team working at other institutions (University of
! Miami (rwalko@rsmas.miami.edu), the Environmental Protection Agency, and
! Princeton University), with significant contributions from other people.

! Portions of this software are copied or derived from the RAMS software
! package.  The following copyright notice pertains to RAMS and its derivatives,
! including OLAM:  

   !----------------------------------------------------------------------------
   ! Copyright (C) 1991-2006  ; All Rights Reserved ; Colorado State University; 
   ! Colorado State University Research Foundation ; ATMET, LLC 

   ! This software is free software; you can redistribute it and/or modify it 
   ! under the terms of the GNU General Public License as published by the Free
   ! Software Foundation; either version 2 of the License, or (at your option)
   ! any later version. 

   ! This software is distributed in the hope that it will be useful, but
   ! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   ! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   ! for more details.
 
   ! You should have received a copy of the GNU General Public License along
   ! with this program; if not, write to the Free Software Foundation, Inc.,
   ! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA 
   ! (http://www.gnu.org/licenses/gpl.html) 
   !----------------------------------------------------------------------------

!===============================================================================
Module leaf4_landcell

Contains

subroutine landcell(iwl, nlev_sfcwater, leaf_class, ntext_soil,             &
                    soil_water, soil_energy,                                &
                    sfcwater_mass, sfcwater_energy, sfcwater_depth,         &
                    rshort_v, rshort_g, rshort, rlong_v, rlong_s, rlong_g,  &
                    veg_height, veg_rough, veg_tai, veg_lai, veg_fracarea,  &
                    hcapveg, can_depth,                                     &
                    rhos, vels, prss, pcpg, qpcpg, dpcpg,                   &
                    sxfer_t, sxfer_r, ustar, snowfac, vf,                   &
                    surface_ssh, ground_shv, veg_water, veg_temp,           &
                    cantemp, canshv, stom_resist, veg_ndvip, veg_ndvif,     &
                    veg_ndvic, veg_albedo, rough, head0, head1,             &
                    glatw, glonw, flag_vg, timefac_ndvi                     )
                              
use leaf_coms,         only: nzg, nzs, soil_rough, snow_rough, slcpd, dt_leaf
use misc_coms,         only: io6
use leaf4_canopy,      only: canopy, vegndvi
use leaf4_sfcwater,    only: sfcwater, sfcwater_adjust, remove_runoff
use leaf4_soil,        only: soil
use leaf4_plot,        only: leaf_plot
use therm_lib,         only: qwtk, qtk

implicit none

integer, intent(in   ) :: iwl               ! current land cell index
integer, intent(inout) :: nlev_sfcwater     ! # active levels of surface water
integer, intent(in   ) :: leaf_class        ! leaf ("vegetation") class
integer, intent(in   ) :: ntext_soil  (nzg) ! soil textural class

logical, intent(in   ) :: flag_vg           ! flag for van Genuchten model
                                            ! (instead of Clapp & Hornberger)

real, intent(inout) :: soil_water     (nzg) ! soil water [vol_water/vol_tot]
real, intent(inout) :: soil_energy    (nzg) ! soil energy [J/m^3]
real, intent(inout) :: sfcwater_mass  (nzs) ! surface water mass [kg/m^2]
real, intent(inout) :: sfcwater_energy(nzs) ! surface water energy [J/kg]
real, intent(inout) :: sfcwater_depth (nzs) ! surface water depth [m]

real, intent(in   ) :: rshort_v     ! s/w net rad flux to veg [W/m^2]
real, intent(in   ) :: rshort_g     ! s/w net rad flux to soil [W/m^2]
real, intent(in   ) :: rshort       ! s/w incident sfc rad flux [W/m^2]
real, intent(in   ) :: rlong_v      ! l/w net rad flux to veg [W/m^2]
real, intent(in   ) :: rlong_s      ! l/w net rad flux to sfc water [W/m^2]
real, intent(in   ) :: rlong_g      ! l/w net rad flux to soil [W/m^2]
real, intent(in   ) :: veg_height   ! veg height [m]
real, intent(inout) :: veg_rough    ! veg roughness height [m]
real, intent(inout) :: veg_tai      ! veg total area index
real, intent(inout) :: veg_lai      ! veg leaf area index
real, intent(inout) :: veg_fracarea ! veg fractional area
real, intent(in   ) :: hcapveg      ! veg heat capacity [J/(m^2 K)]
real, intent(in   ) :: can_depth    ! canopy depth for heat and vap capacity [m]
real, intent(in   ) :: timefac_ndvi ! frac of time from past to future NDVI obs
real, intent(in   ) :: rhos         ! atmosphere air density [kg/m^3]
real, intent(in   ) :: vels         ! surface wind speed [m/s]
real, intent(in   ) :: prss         ! pressure [Pa]
real, intent(inout) :: pcpg         ! new precip amount this leaf timestep [kg/m^2]
real, intent(inout) :: qpcpg        ! new precip energy this leaf timestep [J/m^2]
real, intent(inout) :: dpcpg        ! new precip depth this leaf timestep [m]
real, intent(in   ) :: sxfer_t      ! can-to-atm heat xfer this step [kg_air K/m^2]
real, intent(in   ) :: sxfer_r      ! can-to-atm vapor xfer this step [kg_vap/m^2]
real, intent(in   ) :: ustar        ! friction velocity [m/s]
real, intent(in   ) :: snowfac      ! fractional veg burial by snowcover
real, intent(in   ) :: vf           ! fractional coverage of non-buried part of veg
real, intent(inout) :: surface_ssh  ! surface saturation spec hum [kg_vap/kg_air]
real, intent(inout) :: ground_shv   ! soil vapor spec hum [kg_vap/kg_air]
real, intent(inout) :: veg_water    ! veg sfc water content [kg/m^2]
real, intent(inout) :: veg_temp     ! veg temperature [K]
real, intent(inout) :: cantemp      ! canopy air temperature [K]
real, intent(inout) :: canshv       ! canopy air vapor spec hum [kg/kg]
real, intent(inout) :: stom_resist  ! veg stomatal resistance [s/m]
real, intent(inout) :: veg_ndvip    ! past veg ndvi (obs time)
real, intent(inout) :: veg_ndvif    ! past veg ndvi (obs time)
real, intent(inout) :: veg_ndvic    ! current veg ndvi
real, intent(inout) :: veg_albedo   ! veg albedo
real, intent(inout) :: rough        ! net land cell roughness height [m]
real, intent(inout) :: head0        ! LBC total hydraulic head [m]
real, intent(inout) :: head1        ! UBC total hydraulic head [m]
real, intent(in   ) :: glatw        ! Latitude of land cell 'center' [deg]
real, intent(in   ) :: glonw        ! Longitude of land cell 'center' [deg]

! Local arrays

real :: soil_tempk      (nzg) ! soil temperature [K]
real :: soil_fracliq    (nzg) ! fraction of soil moisture in liquid phase
real :: sfcwater_tempk  (nzs) ! surface water temperature [K]
real :: sfcwater_fracliq(nzs) ! fraction of sfc water in liquid phase
real :: energy_per_m2   (nzs) ! sfcwater energy [J/m^2]

real :: psi         (nzg)   ! soil water potential [m]
real :: head        (nzg)   ! total hydraulic head (including pressure) [m]

! Local variables

!integer :: linit, lframe
integer :: k     ! vertical index over soil layers
integer :: nlsw1 ! maximum of (1,nlev_sfcwater)
integer :: icomb ! implicit heat balance flag [0=no, 1=yes]

integer :: ktrans ! vertical index of soil layer supplying transpiration

real :: transp  ! transpiration xfer this LEAF timestep [kg/m^2]
real :: wfree1  ! sfcwater bottom layer free water mass [kg/m^2]
real :: qwfree1 ! sfcwater bottom layer free water energy [J/m^2]
real :: dwfree1 ! sfcwater bottom layer free water depth [m]

integer, parameter :: iwl_print = 0
real, parameter :: snowmin_expl = 10. ! min sfcwater mass for explicit heat
                                      ! xfer [kg/m^2]

real, external :: rhovsil ! function to compute sat spec hum over liquid or ice

!=================================================================
! TEMPORARY RUNOFF VARIABLES
real :: runoff
real :: qrunoff
!=================================================================

! Initialize variables for this land cell

icomb = 1  ! Default initialization
nlsw1 = max(nlev_sfcwater,1)

! Diagnose soil temperature and liquid fraction

do k = 1,nzg
   call qwtk(soil_energy(k),soil_water(k)*1.e3, &
      slcpd(ntext_soil(k)),soil_tempk(k),soil_fracliq(k))
enddo

! Loop over all possible sfcwater (a.k.a. snowcover) layers

do k = 1,nzs

! Check (temporary) for consistency between nlev_sfcwater and 
! sfcwater_mass.  If inconsistency found, print message and stop.

   if (sfcwater_mass(k) > 1.e-12 .and. k > nlev_sfcwater) then
      write(io6,*) 'sfcwater mass is present in layer above nlev_sfcwater'
      write(io6,*) 'iwl,k,glatw,glonw = ',iwl,k,glatw,glonw
      write(io6,*) 'nlev_sfcwater,sfcwater_mass(k) = ',nlev_sfcwater,sfcwater_mass(k)
      stop 'stop1 leaf4_landcell'
   endif

   if (sfcwater_mass(k) < 1.e-6 .and. k <= nlev_sfcwater) then
      write(io6,*) 'sfcwater mass is absent in layer at or below nlev_sfcwater'
      write(io6,*) 'iwl,k,glatw,glonw =',iwl,k,glatw,glonw
      write(io6,*) 'nlev_sfcwater,sfcwater_mass(k) = ',nlev_sfcwater,sfcwater_mass(k)
      stop 'stop2 leaf4_landcell'
   endif

   if (k <= nlev_sfcwater) then

! Diagnose sfcwater temperature, liquid fraction, and energy per m^2

      call qtk(sfcwater_energy(k),sfcwater_tempk(k),sfcwater_fracliq(k))

      energy_per_m2(k) = sfcwater_energy(k) * sfcwater_mass(k)

   else

! Assign default values above active sfcwater layers

      sfcwater_mass  (k) = 0.
      sfcwater_energy(k) = 0.
      sfcwater_depth (k) = 0.

      sfcwater_tempk  (k) = 273.15
      sfcwater_fracliq(k) = 0.
      energy_per_m2   (k) = 0.

   endif
enddo

! Update vegetation TAI, LAI, fractional area, albedo, and roughness

call vegndvi(iwl,                        &
             leaf_class,   timefac_ndvi, &
             veg_height,   veg_ndvip,    &
             veg_ndvif,    veg_ndvic,    &
             veg_tai,      veg_lai,      &
             veg_fracarea, veg_albedo,   &
             veg_rough                   ) 

! Compute roughness length based on vegetation and snow.

rough = max(soil_rough,veg_rough) * (1. - snowfac) + snow_rough

! Evaluate turbulent exchanges of heat and moisture between vegetation and canopy air
! and also between soil or snow surface and canopy air.  Evaluate transfer of
! precipitation moisture and heat to vegetation and shed from vegetation to surface.
! Update vegetation and canopy air temperatures resulting from these 
! plus radiative fluxes.

call canopy(iwl,                   nlsw1,                   &
            icomb,                 leaf_class,              &
            ktrans,                ntext_soil,              &
            soil_water,            soil_energy,             &
            soil_tempk,            soil_fracliq,            &
            sfcwater_mass (nlsw1), sfcwater_energy (nlsw1), &
            sfcwater_depth(nlsw1), energy_per_m2   (nlsw1), &
            sfcwater_tempk(nlsw1), sfcwater_fracliq(nlsw1), &
            veg_height,            veg_rough,               &
            veg_tai,               veg_lai,                 &
            hcapveg,               can_depth,               &
            rhos,                  vels,                    &
            prss,                  ustar,                   &
            pcpg,                  qpcpg,                   &
            dpcpg,                 rshort,                  &
            rshort_v,              rshort_g,                &
            rlong_v,               rlong_s,                 &
            rlong_g,                                        &
            sxfer_t,               sxfer_r,                 &
            snowfac,               vf,                      &
            surface_ssh,           ground_shv,              &
            veg_water,             veg_temp,                &
            cantemp,               canshv,                  & 
            transp,                stom_resist,             &
            snowmin_expl,                                   &
            glatw,                 glonw,                   &
            flag_vg                                         )


! CALL SFCWATER:
!  1. Compute soil and sfcwater heat conductivities
!  2. Compute surface heat flux for top layer of soil or sfcwater
!  3. Evaluate internal and bottom fluxes for sfcwater
!  4. Update sfcwater layer energies due to heat flux and solar radiation
!  5. Evaluate melting and percolation of liquid through sfcwater layers

call sfcwater(iwl,              icomb,           &
              nlev_sfcwater,    ntext_soil,      &
              soil_water,       soil_energy,     &
              soil_tempk,       soil_fracliq,    &
              sfcwater_mass,    sfcwater_energy, &
              sfcwater_depth,   sfcwater_tempk,  &
              sfcwater_fracliq, energy_per_m2,   &
              head1,            vf,              &
              wfree1,           qwfree1,         &
              dwfree1,          snowmin_expl,    &
              flag_vg                            )


call soil(iwl,               nlev_sfcwater,      &
          ktrans,            ntext_soil,         &
          soil_water,        soil_energy,        &
          soil_tempk,        soil_fracliq,       &
          sfcwater_mass(1),  sfcwater_energy(1), &
          sfcwater_depth(1), energy_per_m2(1),   &
          psi,               head,               &
          head0,             head1,              &
          wfree1,            qwfree1,            &
          transp,                                &
          glatw,             glonw,              &
          flag_vg                                )

! Inventory sfcwater layer(s) and adjust thicknesses if more than one

if (nzs > 1) then
   call sfcwater_adjust(iwl,                                &
                        nlev_sfcwater,    sfcwater_mass,    &
                        sfcwater_energy,  sfcwater_depth,   &
                        energy_per_m2,                      &
                        glatw,            glonw             )
endif

! Compute surface and ground vap mxrat for next timestep; put into surface_ssh 
! and ground_ssv.

nlsw1 = max(nlev_sfcwater,1)

call grndvap(iwl,              nlev_sfcwater,          &
             ntext_soil (nzg), soil_water     (nzg),   &
             soil_energy(nzg), sfcwater_energy(nlsw1), &
             rhos,             canshv,                 &
             surface_ssh,      ground_shv,             &
             flag_vg                                   )
             
!-----------------------------------------------------------------------------
! TEMPORARY UNTIL FULL LEAF-HYDRO MODEL IS COMPLETED WITH STREAM/RIVER RUNOFF:
! Simple representation of runoff

!s call remove_runoff(iwl,            nlev_sfcwater,    &
!s                    sfcwater_fracliq, sfcwater_mass,    &
!s                    sfcwater_tempk,   sfcwater_energy,  &
!s                    sfcwater_depth,   runoff,           &
!s                    qrunoff                             )
!-----------------------------------------------------------------------------

! Call land patch plot routine for selected iwl values.

if (iwl == iwl_print)  &
   call leaf_plot(iwl,                                 &
                  nlev_sfcwater,                       &
                  linit            = 1,                &
                  lframe           = 1,                &
                  ntext_soil       = ntext_soil,       &
                  leaf_class       = leaf_class,       &   
                  ktrans           = ktrans,           &
                  soil_water       = soil_water,       &   
                  soil_energy      = soil_energy,      &
                  soil_tempk       = soil_tempk,       &
                  soil_fracliq     = soil_fracliq,     &   
                  psi              = psi,              &   
                  sfcwater_mass    = sfcwater_mass,    & 
                  sfcwater_energy  = sfcwater_energy,  &
                  sfcwater_depth   = sfcwater_depth,   & 
                  sfcwater_tempk   = sfcwater_tempk,   &
                  sfcwater_fracliq = sfcwater_fracliq, & 
                  rshort_v         = rshort_v,         & 
                  rshort_g         = rshort_g,         &
                  rshort           = rshort,           & 
                  rlong_v          = rlong_v,          &
                  rlong_s          = rlong_s,          &
                  rlong_g          = rlong_g,          &
                  veg_height       = veg_height,       &  
                  veg_rough        = veg_rough,        &
                  veg_tai          = veg_tai,          & 
                  veg_lai          = veg_lai,          &
                  hcapveg          = hcapveg,          & 
                  can_depth        = can_depth,        &
                  rhos             = rhos,             &  
                  vels             = vels,             &
                  prss             = prss,             &
                  pcpg             = pcpg,             &
                  qpcpg            = qpcpg,            &
                  dpcpg            = dpcpg,            &
                  sxfer_t          = sxfer_t,          &
                  sxfer_r          = sxfer_r,          &
                  ustar            = ustar,            &
                  snowfac          = snowfac,          &
                  vf               = vf,               &
                  surface_ssh      = surface_ssh,      &
                  ground_shv       = ground_shv,       &
                  veg_water        = veg_water,        &
                  veg_temp         = veg_temp,         &
                  cantemp          = cantemp,          &
                  canshv           = canshv,           &
                  transp           = transp,           &
                  stom_resist      = stom_resist,      &
                  veg_fracarea     = veg_fracarea,     &
                  head0            = head0,            &
                  head1            = head1,            &
                  head             = head              )

! At this point, any precipitation has been added to vegetation and surface
! water, so zero out precipitation variables.

pcpg  = 0.
qpcpg = 0.
dpcpg = 0.

end subroutine landcell

End Module leaf4_landcell
