!===============================================================================
! OLAM was originally developed at Duke University by Robert Walko, Martin Otte,
! and David Medvigy in the project group headed by Roni Avissar.  Development
! has continued by the same team working at other institutions (University of
! Miami (rwalko@rsmas.miami.edu), the Environmental Protection Agency, and
! Princeton University), with significant contributions from other people.

! Portions of this software are copied or derived from the RAMS software
! package.  The following copyright notice pertains to RAMS and its derivatives,
! including OLAM:  

   !----------------------------------------------------------------------------
   ! Copyright (C) 1991-2006  ; All Rights Reserved ; Colorado State University; 
   ! Colorado State University Research Foundation ; ATMET, LLC 

   ! This software is free software; you can redistribute it and/or modify it 
   ! under the terms of the GNU General Public License as published by the Free
   ! Software Foundation; either version 2 of the License, or (at your option)
   ! any later version. 

   ! This software is distributed in the hope that it will be useful, but
   ! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   ! or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   ! for more details.
 
   ! You should have received a copy of the GNU General Public License along
   ! with this program; if not, write to the Free Software Foundation, Inc.,
   ! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA 
   ! (http://www.gnu.org/licenses/gpl.html) 
   !----------------------------------------------------------------------------

!===============================================================================
Module plotcolors

! This module defines a set of color tables for contouring.  Each table is
! a cross referencing of a set of numerical values (vals) with a set of 
! colors (ipal) selected from the palette defined in subroutine gks_colors.

! Each IPAL color in table applies to the LOWER side of its respective VALS value
! Each VALS value in table is the UPPER LIMIT of value range of its respective IPAL color

Type colortables

   integer :: nvals    ! Number of palette values in each table
   integer :: ifmt(2)  ! Format IDs for colorbar (1) & plotted value (2)
   integer :: ipal(60) ! Set of color palette indices
   real    :: vals(60) ! Set of delimiting contour values
   
End Type

type (colortables) :: clrtab(500)  ! Dimension is max number of tables

!----------------------------------------------------------------------------
! COLORTABLES 1-99: LINEAR SCALING WITH (ESSENTIALLY) NO NEGATIVE VALUES
!----------------------------------------------------------------------------

! Table 1 (leaf_class - positive integers and zero) 

data clrtab(1)%nvals/24/
data clrtab(1)%ifmt(1:2)/10,10/
data clrtab(1)%ipal(1:24)/ &
    10,101,102,141,124,107,105,110,111,113, &
   114,122,130,120,118,109,126,128,138,115, &
   143,140,145, 10                          /
data clrtab(1)%vals(1:24)/ &
    -0.9,  0.1,  1.1,  2.1,  3.1,  4.1,  5.1,  6.1,  7.1,  8.1, &
     9.1, 10.1, 11.1, 12.1, 13.1, 14.1, 15.1, 16.1, 17.1, 18.1, &
    19.1, 20.1, 21.1, 22.1                                      /

! Table 2  (seatp, seatc, seatf [K])

data clrtab(2)%nvals/41/ 
data clrtab(2)%ifmt(1:2)/0,3/
data clrtab(2)%ipal(1:41)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140                                      /
data clrtab(2)%vals(1:41)/ &
   270., 271., 272., 273., 274., 275., 276., 277., 278., 279., &
   280., 281., 282., 283., 284., 285., 286., 287., 288., 289., &
   290., 291., 292., 293., 294., 295., 296., 297., 298., 299., &
   300., 301., 302., 303., 304., 305., 306., 307., 308., 309., &
   310.                                                        /

! Table 3 (small specific humidities(g/kg))

data clrtab(3)%nvals/46/  
data clrtab(3)%ifmt(1:2)/1,6/
data clrtab(3)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(3)%vals(1:46)/ &
    .0,  .1,  .2,  .3,  .4,  .5,  .6,  .7,  .8,  .9, &
   1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, &
   2.0, 2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9, &
   3.0, 3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9, &
   4.0, 4.1, 4.2, 4.3, 4.4, 4.5                      /

! Table 4 (surface/air temperatures K) 

data clrtab(4)%nvals/36/
data clrtab(4)%ifmt(1:2)/0,3/
data clrtab(4)%ipal(1:36)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135                  /
data clrtab(4)%vals(1:36)/ &
  180.,185.,190.,195.,200.,205.,210.,215.,220.,225., &
  230.,235.,240.,245.,250.,255.,260.,265.,270.,275., &
  280.,285.,290.,295.,300.,305.,310.,315.,320.,325., &
  330.,335.,340.,345.,350.,355.                      /

! Table 5 (all specific humidities(g/kg); lpv,lpw)

data clrtab(5)%nvals/46/  
data clrtab(5)%ifmt(1:2)/0,6/
data clrtab(5)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(5)%vals(1:46)/ &
    0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9., &
   10., 11., 12., 13., 14., 15., 16., 17., 18., 19., &
   20., 21., 22., 23., 24., 25., 26., 27., 28., 29., &
   30., 31., 32., 33., 34., 35., 36., 37., 38., 39., &
   40., 41., 42., 43., 44., 45.                      /

! Table 6 (fracliq,albedt,soil_water,veg_ndvic)

data clrtab(6)%nvals/22/
data clrtab(6)%ifmt(1:2)/2,6/
data clrtab(6)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(6)%vals(1:22)/ &
   .00, .05, .10, .15, .20, .25, .30, .35, .40, .45, &
   .50, .55, .60, .65, .70, .75, .80, .85, .90, .95, &
  1.00,1.05                                          /

! Table 7 (moderate specific humidities(g/kg))

data clrtab(7)%nvals/46/  
data clrtab(7)%ifmt(1:2)/1,6/
data clrtab(7)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(7)%vals(1:46)/ &
    .0,  .2,  .4,  .6,  .8, 1.0, 1.2, 1.4, 1.6, 1.8, &
   2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2, 3.4, 3.6, 3.8, &
   4.0, 4.2, 4.4, 4.6, 4.8, 5.0, 5.2, 5.4, 5.6, 5.8, &
   6.0, 6.2, 6.4, 6.6, 6.8, 7.0, 7.2, 7.4, 7.6, 7.8, &
   8.0, 8.2, 8.4, 8.6, 8.8, 9.0                      /

! Table 8 (designed for PRESS in hPa)

data clrtab(8)%nvals/46/
data clrtab(8)%ifmt(1:2)/0,2/
data clrtab(8)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(8)%vals(1:46)/ &
     200., 220., 240., 260., 280., 300., 320., 340., 360., 380., &
     400., 420., 440., 460., 480., 500., 520., 540., 560., 580., &
     600., 620., 640., 660., 680., 700., 720., 740., 760., 780., &
     800., 820., 840., 860., 880., 900., 920., 940., 960., 980., &
    1000.,1020.,1040.,1060.,1080.,1100.                          /

! Table 9 (leaf_class, ntext_soil, nlev_sfcwater) 

data clrtab(9)%nvals/23/
data clrtab(9)%ifmt(1:2)/10,10/
data clrtab(9)%ipal(1:23)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122                              /
data clrtab(9)%vals(1:23)/ &
    0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9., &
   10., 11., 12., 13., 14., 15., 16., 17., 18., 19., &
   20., 21., 22.                                     /

! Table 10 (Uniform THIL, THETA contouring to middle strat.)

data clrtab(10)%nvals/46/ 
data clrtab(10)%ifmt(1:2)/0,3/
data clrtab(10)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(10)%vals(1:46)/ &
   220., 230., 240., 250., 260., 270., 280., 290., 300., 310., &
   320., 330., 340., 350., 360., 370., 380., 390., 400., 410., &
   420., 430., 440., 450., 460., 470., 480., 490., 500., 510., &
   520., 530., 540., 550., 560., 570., 580., 590., 600., 610., &
   620., 630., 640., 650., 660., 670.                          /

! Table 11 (Uniform THIL, THETA contouring to upper strat.)

data clrtab(11)%nvals/46/ 
data clrtab(11)%ifmt(1:2)/0,3/
data clrtab(11)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(11)%vals(1:46)/ &
   220., 240., 260., 280., 300., 320., 340., 360., 380., 400., &
   420., 440., 460., 480., 500., 520., 540., 560., 580., 600., &
   620., 640., 660., 680., 700., 720., 740., 760., 780., 800., &
   820., 840., 860., 880., 900., 920., 940., 960., 980.,1000., &
  1020.,1040.,1060.,1080.,1100.,1120.                          /

! Table 12 (specialized theta)

data clrtab(12)%nvals/22/
data clrtab(12)%ifmt(1:2)/1,4/
data clrtab(12)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(12)%vals(1:22)/ &
   300.0,300.5,301.0,301.5,302.0,302.5,303.0,303.5,304.0,304.5, &
   305.0,305.5,306.0,306.5,307.0,307.5,308.0,308.5,309.0,309.5, &
   310.0,310.5                                                  /

! Table 13 (specialized press)

data clrtab(13)%nvals/22/
data clrtab(13)%ifmt(1:2)/1,3/
data clrtab(13)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(13)%vals(1:22)/ &
    941.6, 942.0, 942.4, 942.8, 943.2, 943.6, 944.0, 944.4, 944.8, 945.2, &
    945.6, 946.0, 946.4, 946.8, 947.2, 947.6, 948.0, 948.4, 948.8, 949.2, &
    949.6, 950.0                                                          /

! Table 14 (specialized press)

data clrtab(14)%nvals/22/
data clrtab(14)%ifmt(1:2)/0,3/
data clrtab(14)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(14)%vals(1:22)/ &
    960., 962., 964., 966., 968., 970., 972., 974., 976., 978., &
    980., 982., 984., 986., 988., 990., 992., 994., 996., 998., &
   1000.,1002.                                                  /

! Table 15 (specialized rho)

data clrtab(15)%nvals/22/
data clrtab(15)%ifmt(1:2)/4,6/
data clrtab(15)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(15)%vals(1:22)/ &
   1.1601, 1.1602, 1.1603, 1.1604, 1.1605, 1.1606, 1.1607, 1.1608, 1.1609, 1.1610, &
   1.1611, 1.1612, 1.1613, 1.1614, 1.1615, 1.1616, 1.1617, 1.1618, 1.1619, 1.1620, &
   1.1621, 1.1622                                                                  /

! Table 16 (special for PRESS in hPa)

data clrtab(16)%nvals/38/
data clrtab(16)%ifmt(1:2)/0,2/
data clrtab(16)%ipal(1:38)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137          /
data clrtab(16)%vals(1:38)/ &
      200., 225., 250., 275., 300., 325., 350., 375., 400., 425., &
      450., 475., 500., 525., 550., 575., 600., 625., 650., 675., &
      700., 725., 750., 775., 800., 825., 850., 875., 900., 925., &
      950., 975.,1000.,1025.,1050.,1075.,1100.,1125.              /

! Table 17 (designed for PRESS in hPa - full range at surface)

data clrtab(17)%nvals/46/
data clrtab(17)%ifmt(1:2)/0,5/
data clrtab(17)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(17)%vals(1:46)/ &
    900., 904., 908., 912., 916., 920., 924., 928., 932., 936., &
    940., 944., 948., 952., 956., 960., 964., 968., 972., 976., &
    980., 984., 988., 992., 996.,1000.,1004.,1008.,1012.,1016., &
   1020.,1024.,1028.,1032.,1036.,1040.,1044.,1048.,1052.,1056., &
   1060.,1064.,1068.,1072.,1076.,1080.                          /

! Table 18 (designed for RHO - full range at surface)

data clrtab(18)%nvals/46/
data clrtab(18)%ifmt(1:2)/2,5/
data clrtab(18)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(18)%vals(1:46)/ &
   1.00,1.02,1.04,1.06,1.08,1.10,1.12,1.14,1.16,1.18, &
   1.20,1.22,1.24,1.26,1.28,1.30,1.32,1.34,1.36,1.38, &
   1.40,1.42,1.44,1.46,1.48,1.50,1.52,1.54,1.56,1.58, &
   1.60,1.62,1.64,1.66,1.68,1.70,1.72,1.74,1.76,1.78, &
   1.88,1.82,1.84,1.86,1.88,1.90                      /

! Table 19 (specialized THETA for Kansas LES)

data clrtab(19)%nvals/46/ 
data clrtab(19)%ifmt(1:2)/1,3/
data clrtab(19)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(19)%vals(1:46)/ &
   280.0, 280.5, 281.0, 281.5, 282.0, 282.5, 283.0, 283.5, 284.0, 284.5, &
   285.0, 285.5, 286.0, 286.5, 287.0, 287.5, 288.0, 288.5, 289.0, 289.5, &
   290.0, 290.5, 291.0, 291.5, 292.0, 292.5, 293.0, 293.5, 294.0, 294.5, &
   295.0, 295.5, 296.0, 296.5, 297.0, 297.5, 298.0, 298.5, 299.0, 299.5, &
   300.0, 300.5, 301.0, 301.5, 302.0, 302.5                              /

! Table 20 (surface theta)

data clrtab(20)%nvals/46/  
data clrtab(20)%ifmt(1:2)/0,4/
data clrtab(20)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(20)%vals(1:46)/ &
   230.,232.,234.,236.,238.,240.,242.,244.,246.,248., &
   250.,252.,254.,256.,258.,260.,262.,264.,266.,268., &
   270.,272.,274.,276.,278.,280.,282.,284.,286.,288., &
   290.,292.,294.,296.,298.,300.,302.,304.,306.,308., &
   310.,312.,314.,316.,318.,320.                      /

! Table 21 (AIRTEMPK for 10 meter mtn - heat bubble)

data clrtab(21)%nvals/22/
data clrtab(21)%ifmt(1:2)/0,5/
data clrtab(21)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(21)%vals(1:22)/ &
   240.0,241.0,242.0,243.0,244.0,245.0,246.0,247.0,248.0,249.0, &
   250.0,251.0,252.0,253.0,254.0,255.0,256.0,257.0,258.0,259.0, &
   260.0,261.0                                                  /
    
! Table 22 (specialized press - frances)

data clrtab(22)%nvals/15/
data clrtab(22)%ifmt(1:2)/0,3/
data clrtab(22)%ipal(1:15)/ &
   255,254,253,252,251,250,249,248,247,246, &
   245,244,243,242,241                      /
data clrtab(22)%vals(1:15)/ &
    932., 936., 940., 944., 948., 952., 956., 960., 964., 968., &
    972., 976., 980., 984., 988.                                /

! Table 23 (U for 10 meter mtn)

data clrtab(23)%nvals/22/
data clrtab(23)%ifmt(1:2)/2,5/
data clrtab(23)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
!data clrtab(23)%vals(1:22)/ &
!   19.90,19.91,19.92,19.93,19.94,19.95,19.96,19.97,19.98,19.99, &
!   20.00,20.01,20.02,20.03,20.04,20.05,20.06,20.07,20.08,20.09, &
!   20.10,20.11                                                  /
data clrtab(23)%vals(1:22)/ &
   17.02,17.05,17.08,17.11,17.14,17.17,17.20,17.23,17.26,17.29, &
   17.32,17.35,17.38,17.41,17.44,17.47,17.50,17.53,17.56,17.59, &
   17.62,17.65                                                  /
    
! Table 24 (AIRTEMPK for 10 meter mtn)

data clrtab(24)%nvals/22/
data clrtab(24)%ifmt(1:2)/2,5/
data clrtab(24)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(24)%vals(1:22)/ &
   250.10,250.11,250.12,250.13,250.14,250.15,250.16,250.17,250.18,250.19, &
   250.20,250.21,250.22,250.23,250.24,250.25,250.26,250.27,250.28,250.29, &
   251.30,251.31                                                          /
    
! Table 25 (specialized RHO)

data clrtab(25)%nvals/46/
data clrtab(25)%ifmt(1:2)/4,5/
data clrtab(25)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(25)%vals(1:46)/ &
   1.3150,1.3151,1.3152,1.3153,1.3154,1.3155,1.3156,1.3157,1.3158,1.3159, &
   1.3160,1.3161,1.3162,1.3163,1.3164,1.3165,1.3166,1.3167,1.3168,1.3169, &
   1.3170,1.3171,1.3172,1.3173,1.3174,1.3175,1.3176,1.3177,1.3178,1.3179, &
   1.3180,1.3181,1.3182,1.3183,1.3184,1.3185,1.3186,1.3187,1.3188,1.3189, &
   1.3190,1.3191,1.3192,1.3193,1.3194,1.3195                              /

! Table 26 (850 mb height)

data clrtab(26)%nvals/46/  
data clrtab(26)%ifmt(1:2)/0,3/
data clrtab(26)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(26)%vals(1:46)/ &
    300., 330., 360., 390., 440., 450., 480., 510., 540., 570., &
    600., 630., 660., 690., 720., 750., 780., 810., 840., 870., &
    900., 930., 960., 990.,1020.,1050.,1080.,1110.,1140.,1170., &
   1200.,1230.,1260.,1290.,1320.,1350.,1380.,1410.,1440.,1470., &
   1500.,1530.,1560.,1590.,1620.,1650.                          /

! Table 27 (700 mb height)

data clrtab(27)%nvals/46/  
data clrtab(27)%ifmt(1:2)/0,3/
data clrtab(27)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(27)%vals(1:46)/ &
   2100.,2130.,2160.,2190.,2220.,2250.,2280.,2310.,2340.,2370., &
   2400.,2430.,2460.,2490.,2520.,2550.,2580.,2610.,2640.,2670., &
   2700.,2730.,2760.,2790.,2820.,2850.,2880.,2910.,2940.,2970., &
   3100.,3130.,3160.,3190.,3220.,3250.,3280.,3210.,3240.,3270., &
   3300.,3330.,3360.,3390.,3420.,3450.                          /

! Table 28 (500 mb height)

data clrtab(28)%nvals/30/  
data clrtab(28)%ifmt(1:2)/0,3/
data clrtab(28)%ipal(1:30)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129  /
data clrtab(28)%vals(1:30)/ &
   4380.,4440.,4500.,4560.,4620.,4680.,4740.,4800.,4860.,4920., &
   4980.,5040.,5100.,5160.,5220.,5280.,5340.,5400.,5460.,5520., &
   5580.,5640.,5700.,5760.,5820.,5880.,5940.,6000.,6060.,6120.  /

! Table 29 (300 mb height)

data clrtab(29)%nvals/46/  
data clrtab(29)%ifmt(1:2)/0,3/
data clrtab(29)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(29)%vals(1:46)/ &
    7680., 7740., 7800., 7860., 7920., 7980., 8040., 8100., 8160., 8220., &
    8280., 8340., 8400., 8460., 8520., 8580., 8640., 8700., 8760., 8820., &
    8880., 8940., 9000., 9060., 9120., 9180., 9240., 9300., 9360., 9420., &
    9480., 9540., 9600., 9660., 9720., 9780., 9840., 9900., 9960.,10020., &
   10080.,10140.,10200.,10260.,10320.,10380.                              /

! Table 30 (200 mb height)

data clrtab(30)%nvals/46/  
data clrtab(30)%ifmt(1:2)/0,3/
data clrtab(30)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(30)%vals(1:46)/ &
    9780., 9840., 9900., 9960.,10020.,10080.,10140.,10200.,10260.,10320., &
   10380.,10440.,10500.,10560.,10620.,10680.,10740.,10800.,10860.,10920., &
   10980.,11040.,11100.,11160.,11220.,11280.,11340.,11400.,11460.,11520., &
   11580.,11640.,11700.,11760.,11820.,11880.,11940.,12000.,12060.,12120., &
   12180.,12240.,12300.,12360.,12420.,12480.                              /

! Table 31 (100 mb height)

data clrtab(31)%nvals/46/  
data clrtab(31)%ifmt(1:2)/0,3/
data clrtab(31)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(31)%vals(1:46)/ &
   13680.,13740.,13800.,13860.,13920.,13980.,14040.,14100.,14160.,14220., &
   14280.,14340.,14400.,14460.,14520.,14580.,14640.,14700.,14760.,14820., &
   14880.,14940.,15000.,15060.,15120.,15180.,15240.,15300.,15360.,15420., &
   15480.,15540.,15600.,15660.,15720.,15780.,15840.,15900.,15960.,16020., &
   16080.,16140.,16200.,16260.,16320.,16380.                              /

! Table 32 (50 mb height)

data clrtab(32)%nvals/46/  
data clrtab(32)%ifmt(1:2)/0,3/
data clrtab(32)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(32)%vals(1:46)/ &
   17880.,17940.,18000.,18060.,18120.,18180.,18240.,18300.,18360.,18420., &
   18480.,18540.,18600.,18660.,18720.,18780.,18840.,18900.,18960.,19020., &
   19080.,19140.,19200.,19260.,19320.,19380.,19440.,19500.,19560.,19620., &
   19680.,19740.,19800.,19860.,19920.,19980.,20040.,20100.,20160.,20220., &
   20280.,20340.,20400.,20460.,20520.,20580.                              /

! Table 33 (30 mb height)

data clrtab(33)%nvals/46/  
data clrtab(33)%ifmt(1:2)/0,3/
data clrtab(33)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(33)%vals(1:46)/ &
   20880.,20940.,21000.,21060.,21120.,21180.,21240.,21300.,21360.,21420., &
   21480.,21540.,21600.,21660.,21720.,21780.,21840.,21900.,21960.,22020., &
   22080.,22140.,22200.,22260.,22320.,22380.,22440.,22500.,22560.,22620., &
   22680.,22740.,22800.,22860.,22920.,22980.,23040.,23100.,23160.,23220., &
   23280.,23340.,23400.,23460.,23520.,23580.                              /

! Table 34 (10 mb height)

data clrtab(34)%nvals/46/  
data clrtab(34)%ifmt(1:2)/0,3/
data clrtab(34)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(34)%vals(1:46)/ &
   27480.,27540.,27600.,27660.,27720.,27780.,27840.,27900.,27960.,28020., &
   28080.,28140.,28200.,28260.,28320.,28380.,28440.,28500.,28560.,28620., &
   28680.,28740.,28800.,28860.,28920.,28980.,29040.,29100.,29160.,29220., &
   29280.,29340.,29400.,29460.,29520.,29580.,29640.,29700.,29760.,29820., &
   29880.,29940.,30000.,30060.,30120.,30180.                              /

! Table 35 (designed for WM2 height field)

data clrtab(35)%nvals/32/
data clrtab(35)%ifmt(1:2)/0,5/
data clrtab(35)%ipal(1:32)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131                                  /
data clrtab(35)%vals(1:32)/ &
      0., 100., 200., 300., 400., 500., 600., 700., 800., 900., &
   1000.,1100.,1200.,1300.,1400.,1500.,1600.,1700.,1800.,1900., &
   2000.,2100.,2200.,2300.,2400.,2500.,2600.,2700.,2800.,2900., &
   3000.,3100.                                                  /

! Table 36 (500 mb height)

data clrtab(36)%nvals/30/  
data clrtab(36)%ifmt(1:2)/0,3/
data clrtab(36)%ipal(1:30)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129  /
data clrtab(36)%vals(1:30)/ &
   4650.,4700.,4750.,4800.,4850.,4900.,4950.,5000.,5050.,5100., &
   5150.,5200.,5250.,5300.,5350.,5400.,5450.,5500.,5550.,5600., &
   5650.,5700.,5750.,5800.,5850.,5900.,5950.,6000.,6050.,6100.  /

! Table 37 (wm6 height)

data clrtab(37)%nvals/32/  
data clrtab(37)%ifmt(1:2)/0,3/
data clrtab(37)%ipal(1:32)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131                                  /
data clrtab(37)%vals(1:32)/ &
    7600., 7700., 7800., 7900., 8000., 8100., 8200., 8300., 8400., 8500., &
    8600., 8700., 8800., 8900., 9000., 9100., 9200., 9300., 9400., 9500., &
    9600., 9700., 9800., 9900.,10000.,10100.,10200.,10300.,10400.,10500., &
   10600.,10700.                                                          /

! Table 38 (wm6 height)

data clrtab(38)%nvals/17/  
data clrtab(38)%ifmt(1:2)/0,3/
data clrtab(38)%ipal(1:17)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116              /
data clrtab(38)%vals(1:17)/ &
    7600., 7800., 8000., 8200., 8400., 8600., 8800., 9000., 9200., 9400., &
    9600., 9800.,10000.,10200.,10400.,10600.,10800.                       /

! Table 39  (seatp, seatc, seatf [K])

data clrtab(39)%nvals/21/ 
data clrtab(39)%ifmt(1:2)/0,3/
data clrtab(39)%ipal(1:21)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120                                      /
data clrtab(39)%vals(1:21)/ &
   282., 283., 284., 285., 286., 287., 288., 289., 290., 291., &
   292., 293., 294., 295., 296., 297., 298., 299., 300., 301., &
   302.                                                        /

! Table 40 (TEMP for HS)

data clrtab(40)%nvals/30/
data clrtab(40)%ifmt(1:2)/0,5/
data clrtab(40)%ipal(1:30)/ &
   141,141,141,141,141,  7,  7,  7,  7,  7, &
     7,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7,  7,  7,  7,143,143,143,143,143  /
data clrtab(40)%vals(1:30)/ &
    180.,185.,190.,195.,200.,205.,210.,215.,220.,225., &
    230.,235.,240.,245.,250.,255.,260.,265.,270.,275., &
    280.,285.,290.,295.,300.,305.,310.,315.,320.,325.  /     

! Table 41 (specialized press)

data clrtab(41)%nvals/12/
data clrtab(41)%ifmt(1:2)/0,3/
data clrtab(41)%ipal(1:12)/ &
    29, 28, 27, 25, 24, 23, &
    21, 51, 53, 55, 57, 59  /
data clrtab(41)%vals(1:12)/ &
    940., 950.,  960., 970., 980., 990., &
   1000.,1010., 1020.,1030.,1040.,1050.  /

! Table 42 (specialized AIRTEMPK)

data clrtab(42)%nvals/22/
data clrtab(42)%ifmt(1:2)/3,5/
data clrtab(42)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(42)%vals(1:22)/ &
   280.7,280.8,280.9,281.0,281.1,281.2,281.3,281.4,281.5,281.6, &
   281.7,281.8,281.9,282.0,282.1,282.2,282.3,282.4,282.5,282.6, &
   282.7,282.8                                                  /
    
! Table 43 (specialized 700 mb height)
 
data clrtab(43)%nvals/14/ 
data clrtab(43)%ifmt(1:2)/0,3/
data clrtab(43)%ipal(1:14)/ &
    28, 27, 26, 25, 24, 23, 22, &
    21, 51, 52, 53, 54, 55, 56  /
data clrtab(43)%vals(1:14)/ &
    2300.,2400.,2500.,2600.,2700.,2800.,2900., &
    3000.,3100.,3200.,3300.,3400.,3500.,3600.  /

! Table 44 (specialized AIRTEMPK)

data clrtab(44)%nvals/14/
data clrtab(44)%ifmt(1:2)/0,5/
data clrtab(44)%ipal(1:14)/ &
   100,102,104,106,108,109,111, &
   113,115,117,118,119,120,121  /
data clrtab(44)%vals(1:14)/ &
   267.,270.,273.,276.,279.,282.,285., &
   288.,291.,294.,297.,300.,303.,306.  /
    
! Table 45 (specialized AIRTEMPK)

data clrtab(45)%nvals/26/
data clrtab(45)%ifmt(1:2)/0,5/
data clrtab(45)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(45)%vals(1:26)/ &
   249.,252.,255.,258.,261.,264.,267.,270.,273.,276., &
   279.,282.,285.,288.,291.,294.,297.,300.,303.,306., &
   309.,312.,315.,318.,321.,324e2                     /
    
! Table 46 (Airtempk near surface - cases 1-2)

data clrtab(46)%nvals/26/
data clrtab(46)%ifmt(1:2)/0,6/
data clrtab(46)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(46)%vals(1:26)/ &
   200., 205., 210., 215., 220., 225., 230., 235., 240., 245., &
   250., 255., 260., 265., 270., 275., 280., 285., 290., 295., &
   300., 305., 310., 315., 320., 325.                          /
    
! Table 47 (Surface P cases 1-2,4)

data clrtab(47)%nvals/26/
data clrtab(47)%ifmt(1:2)/0,6/
data clrtab(47)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(47)%vals(1:26)/ &
    920., 925., 930., 935., 940., 945., 950., 955., 960., 965., &
    970., 975., 980., 985., 990., 995.,1000.,1005.,1010.,1015., &
   1020.,1025.,1030.,1035.,1040.,1045.                          /
     
! Table 48 (Z850 cases 1-2)

data clrtab(48)%nvals/26/
data clrtab(48)%ifmt(1:2)/0,6/
data clrtab(48)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(48)%vals(1:26)/ &
    600., 650., 700., 750., 800., 850., 900., 950.,1000.,1050., &
   1100.,1150.,1200.,1250.,1300.,1350.,1400.,1450.,1500.,1550., &
   1600.,1650.,1700.,1750.,1800.,1850.                          /
     
! Table 49 (T850 case 4)

data clrtab(49)%nvals/26/
data clrtab(49)%ifmt(1:2)/2,6/
data clrtab(49)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(49)%vals(1:26)/ &
   281.20,281.24,281.28,281.32,281.36,281.40,281.44,281.48,281.52,281.56, &
   281.60,281.64,281.68,281.72,281.76,281.80,281.84,281.88,281.94,281.96, &
   282.00,282.04,282.08,282.12,282.16,282.20                              /
    
! Table 50 (Z500 case 4)

data clrtab(50)%nvals/26/
data clrtab(50)%ifmt(1:2)/0,6/
data clrtab(50)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(50)%vals(1:26)/ &
   5000.,5040.,5080.,5120.,5160.,5200.,5240.,5280.,5320.,5360., &
   5400.,5440.,5480.,5520.,5560.,5600.,5640.,5680.,5720.,5760., &
   5800.,5840.,5880.,5920.,5960.,6000.                          /   

! Table 51 (Surface P cases 1-2,4)

data clrtab(51)%nvals/21/
data clrtab(51)%ifmt(1:2)/0,6/
data clrtab(51)%ipal(1:21)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250                                      /
data clrtab(51)%vals(1:21)/ &
    700., 720., 740., 760., 780., 800., 820., 840., 860., 880., &
    900., 920., 940., 960., 980.,1000.,1020.,1040.,1060.,1080., &
   1100.                                                        /
     
! Table 52 (Z700 case 5)

data clrtab(52)%nvals/15/
data clrtab(52)%ifmt(1:2)/0,6/
data clrtab(52)%ipal(1:15)/ &
   235,236,237,238,239,240,241,242,243,244, &
   245,246,247,248,249   /
data clrtab(52)%vals(1:15)/ &
   2200.,2300.,2400.,2500.,2600.,2700.,2800.,2900.,3000.,3100., &
   3200.,3300.,3400.,3500.,3600.                                /

! Table 53 (T700 case 5)

data clrtab(53)%nvals/15/
data clrtab(53)%ifmt(1:2)/0,6/
data clrtab(53)%ipal(1:15)/ &
   235,236,237,238,239,240,241,242,243,244, &
   245,246,247,248,249   /
data clrtab(53)%vals(1:15)/ &
   264.,267.,270.,273.,276.,279.,282.,285.,288.,291., &
   294.,297.,300.,303.,306.                           /

! Table 54 (wind speed) 

data clrtab(54)%nvals/21/
data clrtab(54)%ifmt(1:2)/0,2/
data clrtab(54)%ipal(1:21)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120                                      /
data clrtab(54)%vals(1:21)/ &
     0., 5.,10.,15.,20.,25.,30.,35.,40.,45., &
    50.,55.,60.,65.,70.,75.,80.,85.,90.,95., &
   100.                                      /
   
! Table 55 (THIL, THETA in DEEP atm)

data clrtab(55)%nvals/46/
data clrtab(55)%ifmt(1:2)/0,3/
data clrtab(55)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(55)%vals(1:46)/ &
   1200.,1210.,1220.,1230.,1240.,1250.,1260.,1270.,1280.,1290., &
   1300.,1310.,1320.,1330.,1340.,1350.,1360.,1370.,1380.,1390., &
   1400.,1410.,1420.,1430.,1440.,1450.,1460.,1470.,1480.,1490., &
   1500.,1510.,1520.,1530.,1540.,1550.,1560.,1570.,1580.,1590., &
   1600.,1610.,1620.,1630.,1640.,1650.                          /

! Table 56 (north pole surface height WM5)

data clrtab(56)%nvals/46/  
data clrtab(56)%ifmt(1:2)/0,3/
data clrtab(56)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(56)%vals(1:46)/ &
   4950.,4955.,4960.,4965.,4970.,4975.,4980.,4985.,4990.,4995., &
   5000.,5005.,5010.,5015.,5020.,5025.,5030.,5035.,5040.,5045., &
   5050.,5055.,5060.,5065.,5070.,5075.,5080.,5085.,5090.,5095., &
   5100.,5105.,5110.,5115.,5120.,5125.,5130.,5135.,5140.,5145., &
   5150.,5155.,5160.,5165.,5170.,5175.                          /

! Table 57 (north pole surface height WM5)

data clrtab(57)%nvals/46/  
data clrtab(57)%ifmt(1:2)/0,3/
data clrtab(57)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(57)%vals(1:46)/ &
   3000.,3100.,3200.,3300.,3400.,3500.,3600.,3700.,3800.,3900., &
   4000.,4100.,4200.,4300.,4400.,4500.,4600.,4700.,4800.,4900., &
   5000.,5100.,5200.,5300.,5400.,5500.,5600.,5700.,5800.,5900., &
   6000.,6100.,6200.,6300.,6400.,6500.,6600.,6700.,6800.,6900., &
   7000.,7100.,7200.,7300.,7400.,7500.                         /

! Table 58 (THIL, THETA to lower srat.)

data clrtab(58)%nvals/46/ 
data clrtab(58)%ifmt(1:2)/0,3/
data clrtab(58)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(58)%vals(1:46)/ &
   280., 285., 290., 295., 300., 305., 310., 315., 320., 325., &
   330., 335., 340., 345., 350., 355., 360., 365., 370., 375., &
   380., 385., 390., 395., 400., 405., 410., 415., 420., 425., &
   430., 435., 440., 445., 450., 455., 460., 465., 470., 475., &
   480., 485., 490., 495., 500., 505.                          /

! Table 59 (NOAA/AOML/HRD H*Wind color palette for wind speed in m/s)
! Range: 0-140 knots = 0-72 m/s

data clrtab(59)%nvals/30/
data clrtab(59)%ifmt(1:2)/3,2/
data clrtab(59)%ipal(1:30)/ &
   201, 201, 202, 203, 204, 205, 206, 207, 208, 209, &
   210, 211, 212, 213, 214, 215, 216, 217, 218, 219, &
   220, 221, 222, 223, 224, 225, 226, 227, 228, 229  /
data clrtab(59)%vals(1:30)/ &
     0.000,  2.572,  5.144,  7.717, 10.289, 12.861, 15.433, 18.005, 20.578, 23.150, &
    25.722, 28.294, 30.866, 33.439, 36.011, 38.583, 41.155, 43.727, 46.300, 48.872, &
    51.444, 54.016, 56.588, 59.161, 61.733, 64.305, 66.877, 69.449, 72.022, 75.     /

! Table 60 (dcmip)

data clrtab(60)%nvals/46/ 
data clrtab(60)%ifmt(1:2)/2,3/
data clrtab(60)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(60)%vals(1:46)/ &
    .80,  .81,  .82,  .83,  .84,  .85,  .86,  .87,  .88,  .89, &
    .90,  .91,  .92,  .93,  .94,  .95,  .96,  .97,  .98,  .99, &
   1.00, 1.01, 1.02, 1.03, 1.04, 1.05, 1.06, 1.07, 1.08, 1.09, &
   1.10, 1.11, 1.12, 1.13, 1.14, 1.15, 1.16, 1.17, 1.18, 1.19, &
   1.20, 1.21, 1.22, 1.23, 1.24, 1.25                          /

! Table 61  (special theta plot for chapman test [K])

data clrtab(61)%nvals/41/ 
data clrtab(61)%ifmt(1:2)/0,3/
data clrtab(61)%ipal(1:41)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140                                      /
data clrtab(61)%vals(1:41)/ &
   340., 341., 342., 343., 344., 345., 346., 347., 348., 349., &
   350., 351., 352., 353., 354., 355., 356., 357., 358., 359., &
   360., 361., 362., 363., 364., 365., 366., 367., 368., 369., &
   370., 371., 372., 373., 374., 375., 376., 377., 378., 379., &
   380.                                                        /

! Table 62 (radiative fluxes W/m^2)

data clrtab(62)%nvals/46/
data clrtab(62)%ifmt(1:2)/0,2/
data clrtab(62)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(62)%vals(1:46)/ &
       0.,  30.,  60.,  90., 120., 150., 180., 210., 240., 270., &
     300., 330., 360., 390., 420., 450., 480., 510., 540., 570., &
     600., 630., 660., 690., 720., 750., 780., 810., 840., 870., &
     900., 930., 960., 990.,1020.,1050.,1080.,1110.,1140.,1170., &
    1200.,1230.,1260.,1290.,1320.,1350.                          /

! Table 63 (average radiative fluxes W/m^2)

data clrtab(63)%nvals/46/
data clrtab(63)%ifmt(1:2)/0,2/
data clrtab(63)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(63)%vals(1:46)/ &
       0.,  20.,  40.,  60.,  80., 100., 120., 140., 160., 180., &
     200., 220., 240., 260., 280., 300., 320., 340., 360., 380., &
     400., 420., 440., 460., 480., 500., 520., 540., 560., 580., &
     600., 620., 640., 660., 680., 700., 720., 740., 760., 780., &
     800., 820., 840., 860., 880., 900.                          /

! Table 64 (CERES radiative fluxes W/m^2)

data clrtab(64)%nvals/20/
data clrtab(64)%ifmt(1:2)/0,2/
data clrtab(64)%ipal(1:20)/ &
   121,122,123,135, 80,  8, 18,105,109,248, &
   250,116,119,252,253,131,144,143,142,141  /
data clrtab(64)%vals(1:20)/ &
       0.,  20.,  40.,  60.,  80., 100., 120., 140., 160., 180., &
     200., 220., 240., 260., 280., 300., 320., 340., 360., 380.  /

! Table 65 (integer values, cyclic)

data clrtab(65)%nvals/48/  
data clrtab(65)%ifmt(1:2)/20,0/
data clrtab(65)%ipal(1:48)/ &
    99, &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145,146              /
data clrtab(65)%vals(1:48)/ &
  -1.1, &
  -0.1,  0.9,  1.9,  2.9,  3.9,  4.9,  5.9,  6.9,  7.9,  8.9, &
   9.9, 10.9, 11.9, 12.9, 13.9, 14.9, 15.9, 16.9, 17.9, 18.9, &
  19.9, 20.9, 21.9, 22.9, 23.9, 24.9, 25.9, 26.9, 27.9, 28.9, &
  29.9, 30.9, 31.9, 32.9, 33.9, 34.9, 35.9, 36.9, 37.9, 38.9, &
  39.9, 40.9, 41.9, 42.9, 43.9, 44.9, 45.9                    /

! Table 66 (250 mb height)

data clrtab(66)%nvals/46/  
data clrtab(66)%ifmt(1:2)/0,3/
data clrtab(66)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(66)%vals(1:46)/ &
    8780., 8840., 8900., 8960., 9020., 9080., 9140., 9200., 9260., 9320., &
    9380., 9440., 9500., 9560., 9620., 9680., 9740., 9800., 9860., 9920., &
    9980.,10040.,10100.,10160.,10220.,10280.,10340.,10400.,10460.,10520., &
   10580.,10640.,10700.,10760.,10820.,10880.,10940.,11000.,11060.,11120., &
   11180.,11240.,11300.,11360.,11420.,11480.                              /

! Table 67 (con_c, con_ccn)

data clrtab(67)%nvals/22/  
data clrtab(67)%ifmt(1:2)/0,3/
data clrtab(67)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(67)%vals(1:22)/ &
    0.,  50.,100.,150.,200.,250.,300.,350.,400.,450., &
  500., 550.,600.,650.,700.,750.,800.,850.,900.,950., &
 1000.,1050.                                          /

! Table 68 (average radiative fluxes W/m^2)

data clrtab(68)%nvals/27/
data clrtab(68)%ifmt(1:2)/0,2/
data clrtab(68)%ipal(1:27)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126              /
data clrtab(68)%vals(1:27)/ &
       0.,  20.,  40.,  60.,  80., 100., 120., 140., 160., 180., &
     200., 220., 240., 260., 280., 300., 320., 340., 360., 380., &
     400., 420., 440., 460., 480., 500., 520.                    /

! Table 69 (surface albedo)

data clrtab(69)%nvals/46/  
data clrtab(69)%ifmt(1:2)/2,4/
data clrtab(69)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(69)%vals(1:46)/ &
   .00,.02,.04,.06,.08,.10,.12,.14,.16,.18, &
   .20,.22,.24,.26,.28,.30,.32,.34,.36,.38, &
   .40,.42,.44,.46,.48,.50,.52,.54,.56,.58, &
   .60,.62,.64,.66,.68,.70,.72,.74,.76,.78, &
   .80,.82,.84,.86,.88,.90                  /

! Table 70 (con_p, con_ifn)

data clrtab(70)%nvals/22/  
data clrtab(70)%ifmt(1:2)/0,3/
data clrtab(70)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(70)%vals(1:22)/ &
    0., 10., 20., 30., 40., 50., 60., 70., 80., 90., &
  100.,110.,120.,130.,140.,150.,160.,170.,180.,190., &
  200.,210.                                                  /

! Table 71 (soil dry bulk density (kg/m^3)

data clrtab(71)%nvals/46/
data clrtab(71)%ifmt(1:2)/0,2/
data clrtab(71)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(71)%vals(1:46)/ &
       0.,  50., 100., 150., 200., 250., 300., 350., 400., 450., &
     500., 550., 600., 650., 700., 750., 800., 850., 900., 950., &
    1000.,1050.,1100.,1150.,1200.,1250.,1300.,1350.,1400.,1450., &
    1500.,1550.,1600.,1650.,1700.,1750.,1800.,1850.,1900.,1950., &
    2000.,2050.,2100.,2150.,2200.,2250.                          /

!----------------------------------------------------------------------------
! COLORTABLES 100-199: LINEAR SCALING WITH NEGATIVE VALUES
!----------------------------------------------------------------------------

! Table 100 (surface/air temperatures C) 

data clrtab(100)%nvals/36/
data clrtab(100)%ifmt(1:2)/0,3/
data clrtab(100)%ipal(1:36)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135                  /
data clrtab(100)%vals(1:36)/ &
  -90.,-85.,-80.,-75.,-70.,-65.,-60.,-55.,-50.,-45., &
  -40.,-35.,-30.,-25.,-20.,-15.,-10., -5.,  0.,  5., &
   10., 15., 20., 25., 30., 35., 40., 45., 50., 55., &
   60., 65., 70., 75., 80., 85.                      /

! Table 101 (TOPM - uniform contouring)

data clrtab(101)%nvals/46/
data clrtab(101)%ifmt(1:2)/0,2/
data clrtab(101)%ipal(1:46)/ &
   140,139,138,100,101,102,103,104,105,106, &
   107,108,109,110,111,112,113,114,115,116, &
   117,118,119,120,121,122,123,124,125,126, &
   127,128,129,130,131,132,133,134,135,136, &
   137,142,143,144,145,146  /
data clrtab(101)%vals(1:46)/ &
    -400.,-200.,  -.1,   .1, 200., 400., 600., 800.,1000.,1200., &
    1400.,1600.,1800.,2000.,2200.,2400.,2600.,2800.,3000.,3200., &
    3400.,3600.,3800.,4000.,4200.,4400.,4600.,4800.,5000.,5200., &
    5400.,5600.,5800.,6000.,6200.,6400.,6600.,6800.,7000.,7200., &
    7400.,7600.,7800.,8000.,8200.,8400.                          /

! Table 102 (surface energy fluxes W/m^2)

data clrtab(102)%nvals/46/
data clrtab(102)%ifmt(1:2)/0,2/
data clrtab(102)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(102)%vals(1:46)/ &
    -700.,-650.,-600.,-550.,-500.,-450.,-400.,-350.,-300.,-250., &
    -200.,-150.,-100., -50.,   0.,  50., 100., 150., 200., 250., &
     300., 350., 400., 450., 500., 550., 600., 650., 700., 750., &
     800., 850., 900., 950.,1000.,1050.,1100.,1150.,1200.,1250., &
    1300.,1350.,1400.,1450.,1500.,1550.                          /

! Table 103 (glatw, glonw)

data clrtab(103)%nvals/39/ 
data clrtab(103)%ifmt(1:2)/0,2/
data clrtab(103)%ipal(1:39)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138      /
data clrtab(103)%vals(1:39)/ &
   -190.,-180.,-170.,-160.,-150.,-140.,-130.,-120.,-110.,-100., &
    -90., -80., -70., -60., -50., -40., -30., -20., -10.,   0., &
     10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90., 100., &
    110., 120., 130., 140., 150., 160., 170., 180., 190.        /

! Table 104 (azim)

data clrtab(104)%nvals/39/
data clrtab(104)%ifmt(1:2)/0,5/
data clrtab(104)%ipal(1:39)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138      /
data clrtab(104)%vals(1:39)/ &
     -10.,  0., 10., 20., 30., 40., 50., 60., 70., 80., &
      90.,100.,110.,120.,130.,140.,150.,160.,170.,180., &
     190.,200.,210.,220.,230.,240.,250.,260.,270.,280., &
     290.,300.,310.,320.,330.,340.,350.,360.,370.       /

! Table 105 (vertical vorticity)

data clrtab(105)%nvals/22/
data clrtab(105)%ifmt(1:2)/-2,5/
data clrtab(105)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(105)%vals(1:22)/ &
   -50.e-5,-45.e-5,-40.e-5,-35.e-5,-30.e-5,-25.e-5,-20.e-5,-15.e-5,-10.e-5, -5.e-5, &
     0.e-5,  5.e-5, 10.e-5, 15.e-5, 20.e-5, 25.e-5, 30.e-5, 35.e-5, 40.e-5, 45.e-5, &
    50.e-5, 55.e-5                                                                  /

! Table 106 (U,W with deep convection)
 
data clrtab(106)%nvals/22/ 
data clrtab(106)%ifmt(1:2)/0,3/
data clrtab(106)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(106)%vals(1:22)/ &
   -50.,-45.,-40.,-35.,-30.,-25.,-20.,-15.,-10.,- 5., &
     0.,  5., 10., 15., 20., 25., 30., 35., 40., 45., &
    50., 55.                                          /

! Table 107 (strong U,W)

data clrtab(107)%nvals/22/
data clrtab(107)%ifmt(1:2)/0,3/
data clrtab(107)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(107)%vals(1:22)/ &
  -100.,-90.,-80.,-70.,-60.,-50.,-40.,-30.,-20.,-10., &
     0., 10., 20., 30., 40., 50., 60., 70., 80., 90., &
   100.,110.                                          /
    
! Table 108 (weak U,W)

data clrtab(108)%nvals/22/
data clrtab(108)%ifmt(1:2)/1,6/
data clrtab(108)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(108)%vals(1:22)/ &
   -5.0, -4.5, -4.0, -3.5, -3.0, -2.5, -2.0, -1.5, -1.0, -0.5, &
    0.0,  0.5,  1.0,  1.5,  2.0,  2.5,  3.0,  3.5,  4.0,  4.5, &
    5.0,  5.5                                                  /
    
! Table 109 (small U,W)

data clrtab(109)%nvals/22/
data clrtab(109)%ifmt(1:2)/0,5/
data clrtab(109)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(109)%vals(1:22)/ &
   -20.,-18.,-16.,-14.,-12.,-10., -8., -6., -4.,- 2., &
     0.,  2.,  4.,  6.,  8., 10., 12., 14., 16., 18., &
    20., 22.                                          /

! Table 110 (very small U,W)

data clrtab(110)%nvals/22/
data clrtab(110)%ifmt(1:2)/1,5/
data clrtab(110)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(110)%vals(1:22)/ &
   -2.0,-1.8,-1.6,-1.4,-1.2,-1.0, -.8, -.6, -.4,- .2, &
     0.,  .2,  .4,  .6,  .8, 1.0, 1.2, 1.4, 1.6, 1.8, &
    2.0, 2.2                                          /
    
! Table 111 (triangle coefficients)

data clrtab(111)%nvals/22/
data clrtab(111)%ifmt(1:2)/1,5/
data clrtab(111)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(111)%vals(1:22)/ &
   -3.0,-2.7,-2.4,-2.1,-1.8,-1.5,-1.2, -.9, -.6,- .3, &
     0.,  .3,  .6,  .9, 1.2, 1.5, 1.8, 2.1, 2.4, 2.7, &
    3.0, 3.3                                          /
    
! Table 112 (designed for TEMP/THETA in C - full range at surface)

data clrtab(112)%nvals/46/
data clrtab(112)%ifmt(1:2)/0,2/
data clrtab(112)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(112)%vals(1:46)/ &
   -75.,-72.,-69.,-66.,-63.,-60.,-57.,-54.,-51.,-48., &
   -45.,-42.,-39.,-36.,-33.,-30.,-27.,-24.,-21.,-18., &
   -15.,-12., -9., -6., -3.,  0.,  3.,  6.,  9., 12., &
    15., 18., 21., 24., 27., 30., 33., 36., 39., 42., &
    45., 48., 51., 54., 57., 60.                      /

! Table 113 (special for zonal_wind, merid_wind)

data clrtab(113)%nvals/46/  
data clrtab(113)%ifmt(1:2)/0,3/
data clrtab(113)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(113)%vals(1:46)/ &
   -45.,-43.,-41.,-39.,-37.,-35.,-33.,-31.,-29.,-27., &
   -25.,-23.,-21.,-19.,-17.,-15.,-13.,-11.,- 9.,- 7., &
   - 5.,- 3.,- 1.,  1.,  3.,  5.,  7.,  9., 11., 13., &
    15., 17., 19., 21., 23., 25., 27., 29., 31., 33., &
    35., 37., 39., 41., 43., 45.                      /

! Table 114 (designed for TEMP/THETA in C - average range at surface)

data clrtab(114)%nvals/40/
data clrtab(114)%ifmt(1:2)/0,2/
data clrtab(114)%ipal(1:40)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139  /
data clrtab(114)%vals(1:40)/ &
   -30.,-28.,-26.,-24.,-22.,-20.,-18.,-16.,-14.,-12., &
   -10., -8., -6., -4., -2.,  0.,  2.,  4.,  6.,  8., &
    10., 12., 14., 16., 18., 20., 22., 24., 26., 28., &
    30., 32., 34., 36., 38., 40., 42., 44., 46., 48.  /

! Table 115 

data clrtab(115)%nvals/22/
data clrtab(115)%ifmt(1:2)/0,4/
data clrtab(115)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(115)%vals(1:22)/ &
     -15., -12.,  -9.,  -6.,  -3.,   0.,   3.,   6.,   9.,  12., &
      15.,  18.,  21.,  24.,  27.,  30.,  33.,  36.,  39.,  42., &
      45.,  48.                                                  /

! Table 116 (W for 10 meter mtn)

data clrtab(116)%nvals/22/
data clrtab(116)%ifmt(1:2)/3,5/
data clrtab(116)%ipal(1:22)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121                                  /
data clrtab(116)%vals(1:22)/ &
   -.040,-.036,-.032,-.028,-.024,-.020, -.016, -.012, -.008,-.004, &
      0., .004, .008, .012, .016, .020,  .024,  .028,  .032, .036, &
    .040, .044                                                     /
    
! Table 117 (1000 mb height)

data clrtab(117)%nvals/46/  
data clrtab(117)%ifmt(1:2)/0,3/
data clrtab(117)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(117)%vals(1:46)/ &
   -600.,-570.,-540.,-510.,-480.,-450.,-420.,-390.,-360.,-330., &
   -300.,-270.,-240.,-210.,-180.,-150.,-120.,- 90.,- 60.,- 30., &
      0.,  30.,  60.,  90., 120., 150., 180., 210., 240., 270., &
    300., 330., 360., 390., 420., 450., 480., 510., 540., 570., &
    600., 630., 660., 690., 720., 750.                          /

! Table 118 (W for 10 meter mtn)

data clrtab(118)%nvals/22/
data clrtab(118)%ifmt(1:2)/3,5/
data clrtab(118)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(118)%vals(1:22)/ &
   -.040,-.036,-.032,-.028,-.024,-.020, -.016, -.012, -.008,-.004, &
      0., .004, .008, .012, .016, .020,  .024,  .028,  .032, .036, &
    .040, .044                                                     /
    
! Table 119 (W for 10 meter mtn)

data clrtab(119)%nvals/22/
data clrtab(119)%ifmt(1:2)/3,5/
data clrtab(119)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(119)%vals(1:22)/ &
   -.042,-.038,-.034,-.030,-.026,-.022, -.018, -.014, -.010,-.006, &
   -.002, .002, .006, .010, .014, .018,  .022,  .026,  .030, .034, &
    .038, .042                                                     /
    
! Table 120 (W for 10 meter mtn)

data clrtab(120)%nvals/22/
data clrtab(120)%ifmt(1:2)/3,5/
data clrtab(120)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(120)%vals(1:22)/ &
   -.040,-.036,-.032,-.028,-.024,-.020,-.016,-.012,-.008,-.004, &
      0., .004, .008, .012, .016, .020, .024, .028, .032, .036, &
    .040, .044                                                  /
    
! Table 121 (W for 10 meter mtn)

data clrtab(121)%nvals/22/
data clrtab(121)%ifmt(1:2)/3,5/
data clrtab(121)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(121)%vals(1:22)/ &
   -.042,-.038,-.034,-.030,-.026,-.022,-.018,-.014,-.010,-.006, &
   -.002, .002, .006, .010, .014, .018, .022, .026, .030, .034, &
    .038, .042                                                  /
    
! Table 122 (W for 20 meter mtn)

data clrtab(122)%nvals/22/
data clrtab(122)%ifmt(1:2)/2,5/
data clrtab(122)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(122)%vals(1:22)/ &
    -.10,-.09,-.08,-.07,-.06,-.05,-.04,-.03,-.02,-.01, &
      0., .01, .02, .03, .04, .05, .06, .07, .08, .09, &
     .10, .11                                          /
    
! Table 123 (W for 20 meter mtn)

data clrtab(123)%nvals/22/
data clrtab(123)%ifmt(1:2)/2,5/
data clrtab(123)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(123)%vals(1:22)/ &
    -.10,-.09,-.08,-.07,-.06,-.05,-.04,-.03,-.02,-.01, &
      0., .01, .02, .03, .04, .05, .06, .07, .08, .09, &
     .10, .11                                          /
    
! Table 124 (integer values, cyclic)

data clrtab(124)%nvals/49/  
data clrtab(124)%ifmt(1:2)/20,0/
data clrtab(124)%ipal(1:49)/ &
    98, 99,                                 &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145,146              /
data clrtab(124)%vals(1:49)/ &
   -2., -1.,                                         &
    0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9., &
   10., 11., 12., 13., 14., 15., 16., 17., 18., 19., &
   20., 21., 22., 23., 24., 25., 26., 27., 28., 29., &
   30., 31., 32., 33., 34., 35., 36., 37., 38., 39., &
   40., 41., 42., 43., 44., 45., 46.                 /

! Table 125 (Merid wind for WM2)

data clrtab(125)%nvals/22/
data clrtab(125)%ifmt(1:2)/2,5/
data clrtab(125)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(125)%vals(1:22)/ &
    -.30,-.27,-.24,-.21,-.18,-.15,-.12,-.09,-.06,-.03, &
      0., .03, .06, .09, .12, .15, .18, .21, .24, .27, &
     .30, .33                                          /
    
! Table 126 (HERROR for WM2)

data clrtab(126)%nvals/11/
data clrtab(126)%ifmt(1:2)/2,5/
data clrtab(126)%ipal(1:11)/ &
   141,29,27,25,23,21,51,53,55,57, &
    59                             /
data clrtab(126)%vals(1:11)/ &
   -.25,-.20,-.15, -.10, -.05, 00., .05, .10, .15, .20, &
    .25                                                 /
    
! Table 127 (vertical vorticity)

data clrtab(127)%nvals/22/
data clrtab(127)%ifmt(1:2)/-2,5/
data clrtab(127)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(127)%vals(1:22)/ &
   -20.e-5,-18.e-5,-16.e-5,-14.e-5,-12.e-5,-10.e-5, -8.e-5, -6.e-5, -4.e-5,- 2.e-5, &
     0.e-5,  2.e-5,  4.e-5,  6.e-5,  8.e-5, 10.e-5, 12.e-5, 14.e-5, 16.e-5, 18.e-5, &
    20.e-5, 22.e-5                                                                  /

! Table 128 (vertical vorticity)

data clrtab(128)%nvals/22/
data clrtab(128)%ifmt(1:2)/-2,5/
data clrtab(128)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(128)%vals(1:22)/ &
   -10.e-5, -9.e-5, -8.e-5, -7.e-5, -6.e-5, -5.e-5, -4.e-5, -3.e-5, -2.e-5,- 1.e-5, &
     0.e-5,  1.e-5,  2.e-5,  3.e-5,  4.e-5,  5.e-5,  6.e-5,  7.e-5,  8.e-5,  9.e-5, &
    10.e-5, 11.e-5                                                                  /

! Table 129

data clrtab(129)%nvals/22/
data clrtab(129)%ifmt(1:2)/0,5/
data clrtab(129)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(129)%vals(1:22)/ &
     -500.,-450.,-400.,-350.,-300.,-250.,-200.,-150.,-100., -50., &
        0.,  50., 100., 150., 200., 250., 300., 350., 400., 450., &
      500., 550.                                                  /

! Table 130 (integer values, mod(val/100))

data clrtab(130)%nvals/47/  
data clrtab(130)%ifmt(1:2)/30,0/
data clrtab(130)%ipal(1:47)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145,146              /
data clrtab(130)%vals(1:47)/ &
  -23.,-22.,-21.,-20.,-19.,-18.,-17.,-16.,-15.,-14., &
  -13.,-12.,-11.,-10., -9., -8., -7., -6., -5., -4., &
   -3., -2., -1.,  0.,  1.,  2.,  3.,  4.,  5.,  6., &
    7.,  8.,  9., 10., 11., 12., 13., 14., 15., 16., &
   17., 18., 19., 20., 21., 22., 23.                 /

! Table 131 (weak U,W)

data clrtab(131)%nvals/22/
data clrtab(131)%ifmt(1:2)/1,6/
data clrtab(131)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(131)%vals(1:22)/ &
   -10.0, -9.0, -8.0, -7.0, -6.0, -5.0, -4.0, -3.0, -2.0, -1.0, &
     0.0,  1.0,  2.0,  3.0,  4.0,  5.0,  6.0,  7.0,  8.0,  9.0, &
    10.0, 11.0                                                  /
    
! Table 132 (U_ZONAL for HS)

data clrtab(132)%nvals/22/
data clrtab(132)%ifmt(1:2)/2,5/
data clrtab(132)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(132)%vals(1:22)/ &
    -40.,-36.,-32.,-28.,-24.,-20.,-16.,-12., -8., -4., &
      0.,  4.,  8., 12., 16., 20., 24., 28., 32., 36., &
     40., 44.                                          /
    
! Table 133 (U_MERID for HS)

data clrtab(133)%nvals/22/
data clrtab(133)%ifmt(1:2)/1,5/
data clrtab(133)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(133)%vals(1:22)/ &
    -5.0, -4.5, -4.0, -3.5, -3.0, -2.5, -2.0, -1.5, -1.0, -0.5, &
     0.0,  0.5,  1.0,  1.5,  2.0,  2.5,  3.0,  3.5,  4.0,  4.5, &
     5.0,  5.5                                                  /
    
! Table 134 (TEMP2 for HS)

data clrtab(134)%nvals/22/
data clrtab(134)%ifmt(1:2)/2,5/
data clrtab(134)%ipal(1:22)/ &
   142,142,142,142,142,142,142,142,142,142, &
   142,  7,  7,  7,  7,  7,  7,  7,  7,  7, &
     7,  7                                  /
data clrtab(134)%vals(1:22)/ &
    -50., -45., -40., -35., -30., -25., -20., -15., -10.,  -5., &
      0.,   5.,  10.,  15.,  20.,  25.,  30.,  35.,  40.,  45., &
     50.,  55.                                                  /
    
! Table 135 (small +/-) 

data clrtab(135)%nvals/18/
data clrtab(135)%ifmt(1:2)/1,4/
data clrtab(135)%ipal(1:18)/ &
    39, 28, 26, 24, 22, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 64, 68                              /
data clrtab(135)%vals(1:18)/ &
      -.5, -.4, -.3, -.2, -.1,                          &
       0.,  .1,  .2,  .3,  .4,  .5,  .6,  .7,  .8,  .9, &
      1.0, 1.1, 1.2                                     /

! Table 136 (specialized U,W)
 
data clrtab(136)%nvals/22/ 
data clrtab(136)%ifmt(1:2)/0,3/
data clrtab(136)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(136)%vals(1:22)/ &
   -40.,-36.,-32.,-28.,-24.,-20.,-16.,-12.,- 8.,- 4., &
     0.,  4.,  8., 12., 16., 20., 24., 28., 32., 36., &
    40., 44.                                          /

! Table 137 (vertical velocity)

data clrtab(137)%nvals/22/
data clrtab(137)%ifmt(1:2)/3,5/
data clrtab(137)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(137)%vals(1:22)/ &
   -10.e-3,-9.e-3,-8.e-3,-7.e-3,-6.e-3,-5.e-3,-4.e-3,-3.e-3,-2.e-3,-1.e-3, &
     0.e-3, 1.e-3, 2.e-3, 3.e-3, 4.e-3, 5.e-3, 6.e-3, 7.e-3, 8.e-3, 9.e-3, &
    10.e-3,11.e-3                                                          /

! Table 138 (theta perturbation case6)

data clrtab(138)%nvals/26/
data clrtab(138)%ifmt(1:2)/1,6/
data clrtab(138)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(138)%vals(1:26)/ &
   -3.0, -2.5, -2.0, -1.5, -1.0, -0.5, -0.0,  0.5, 1.0,  1.5, &
    2.0,  2.5,  3.0,  3.5,  4.0,  4.5,  5.0,  5.5, 6.0,  6.5, &
    7.0,  7.5,  8.0,  8.5,  9.0,  9.5e2                       /
    
! Table 139 (PS perturbation cases 6-0-0, 6-1-0, 6-2-0)

data clrtab(139)%nvals/26/
data clrtab(139)%ifmt(1:2)/2,6/
data clrtab(139)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(139)%vals(1:26)/ &
   -4.50, -4.25, -4.00, -3.75, -3.50, -3.25, -3.00, -2.75, -2.50, -2.25, &
   -2.00, -1.75, -1.50, -1.25, -1.00, -0.75, -0.50,- 0.25,  0.00,  0.25, &
    0.50,  0.75,  1.00,  1.25,  1.50,  1.75e2                            /
    
! Table 140 (PS perturbation case 6-3-0)

data clrtab(140)%nvals/26/
data clrtab(140)%ifmt(1:2)/1,6/
data clrtab(140)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(140)%vals(1:26)/ &
   -7.5, -7.0, -6.5, -6.0, -5.5, -5.0, -4.5, -4.0, -3.5, -3.0, &
   -2.5, -2.0, -1.5, -1.0, -0.5,  0.0,  0.5,  1.0,  1.5,  2.0, &
    2.5,  3.0,  3.5,  4.0,  4.5,  5.0e2                            /
    
! Table 141 (Scalar fields)

data clrtab(141)%nvals/26/
data clrtab(141)%ifmt(1:2)/2,6/
data clrtab(141)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(141)%vals(1:26)/ &
   -0.20, -0.15, -0.10, -0.05,  0.00,  0.05,  0.10,  0.15,  0.20,  0.25, &
    0.30,  0.35,  0.40,  0.45,  0.50,  0.55,  0.60,  0.65,  0.70,  0.75, &
    0.80,  0.85,  0.90,  0.95,  1.00,  1.05e2                            /
    
! Table 142 (Wind)

data clrtab(142)%nvals/26/
data clrtab(142)%ifmt(1:2)/0,6/
data clrtab(142)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(142)%vals(1:26)/ &
   -12.,-11.,-10., -9., -8., -7., -6., -5., -4., -3., &
    -2., -1.,  0.,  1.,  2.,  3.,  4.,  5.,  6.,  7., &
     8.,  9., 10., 11., 12., 13.                      /
    
! Table 143 (Vertical vorticity cases 1-2)

data clrtab(143)%nvals/15/
data clrtab(143)%ifmt(1:2)/-2,6/
data clrtab(143)%ipal(1:15)/ &
   235,236,237,238,239,240,241,242, &
   243,244,245,246,247,248,249      /
data clrtab(143)%vals(1:15)/ &
    -20.e-5,-15.e-5,-10.e-5, -5.e-5,  0.e-5,  5.e-5, 10.e-5, 15.e-5, &
     20.e-5, 25.e-5, 30.e-5, 35.e-5, 40.e-5, 45.e-5, 50.e-5          /
     
! Table 144 (Wind)

data clrtab(144)%nvals/26/
data clrtab(144)%ifmt(1:2)/0,6/
data clrtab(144)%ipal(1:26)/ &
   230,231,232,233,234,235,236,237,238,239, &
   240,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,255                  /
data clrtab(144)%vals(1:26)/ &
   -20.,-18.,-16.,-14.,-12.,-10., -8., -6., -4., -2., &
    -0.,  2.,  4.,  6.,  8., 10., 12., 14., 16., 18., &
    20., 22., 24., 26., 28., 30.                      /
    
! Table 145 (Wind)

data clrtab(145)%nvals/19/
data clrtab(145)%ifmt(1:2)/0,6/
data clrtab(145)%ipal(1:19)/ &
   234,235,236,237,238,239,240,241,242,243, &
   244,245,246,247,248,249,250,251,252      /
data clrtab(145)%vals(1:19)/ &
   -40.,-35.,-30.,-25.,-20.,-15.,-10., -5.,  0.,  5., &
    10., 15., 20., 25., 30., 35., 40., 45., 50.       /
    
! Table 146 (vertical velocity)

data clrtab(146)%nvals/22/
data clrtab(146)%ifmt(1:2)/3,5/
data clrtab(146)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(146)%vals(1:22)/ &
   -30.e-3,-27.e-3,-24.e-3,-21.e-3,-18.e-3,-15.e-3,-12.e-3,-9.e-3,-6.e-3,-3.e-3, &
     0.e-3,  3.e-3,  6.e-3,  9.e-3, 12.e-3, 15.e-3, 18.e-3,21.e-3,24.e-3,27.e-3, &
    30.e-3, 33.e-3                                                               /

! Table 147 (very small U,W)

data clrtab(147)%nvals/22/
data clrtab(147)%ifmt(1:2)/1,5/
data clrtab(147)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(147)%vals(1:22)/ &
   -1.0,-0.9,-0.8,-0.7,-0.6,-0.5,-0.4,-0.3,-0.2,-0.1, &
    0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, &
    1.0, 1.1                                          /
    
! Table 148 (vertical vorticity perturbation)

data clrtab(148)%nvals/22/
data clrtab(148)%ifmt(1:2)/-2,-2/
data clrtab(148)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(148)%vals(1:22)/ &
   -10.e-7, -9.e-7, -8.e-7, -7.e-7, -6.e-7, -5.e-7, -4.e-7, -3.e-7, -2.e-7, -1.e-7, &
     0.e-7,  1.e-7,  2.e-7,  3.e-7,  4.e-7,  5.e-7,  6.e-7,  7.e-7,  8.e-7,  9.e-7, &
    10.e-7, 11.e-7                                                                  /

! Table 149 (designed for WM1 scalar field)

data clrtab(149)%nvals/14/
data clrtab(149)%ifmt(1:2)/2,5/
data clrtab(149)%ipal(1:14)/ &
   141,141,141,  7,  7, &
     7,  7,  7,  7,  7, &
     7,  7,  7,  7      /
data clrtab(149)%vals(1:14)/ &
    -.25,-.15,-.05, .05, .15, &
     .25, .35, .45, .55, .65, &
     .75, .85, .95,1.05       /

! Table 150 (designed for WM1 scalar error field)

data clrtab(150)%nvals/16/
data clrtab(150)%ifmt(1:2)/2,5/
data clrtab(150)%ipal(1:16)/ &
   141,141,141,141,141,141,141,141, &
     7,  7,  7,  7,  7,  7,  7,  7  /
data clrtab(150)%vals(1:16)/ &
    -.15,-.13,-.11,-.09,-.07,-.05,-.03,-.01, &
     .01, .03, .05, .07, .09, .11, .13, .15  /

! Table 151 (Merid wind for WM2)

data clrtab(151)%nvals/22/
data clrtab(151)%ifmt(1:2)/2,5/
data clrtab(151)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(151)%vals(1:22)/ &
    -.21,-.19,-.17,-.15,-.13,-.11,-.09,-.07,-.05,-.03, &
    -.01, .01, .03, .05, .07, .09, .11, .13, .15, .17, &
     .19, .21                                          /
    
! Table 152 (Press for nested WM2)

data clrtab(152)%nvals/22/
data clrtab(152)%ifmt(1:2)/1,5/
data clrtab(152)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(152)%vals(1:22)/ &
    -2.1,-1.9,-1.7,-1.5,-1.3,-1.1,-0.9,-0.7,-0.5,-0.3, &
    -0.1,  .1,  .3,  .5,  .7,  .9, 1.1, 1.3, 1.5, 1.7, &
     1.9, 2.1                                          /
    
! Table 153 ( )

data clrtab(153)%nvals/22/
data clrtab(153)%ifmt(1:2)/2,5/
data clrtab(153)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(153)%vals(1:22)/ &
   -.42,-.38,-.34,-.30,-.26,-.22, -.18, -.14, -.10,-.06, &
   -.02, .02, .06, .10, .14, .18,  .22,  .26,  .30, .34, &
    .38, .42                                             /
    
! Table 154 ( )

data clrtab(154)%nvals/22/
data clrtab(154)%ifmt(1:2)/-2,-5/
data clrtab(154)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(154)%vals(1:22)/ &
   -2.0e-5,-1.8e-5,-1.6e-5,-1.4e-5,-1.2e-5,-1.0e-5, -.8e-5, -.6e-5, -.4e-5,- .2e-5, &
     0.e-5,  .2e-5,  .4e-5,  .6e-5,  .8e-5, 1.0e-5, 1.2e-5, 1.4e-5, 1.6e-5, 1.8e-5, &
    2.0e-5, 2.2e-5                                                                  /

! Table 155 (Merid wind for WM2)

data clrtab(155)%nvals/22/
data clrtab(155)%ifmt(1:2)/3,5/
data clrtab(155)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20,  7, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(155)%vals(1:22)/ &
    -.021,-.019,-.017,-.015,-.013,-.011,-.009,-.007,-.005,-.003, &
    -.001, .001, .003, .005, .007, .009, .011, .013, .015, .017, &
     .019, .021                                                  /
    
! Table 156 (height perturbation for WM5 - figs for paper)
 
data clrtab(156)%nvals/12/ 
data clrtab(156)%ifmt(1:2)/0,3/
data clrtab(156)%ipal(1:12)/ &
     29,  28,  26,  24,  22,  20, &
     50,  52,  54,  56,  58,  59  /
data clrtab(156)%vals(1:12)/ &
   -25.,-20.,-15.,-10.,- 5.,  0., &
     5., 10., 15., 20., 25., 30.  /

! Table 157 (relative vorticity for WM5 - figs for paper)
 
data clrtab(157)%nvals/12/ 
data clrtab(157)%ifmt(1:2)/-1,3/
data clrtab(157)%ipal(1:12)/ &
     29,  28,  26,  24,  22,  20, &
     50,  52,  54,  56,  58,  59  /
data clrtab(157)%vals(1:12)/ &
    -5.e-5,-4.e-5,-3.e-5,-2.e-5,-1.e-5, 0.e-5, &
     1.e-5, 2.e-5, 3.e-5, 4.e-5, 5.e-5, 6.e-5  /

! Table 158 (vertical vorticity - hurricane)

data clrtab(158)%nvals/22/
data clrtab(158)%ifmt(1:2)/-2,5/
data clrtab(158)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(158)%vals(1:22)/ &
   -20.e-4,-18.e-4,-16.e-4,-14.e-4,-12.e-4,-10.e-4, -8.e-4, -6.e-4, -4.e-4, -2.e-4, &
     0.e-4,  2.e-4,  4.e-4,  6.e-4,  8.e-4, 10.e-4, 12.e-4, 14.e-4, 16.e-4, 18.e-4, &
    20.e-4, 22.e-4                                                                  /

! Table 159 (special for zonal_wind, merid_wind)

data clrtab(159)%nvals/46/  
data clrtab(159)%ifmt(1:2)/0,3/
data clrtab(159)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(159)%vals(1:46)/ &
   -15.,-13.,-11., -9., -7., -5., -3., -1.,  1.,  3., &
     5.,  7.,  9., 11., 13., 15., 17., 19., 21., 23., &
    25., 27., 29., 31., 33., 35., 37., 39., 41., 43., &
    45., 47., 49., 51., 53., 55., 57., 59., 61., 63., &
    65., 67., 69., 71., 73., 75.                      /

! Table 160 (tracer values - NCAR Colloquium 2012)

data clrtab(160)%nvals/26/  
data clrtab(160)%ifmt(1:2)/2,6/
data clrtab(160)%ipal(1:26)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125                  /
data clrtab(160)%vals(1:26)/ &
   -0.10, -0.05, 0.00, 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, &
    0.40,  0.45, 0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, &
    0.90,  0.95, 1.00, 1.05, 1.10, 1.15                          /

! Table 161 (PCP for wtd differences)

data clrtab(161)%nvals/20/
data clrtab(161)%ifmt(1:2)/0,5/
data clrtab(161)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(161)%vals(1:20)/ &
     -9.,-8.,-7.,-6.,-5.,-4.,-3.,-2.,-1., 0., &
      1., 2., 3., 4., 5., 6., 7., 8., 9.,10.  /
    
! Table 162 (PCP for wtd fractional differences)

data clrtab(162)%nvals/20/
data clrtab(162)%ifmt(1:2)/1,5/
data clrtab(162)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(162)%vals(1:20)/ &
     -.9,-.8,-.7,-.6,-.5,-.4,-.3,-.2,-.1, .0, &
      .1, .2, .3, .4, .5, .6, .7, .8, .9,1.0  /
    
! Table 163 (avg sfc energy fluxes)

data clrtab(163)%nvals/40/ 
data clrtab(163)%ifmt(1:2)/0,2/
data clrtab(163)%ipal(1:40)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139  /
data clrtab(163)%vals(1:40)/ &
    -90., -80., -70., -60., -50., -40., -30., -20., -10.,   0., &
     10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90., 100., &
    110., 120., 130., 140., 150., 160., 170., 180., 190., 200., &
    210., 220., 230., 240., 250., 260., 270., 280., 290., 300.  /

! Table 164 (PCP for wtd fractional differences)

data clrtab(164)%nvals/20/
data clrtab(164)%ifmt(1:2)/1,5/
data clrtab(164)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(164)%vals(1:20)/ &
    -2.7,-2.4,-2.1,-1.8,-1.5,-1.2, -.9, -.6, -.3, .0, &
      .3,  .6,  .9, 1.2, 1.5, 1.8, 2.1, 2.4, 2.7,3.0  /
    
! Table 165 (D_THIL, D_THETA for micsgs)

data clrtab(165)%nvals/22/
data clrtab(165)%ifmt(1:2)/1,6/
data clrtab(165)%ipal(1:22)/ &
    29, 28, 27, 26, 25, 24, 23, 22, 21, 20, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(165)%vals(1:22)/ &
   -10.0, -9.0, -8.0, -7.0, -6.0, -5.0, -4.0, -3.0, -2.0, -1.0, &
     1.0,  2.0,  3.0,  4.0,  5.0,  6.0,  7.0,  8.0,  9.0, 10.0, &
    11.0, 12.0                                                  /
    
! Table 166 (pcp for Chapman - comparison with WRF [mm/day])

data clrtab(166)%nvals/15/
data clrtab(166)%ifmt(1:2)/0,6/
data clrtab(166)%ipal(1:15)/ &
   241,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254                      /
data clrtab(166)%vals(1:15)/ &
    -6.,-4.,-2., 0., 2., 4., 6., 8.,10.,12., &
    14.,16.,18.,20.,22.                      /

! Table 167 (sfc energy fluxes)

data clrtab(167)%nvals/46/ 
data clrtab(167)%ifmt(1:2)/0,2/
data clrtab(167)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(167)%vals(1:46)/ &
   -200.,-180.,-160.,-140.,-120.,-100., -80., -60., -40., -20., &
      0.,  20.,  40.,  60.,  80., 100., 120., 140., 160., 180., &
    200., 220., 240., 260., 280., 300., 320., 340., 360., 380., &
    400., 420., 440., 460., 480., 500., 520., 540., 560., 580., &
    600., 620., 640., 660., 680., 700.                          /

! Table 168

data clrtab(168)%nvals/46/  
data clrtab(168)%ifmt(1:2)/1,3/
data clrtab(168)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(168)%vals(1:46)/ &
   -5.9,-5.7,-5.5,-5.3,-5.1,-4.9,-4.7,-4.5,-4.3,-4.1, &
   -3.9,-3.7,-3.5,-3.3,-3.1,-2.9,-2.7,-2.5,-2.3,-2.1, &
   -1.9,-1.7,-1.5,-1.3,-1.1,- .9,- .7,- .5,- .3,- .1, &
     .1,  .3,  .5,  .7,  .9, 1.1, 1.3, 1.5, 1.7, 1.9, &
    2.1, 2.3, 2.5, 2.7, 2.9, 3.1                      /

! Table 169 (was used in dcmip2012 - as table 161)

data clrtab(169)%nvals/22/
data clrtab(169)%ifmt(1:2)/-2,-5/
data clrtab(169)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(169)%vals(1:22)/ &
   -1.0e-5,-0.9e-5,-0.8e-5,-0.7e-5,-0.6e-5,-0.5e-5,-0.4e-5,-0.3e-5,-0.2e-5,-0.1e-5, &
     0.e-5, 0.1e-5, 0.2e-5, 0.3e-5, 0.4e-5, 0.5e-5, 0.6e-5, 0.7e-5, 0.8e-5, 0.9e-5, &
    1.0e-5, 1.1e-5                                                                  /

! Table 170 ( ) dcmip 2016

data clrtab(170)%nvals/15/
data clrtab(170)%ifmt(1:2)/1,-5/
data clrtab(170)%ipal(1:15)/ &
   241,242,243,244,245,246,247,248,249,250, &
   251,252,253,254,255                          /
data clrtab(170)%vals(1:15)/ &
    -.1, -.0e-12,  .1,  .2,  .3, .4, .5, .6, .7, .8, &
     .9,     1.0, 1.1, 1.2, 1.3                      /

! Table 171 (PCP for wtd differences)

data clrtab(171)%nvals/20/
data clrtab(171)%ifmt(1:2)/0,5/
data clrtab(171)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(171)%vals(1:20)/ &
     -20.,-16.,-14.,-12.,-10., -8., -6., -4., -2.,  0., &
       2.,  4.,  6.,  8., 10., 12., 14., 16., 18., 20.  /
    
! Table 172 (PCP for wtd differences)

data clrtab(172)%nvals/20/
data clrtab(172)%ifmt(1:2)/0,5/
data clrtab(172)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(172)%vals(1:20)/ &
     -27.,-24.,-21.,-18.,-15.,-12., -9., -6., -3.,  0., &
       3.,  6.,  9., 12., 15., 18., 21., 24., 27., 30.  /
    
! Table 173 (PCP for wtd differences)

data clrtab(173)%nvals/20/
data clrtab(173)%ifmt(1:2)/0,5/
data clrtab(173)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(173)%vals(1:20)/ &
     -45.,-40.,-35.,-30.,-25.,-20.,-15.,-10., -5.,  0., &
       5., 10., 15., 20., 25., 30., 35., 40., 45., 50.  /
    
! Table 174 (PCP for wtd differences)

data clrtab(174)%nvals/20/
data clrtab(174)%ifmt(1:2)/2,5/
data clrtab(174)%ipal(1:20)/ &
    28, 27, 26, 25, 24, 23, 22, 21, 20,  7, &
     7, 50, 51, 52, 53, 54, 55, 56, 57, 58  /
data clrtab(174)%vals(1:20)/ &
   -.45,-.40,-.35,-.30,-.25,-.20,-.15,-.10,-.05, .00, &
    .05, .10, .15, .20, .25, .30, .35, .40, .45, .50  /
    
! Table 175 ( ) dcmip 2016

data clrtab(175)%nvals/15/
data clrtab(175)%ifmt(1:2)/2,-5/
data clrtab(175)%ipal(1:15)/ &
   241,242,243,244,245,246,247,248,249,250, &
   251,252,253,254,255                      /
data clrtab(175)%vals(1:15)/ &
   -.04, -1.0e-12, .04, .08, .12, .16, .20, .24, .28, .32, &
    .36,      .40, .44, .48, .52                           /

! Table 176 ( ) dcmip 2016

data clrtab(176)%nvals/14/
data clrtab(176)%ifmt(1:2)/2,-5/
data clrtab(176)%ipal(1:14)/ &
   241,242,243,244,245,246,247,248,249,250, &
   252,253,254,255                          /
data clrtab(176)%vals(1:14)/ &
    .78,  .82,  .86,  .90, .94, .98, 1.02, 1.06, 1.10, 1.14, &
   1.18, 1.22, 1.24, 1.28                                    /

! Table 177 ( ) dcmip 2016

data clrtab(177)%nvals/13/
data clrtab(177)%ifmt(1:2)/0,-5/
data clrtab(177)%ipal(1:13)/ &
   241,242,243,244,246,247,248,249,250,251, &
   252,254,255                              /
data clrtab(177)%vals(1:13)/ &
   968., 972., 976., 980., 984., 988., 992., 996.,1000.,1004., &
  1008.,1012.,1016.                                            /

!----------------------------------------------------------------------------
! COLORTABLES 200-299: EXPONENTIAL SCALING WITH NO NEGATIVE VALUES
!----------------------------------------------------------------------------

! Table 200 (pcp rates [kg m^-2 h^-1], 
!           accum pcp & sfcwater_mass [kg m^-2],
!           stom_resist [s/m],
!           micphys concens [#/kg_tot],
!           vkm [m^2/s],
!           wind speed [m/s]

data clrtab(200)%nvals/26/
data clrtab(200)%ifmt(1:2)/-1,-5/
data clrtab(200)%ipal(1:26)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125                  /
data clrtab(200)%vals(1:26)/ &
     0., 1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1, 1.e0, 2.e0, 5.e0, &
   1.e1, 2.e1 , 5.e1 , 1.e2 , 2.e2 , 5.e2 , 1.e3 , 2.e3, 5.e3, 1.e4, &
   2.e4, 5.e4 , 1.e5 , 2.e5 , 5.e5 , 1.e6                            /

! Table 201 (roughness height [m],
!           sfcwater_depth [m],
!           sfluxv,w [N/m^2],
!           veg_water [kg/m^2] 

data clrtab(201)%nvals/20/ 
data clrtab(201)%ifmt(1:2)/-1,-3/
data clrtab(201)%ipal(1:20)/ &
   101,102,103,105,106,107,109,110,111,113, &
   114,115,116,118,119,122,123,124,126,127  /
data clrtab(201)%vals(1:20)/ &
     0., 1.e-5, 2.e-5, 5.e-5, 1.e-4, 2.e-4, 5.e-4, 1.e-3, 2.e-3, 5.e-3, &
  1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1,  1.e0, 2.e0 , 5.e0 , 1.e1   /

! Table 202 (ARV, ARW, VOLT)

data clrtab(202)%nvals/46/
data clrtab(202)%ifmt(1:2)/-1,-6/
data clrtab(202)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(202)%vals(1:46)/ &
      0.,    1.,    3.,   10.,   30.,  100.,  300.,  1.e3,  3.e3,  1.e4, &
    3.e4,  1.e5,  3.e5,  1.e6,  3.e6,  1.e7,  3.e7,  1.e8,  3.e8,  1.e9, &
    3.e9, 1.e10, 3.e10, 1.e11, 3.e11, 1.e12, 3.e12, 1.e13, 3.e13, 1.e14, &
   3.e14, 1.e15, 3.e15, 1.e16, 3.e16, 1.e17, 3.e17, 1.e18, 3.e18, 1.e19, &
   3.e19, 1.e20, 3.e20, 1.e21, 3.e21, 1.e22                              /

! Table 203 (dniu(v) [m^-1])

data clrtab(203)%nvals/26/ 
data clrtab(203)%ifmt(1:2)/-1,-4/
data clrtab(203)%ipal(1:26)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125                  /
data clrtab(203)%vals(1:26)/ &
     0., 1.e-7, 2.e-7, 5.e-7, 1.e-6, 2.e-6, 5.e-6, 1.e-5, 2.e-5, 5.e-5, &
  1.e-4, 2.e-4, 5.e-4, 1.e-3, 2.e-3, 5.e-3, 1.e-2, 2.e-2, 5.e-2, 1.e-1, &
  2.e-1, 5.e-1, 1.e0 , 2.e0 , 5.e0 , 1.e1                               /

! Table 204 (general positive exponential)

data clrtab(204)%nvals/46/
data clrtab(204)%ifmt(1:2)/-1,-6/
data clrtab(204)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(204)%vals(1:46)/ &
      0.e-2, 1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1,  1.e0,  2.e0,  5.e0,  &
      1.e1,  2.e1,  5.e1,  1.e2,  2.e2,  5.e2,  1.e3,   2.e3,  5.e3,  1.e4,  &
      2.e4,  5.e4,  1.e5,  2.e5,  5.e5,  1.e6,  2.e6,   5.e6,  1.e7,  2.e7,  &
      5.e7,  1.e8,  2.e8,  5.e8,  1.e9,  2.e9,  5.e9,   1.e10, 2.e10, 5.e10, &
      1.e11, 2.e11, 5.e11, 1.e12, 2.e12, 5.e12                               /

! Table 205 (general positive exponential)

data clrtab(205)%nvals/32/
data clrtab(205)%ifmt(1:2)/-1,-6/
data clrtab(205)%ipal(1:32)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131                                  /
data clrtab(205)%vals(1:32)/ &
      0.e-12, 1.e-12, 2.e-12, 5.e-12, 1.e-11, 2.e-11, 5.e-11, 1.e-10, 2.e-10, 5.e-10,  &
      1.e-9,  2.e-9,  5.e-9,  1.e-8,  2.e-8,  5.e-8,  1.e-7,   2.e-7,  5.e-7, 1.e-6,   &
      2.e-6,  5.e-6,  1.e-5,  2.e-5,  5.e-5,  1.e-4,  2.e-4,   5.e-4,  1.e-3, 2.e-3,   &
      5.e-3,  1.e-2                                                                    /

!----------------------------------------------------------------------------
! COLORTABLES 300-399: EXPONENTIAL SCALING WITH NEGATIVE VALUES
!----------------------------------------------------------------------------

! Table 300 (vortz [s^-1],
!           fthrd [K/s],
!           fcorw [s^-1]) 

data clrtab(300)%nvals/22/
data clrtab(300)%ifmt(1:2)/-1,-4/
data clrtab(300)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(300)%vals(1:22)/ &
   -1.e-2,-5.e-3,-2.e-3,-1.e-3,-5.e-4,-2.e-4,-1.e-4,-5.e-5,-2.e-5,-1.e-5, &
       0., 1.e-5, 2.e-5, 5.e-5, 1.e-4, 2.e-4, 5.e-4, 1.e-3, 2.e-3, 5.e-3, &
    1.e-2, 2.e-2                                                          /

! Table 301 (wide range U,W)

data clrtab(301)%nvals/22/
data clrtab(301)%ifmt(1:2)/1,3/
data clrtab(301)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(301)%vals(1:22)/ &
   -200., -100., -50., -20.,  -10., -5.,  -2.,  -1., -.5,  -.2, &
      0.,    .2,   .5,   1.,    2.,  5.,  10.,  20.,  50.,100., &
    200.,  500.                                                 /

! Table 302 (general +/- exponential) 

data clrtab(302)%nvals/22/
data clrtab(302)%ifmt(1:2)/-1,-4/
data clrtab(302)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(302)%vals(1:22)/ &
    -1.e0,-5.e-1,-2.e-1,-1.e-1,-5.e-2,-2.e-2,-1.e-2,-5.e-3,-2.e-3,-1.e-3, &
       0., 1.e-3, 2.e-3, 5.e-3, 1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1, &
     1.e0,  2.e0                                                          /

! Table 303 (vortz [s^-1],
!            fthrd [K/s],
!            fcorw [s^-1]) 

data clrtab(303)%nvals/22/
data clrtab(303)%ifmt(1:2)/-1,-4/
data clrtab(303)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(303)%vals(1:22)/ &
   -1.e-5,-5.e-6,-2.e-6,-1.e-6,-5.e-7,-2.e-7,-1.e-7,-5.e-8,-2.e-8,-1.e-8, &
       0., 1.e-8, 2.e-8, 5.e-8, 1.e-7, 2.e-7, 5.e-7, 1.e-6, 2.e-6, 5.e-6, &
    1.e-5, 2.e-5                                                          /

! Table 304 (XEM, etc)    

data clrtab(304)%nvals/46/
data clrtab(304)%ifmt(1:2)/-1,-6/
data clrtab(304)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(304)%vals(1:46)/ &
   -3.e10, -1.e10, -3.e9, -1.e9, -3.e8, -1.e8, -3.e7, -1.e7, -3.e6, -1.e6, &
    -3.e5,  -1.e5, -3.e4, -1.e4, -3.e3, -1.e3, -3.e2, -1.e2, -3.e1, -1.e1, &
    -3.e0,  -1.e0,    0.,  1.e0,  3.e0,  1.e1,  3.e1,  1.e2,  3.e2,  1.e3, &
     3.e3,   1.e4,  3.e4,  1.e5,  3.e5,  1.e6,  3.e6,  1.e7,  3.e7,  1.e8, &
     3.e8,   1.e9,  3.e9, 1.e01, 3.e10,  1.e11                             /

! Table 305 (general +/- exponential) 

data clrtab(305)%nvals/22/
data clrtab(305)%ifmt(1:2)/-1,-4/
data clrtab(305)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(305)%vals(1:22)/ &
    -3.e1, -1.e1, -3.e0, -1.e0,-3.e-1,-1.e-1,-3.e-2,-1.e-2,-3.e-3,-1.e-3, &
       0., 1.e-3, 3.e-3, 1.e-2, 3.e-2, 1.e-1, 3.e-1,  1.e0,  3.e0,  1.e1, &
     3.e1,  1.e2                                                          /

! Table 306 (general +/- exponential) 

data clrtab(306)%nvals/22/
data clrtab(306)%ifmt(1:2)/-1,-4/
data clrtab(306)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(306)%vals(1:22)/ &
    -3.e-1, -1.e-1, -3.e-2, -1.e-2,-3.e-3,-1.e-3,-3.e-4,-1.e-4,-3.e-5,-1.e-5, &
        0.,  1.e-5,  3.e-5,  1.e-4, 3.e-4, 1.e-3, 3.e-3, 1.e-2, 3.e-2, 1.e-1, &
     3.e-1,  1.e0                                                             /

! Table 307 (general +/- exponential) 

data clrtab(307)%nvals/22/
data clrtab(307)%ifmt(1:2)/-1,-4/
data clrtab(307)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(307)%vals(1:22)/ &
    -1.e4,-5.e3,-2.e3,-1.e3,-5.e2,-2.e2,-1.e2,-5.e1,-2.e1,-1.e1, &
       0., 1.e1, 2.e1, 5.e1, 1.e2, 2.e2, 5.e2, 1.e3, 2.e3, 5.e3, &
     1.e4,  2.e4                                                 /

! Table 308 (vortz [s^-1],
!           fthrd [K/s],
!           fcorw [s^-1]) 

data clrtab(308)%nvals/22/
data clrtab(308)%ifmt(1:2)/-1,-4/
data clrtab(308)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(308)%vals(1:22)/ &
   -1.e-4,-5.e-5,-2.e-5,-1.e-5,-5.e-6,-2.e-6,-1.e-6,-5.e-7,-2.e-7,-1.e-7, &
       0., 1.e-7, 2.e-7, 5.e-7, 1.e-6, 2.e-6, 5.e-6, 1.e-5, 2.e-5, 5.e-5, &
    1.e-4, 2.e-4                                                          /

! Table 309 (general +/- exponential) 

data clrtab(309)%nvals/22/
data clrtab(309)%ifmt(1:2)/-1,-4/
data clrtab(309)%ipal(1:22)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59, 69                                  /
data clrtab(309)%vals(1:22)/ &
    -1.e2, -5.e1, -2.e1, -1.e1,-5.e0,-2.e0,-1.e0,-5.e-1,-2.e-1,-1.e-1, &
       0., 1.e-1, 2.e-1, 5.e-1, 1.e0, 2.e0, 5.e0,  1.e1,  2.e1,  5.e1, &
     1.e2,  2.e2                                                       /

! Table 310 (general +/- exponential) 

data clrtab(310)%nvals/21/
data clrtab(310)%ifmt(1:2)/-1,-4/
data clrtab(310)%ipal(1:21)/ &
    39, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
     7, 51, 52, 53, 54, 55, 56, 57, 58, 59, &
    69                                      /
data clrtab(310)%vals(1:21)/ &
    -1.e1, -5.e0, -2.e0, -1.e0,-5.e-1,-2.e-1,-1.e-1,-5.e-2,-2.e-2,-1.e-2, &
    1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1,  1.e0,  2.e0,  5.e0,  1.e1, &
     2.e1                                                                 /

!----------------------------------------------------------------------------
! COLORTABLES 400-499: IRREGULAR SCALING
!----------------------------------------------------------------------------

! Table 400 (ALL_WATER)

data clrtab(400)%nvals/46/ 
data clrtab(400)%ifmt(1:2)/-2,-5/
data clrtab(400)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(400)%vals(1:46)/ &
      0., 5.e-6, 1.e-5, 2.e-5, 5.e-5, 1.e-4, 2.e-4, 5.e-4, 1.e-3, 2.e-3, &
   5.e-3, 1.e-2, 2.e-2, 5.e-2, 1.e-1, 2.e-1, 5.e-1,    1.,    2.,    3., &
      4.,    5.,    6.,    7.,    8.,    9.,   10.,   12.,   14.,   16., &
     18.,   20.,   30.,   40.,   50.,   60.,   70.,   80.,   90.,  100., &
    200.,  500., 1000., 2000., 5000.,10000.                              /

! Table 401 (TOPM - nonuniform contouring and green land at low elev)

data clrtab(401)%nvals/40/
data clrtab(401)%ifmt(1:2)/1,2/
data clrtab(401)%ipal(1:40)/ &
   151,152,153,154,155,156,157,158,159,160, &
   161,162,163,164,165,166,167,168,169,170, &
   171,172,173,174,175,176,177,178,179,180, &
   181,182,183,184,185,186,187,188,189,190  /
data clrtab(401)%vals(1:40)/ &
    -300.,-200.,-100., -0.1,  0.1,  50., 100., 150., 200., 300., &
     400., 500., 600., 700., 800., 900.,1000.,1200.,1400.,1600., &
    1800.,2000.,2200.,2400.,2600.,2800.,3000.,3500.,4000.,4500., &
    5000.,5500.,6000.,6500.,7000.,7500.,8000.,8500.,9000.,9500.  /

! Table 402 (TOPM - nonuniform contouring)

data clrtab(402)%nvals/40/
data clrtab(402)%ifmt(1:2)/1,2/
data clrtab(402)%ipal(1:40)/ &
   140,139,138,137,100,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,141,142,143  /
data clrtab(402)%vals(1:40)/ &
    -300.,-200.,-100., -0.1,  0.1,  50., 100., 150., 200., 300., &
     400., 500., 600., 700., 800., 900.,1000.,1200.,1400.,1600., &
    1800.,2000.,2200.,2400.,2600.,2800.,3000.,3500.,4000.,4500., &
    5000.,5500.,6000.,6500.,7000.,7500.,8000.,8500.,9000.,9500.  /

! Table 403 (specialized rho)

data clrtab(403)%nvals/22/
data clrtab(403)%ifmt(1:2)/3,5/
data clrtab(403)%ipal(1:22)/ &
   141, 29, 28, 27, 26, 25, 24, 23, 22, 21, &
    20, 50, 51, 52, 53, 54, 55, 56, 57, 58, &
    59,133                                  /
data clrtab(403)%vals(1:22)/ &
   0.   ,1.152,1.153,1.154,1.155,1.156,1.157,1.158,1.159,1.160, &
   1.161,1.162,1.163,1.164,1.165,1.166,1.167,1.168,1.169,1.170, &
   1.171,2.172                                                  /

! Table 404 Specialized specific humidity [g/kg_tot] (for Frances)

data clrtab(404)%nvals/12/
data clrtab(404)%ifmt(1:2)/1,-5/
data clrtab(404)%ipal(1:12)/ &
   241,242,243,244,245,246,247,248,249,250, &
   251,252                                  /
data clrtab(404)%vals(1:12)/ &
    0.0,  0.1,  0.3,  0.7,  1.0,  2.0,  3.0,  4.0,  6.0, 8.0, &
   10.0, 15.0                                                 /

! Table 405 (monocolor for all values)

data clrtab(405)%nvals/2/
data clrtab(405)%ifmt(1:2)/10,10/
data clrtab(405)%ipal(1:2)/ &
   100,100  /
data clrtab(405)%vals(1:2)/ &
    -1.e30,1.e30  /
    
! Table 406 (designed for WM1 scalar field)

data clrtab(406)%nvals/27/
data clrtab(406)%ifmt(1:2)/2,5/
data clrtab(406)%ipal(1:27)/ &
   100,101,102,103,104,    106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127          /
data clrtab(406)%vals(1:27)/ &
    -.25,-.20,-.15,-.10,-.05,      .05, .10, .15, .20, &
     .25, .30, .35, .40, .45, .50, .55, .60, .65, .70, &
     .75, .80, .85, .90, .95,1.00,1.05,1.10            /

! Table 407 (vertical vorticity for NCAR cases)

data clrtab(407)%nvals/22/
data clrtab(407)%ifmt(1:2)/-2,5/
data clrtab(407)%ipal(1:22)/ &
    29, 28, 27, 26, 25, 23, 21, 47, 45, 43, &
    41, 61, 63, 65, 67, 51, 53, 55, 56, 57, &
    58,59                                   /
data clrtab(407)%vals(1:22)/ &
  -15.00e-5,-10.00e-5, -5.00e-5, -4.00e-5, -3.00e-5, &
   -2.00e-5, -1.00e-5, -0.75e-5, -0.50e-5, -0.25e-5, &
    0.00e-5,  0.25e-5,  0.50e-5,  0.75e-5,  1.00e-5, &
    2.00e-5,  3.00e-5,  4.00e-5,  5.00e-5, 10.00e-5, &
   15.00e-5, 20.00e-5                                /

! Table 408 (accum pcp frances)

data clrtab(408)%nvals/8/
data clrtab(408)%ifmt(1:2)/1,-5/
data clrtab(408)%ipal(1:8)/ &
   146,101,110,113,118,126,127,134 /
data clrtab(408)%vals(1:8)/ &
    1.e-1, 1.e0, 1.e1, 5.e1, 1.e2, 2.e2 , 4.e2 , 10.e2 /

! Table 409  (Theta in troposphere [K] for Hurricane Frances)

data clrtab(409)%nvals/41/ 
data clrtab(409)%ifmt(1:2)/0,3/
data clrtab(409)%ipal(1:41)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140                                      /
data clrtab(409)%vals(1:41)/ &
   290., 292., 294., 296., 298., 300., 302., 304., 306., 308., &
   310., 312., 314., 316., 318., 320., 322., 324., 326., 328., &
   330., 333., 336., 339., 342., 345., 348., 351., 354., 357., &
   360., 363., 366., 369., 372., 375., 378., 381., 384., 387., &
   390.                                                        /

! Table 410 (THIL, THETA to upper stratosphere)

data clrtab(410)%nvals/46/
data clrtab(410)%ifmt(1:2)/0,3/
data clrtab(410)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(410)%vals(1:46)/ &
     290., 295., 300., 305., 310., 315., 320., 325., 330., 335., &
     340., 345., 350., 355., 360., 365., 370., 380., 390., 400., &
     450., 500., 550., 600., 650., 700., 750., 800., 850., 900., &
     950.,1000.,1050.,1100.,1150.,1200.,1250.,1300.,1350.,1400., &
    1450.,1500.,1550.,1600.,1650.,1700.                          /

! Table 411 (surface theta)

data clrtab(411)%nvals/46/  
data clrtab(411)%ifmt(1:2)/0,3/
data clrtab(411)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(411)%vals(1:46)/ &
   285.,286.,287.,288.,289.,290.,291.,292.,293.,294., &
   295.,296.,297.,298.,299.,300.,301.,302.,303.,304., &
   305.,306.,307.,308.,309.,310.,311.,312.,313.,314., &
   315.,316.,317.,318.,319.,320.,321.,324.,327.,330., &
   333.,336.,339.,342.,345.,348.                      /

! Table 412 (THIL, THETA in DEEP atm)

data clrtab(412)%nvals/46/
data clrtab(412)%ifmt(1:2)/0,3/
data clrtab(412)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(412)%vals(1:46)/ &
   200., 210., 220., 230., 240., 250., 260., 270., 280., 290., &
   300., 310., 320., 330., 340., 350., 360., 370., 380., 390., &
   400., 450., 500., 550., 600., 650., 700., 750., 800., 850., &
   900., 950.,1000.,1050.,1100.,1150.,1200.,1250.,1300.,1350., &
  1400.,1450.,1500.,1550.,1600.,1650.                          /

! Table 413 (THIL, THETA to lower srat.)

data clrtab(413)%nvals/46/ 
data clrtab(413)%ifmt(1:2)/0,3/
data clrtab(413)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(413)%vals(1:46)/ &
   220., 225., 230., 235., 240., 245., 250., 255., 260., 265., &
   270., 275., 280., 285., 290., 295., 300., 305., 310., 315., &
   320., 325., 330., 335., 340., 345., 350., 355., 360., 365., &
   370., 375., 380., 385., 390., 395., 400., 410., 420., 430., &
   440., 450., 460., 470., 480., 490.                          /

! Table 414 (designed for RHO)

data clrtab(414)%nvals/40/
data clrtab(414)%ifmt(1:2)/2,5/
data clrtab(414)%ipal(1:40)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139  /
data clrtab(414)%vals(1:40)/ &
     .0,  .01,  .02,  .03,  .04,  .05,  .07,  .09,  .11,  .13, &
    .15,  .18,  .21,  .24,  .28,  .32,  .36,  .40,  .45,  .50, &
    .55,  .60,  .65,  .70,  .75,  .80,  .85,  .90,  .95, 1.00, &
   1.05, 1.10, 1.15, 1.20, 1.25, 1.30, 1.35, 1.40, 1.45, 1.50  /

! Table 415 (designed for PRESS in hPa)

data clrtab(415)%nvals/46/
data clrtab(415)%ifmt(1:2)/0,2/
data clrtab(415)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(415)%vals(1:46)/ &
       0.,   1.,   5.,  10.,  15.,  20.,  30.,  40.,  50.,  60., &
      80., 100., 120., 150., 180., 210., 240., 270., 300., 330., &
     360., 390., 420., 450., 480., 510., 540., 570., 600., 630., &
     660., 690., 720., 750., 780., 810., 840., 870., 900., 930., &
     960., 990.,1020.,1050.,1080.,1110.                          /

! Table 416 (sfcwater_energy, soil_energy) 

data clrtab(416)%nvals/36/
data clrtab(416)%ifmt(1:2)/0,2/
data clrtab(416)%ipal(1:36)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135                  /
data clrtab(416)%vals(1:36)/ &
   -50.,-45.,-40.,-35.,-30.,-25.,-20.,-15.,-10., -5., &
     0., 20., 40., 60., 80.,100.,120.,140.,160.,180., &
   200.,220.,240.,260.,280.,300.,320.,340.,360.,380., &
   400.,420.,440.,460.,480.,500.                      /

! Table 417 (Radiative fluxes (defined to be non-negative)

data clrtab(417)%nvals/46/
data clrtab(417)%ifmt(1:2)/0,3/
data clrtab(417)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(417)%vals(1:46)/ &
     0.,  10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90., &
   100., 120., 140., 160., 180., 200., 220., 240., 260., 280., &
   300., 350., 400., 450., 500., 550., 600., 650., 700., 750., &
   800., 850., 900., 950.,1000.,1050.,1100.,1150.,1200.,1250., &
  1300.,1350.,1400.,1450.,1500.,1550.                          /

! Table 418 (pcp for Chapman - comparison with WRF [mm/day])

data clrtab(418)%nvals/15/
data clrtab(418)%ifmt(1:2)/1,6/
data clrtab(418)%ipal(1:15)/ &
   241,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254                      /
data clrtab(418)%vals(1:15)/ &
    0.0, 0.1, 0.2, 0.5, 1.0,  1.5,  2.0,  3.0,  4.0,  5.0, &
    7.5,10.0,15.0,20.0,50.0                                /

!! Table 418 (pcp for Chapman - comparison with WRF [mm/day])

!data clrtab(418)%nvals/20/    ! was 15
!data clrtab(418)%ifmt(1:2)/1,6/
!data clrtab(418)%ipal(1:20)/ &
!   241,241,242,243,244,245,246,247,248,249, &
!   250,251,252,253,254,134,135,136,142,144  /
!data clrtab(418)%vals(1:20)/ &
!    0.0, 0.1, 0.2, 0.5, 1.0,  1.5,  2.0,  3.0,  4.0,  5.0, &
!    7.5,10.0,15.0,20.0,50.0,100.0,200.0,300.0,400.0,999.0  /

! Table 419 (head0)

data clrtab(419)%nvals/45/  
data clrtab(419)%ifmt(1:2)/2,6/
data clrtab(419)%ipal(1:45)/ &
   145,144,143,142,141,140,139,138,137,136, &
   135,134,133,132,131,130,129,128,127,126, &
   125,124,123,122,121,120,119,118,117,116, &
   115,114,113,112,111,110,109,108,107,106, &
   105,104,103,102,101                      /
data clrtab(419)%vals(1:45)/ &
     -500.,-450.,-400.,-350.,-300.,-280.,-260.,-240.,-220.,-200., &
     -150.,-120.,-100., -80., -60., -50., -40., -30., -20., -15., &
      -12., -10.,  -8.,  -6.,  -5.,  -4.,  -3.,  -2.,  -1.,  -.5, &
       -.3,  -.2,  -.1, -.05, -.01,  .01,  .05,   .1,   .3,   .5, &
        1.,   2.,   5.,  10.,  20.                                /

! Table 420 (soil_water (fraction of saturation))

data clrtab(420)%nvals/24/
data clrtab(420)%ifmt(1:2)/2,6/
data clrtab(420)%ipal(1:24)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123                          /
data clrtab(420)%vals(1:24)/ &
   .00, .05, .10, .15, .20, .25, .30, .35, .40, .45, &
   .50, .55, .60, .65, .70, .75, .80, .85, .90, .95, &
   .98, .99,1.00,1.01                                /

! Table 421 (pcp for GPCP comparison [mm/day])

data clrtab(421)%nvals/12/
data clrtab(421)%ifmt(1:2)/1,6/
data clrtab(421)%ipal(1:12)/ &
   71,72,73,74,75,76,77,78,79,80, &
   81,82 /
data clrtab(421)%vals(1:12)/ &
    0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, &
   10.0,99.0 /

! Table 422 (all specific humidities(g/kg))

data clrtab(422)%nvals/28/  
data clrtab(422)%ifmt(1:2)/1,6/
data clrtab(422)%ipal(1:28)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127          /
data clrtab(422)%vals(1:28)/ &
   0.1, 0.2, 0.5,  1.,  2.,  3.,  4.,  5.,  6.,  7., &
    8.,  9., 10., 11., 12., 13., 14., 15., 16., 17., &
   18., 19., 20., 21., 22., 23., 24., 25.            /

! Table 423 (all specific humidities(g/kg))

data clrtab(423)%nvals/31/  
data clrtab(423)%ifmt(1:2)/2,6/
data clrtab(423)%ipal(1:31)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130                                      /
data clrtab(423)%vals(1:31)/ &
    .01, .02, .05, 0.1, 0.2, 0.5,  1.,  2.,  3.,  4., &
     5.,  6.,  7.,  8.,  9., 10., 11., 12., 13., 14., &
    15., 16., 17., 18., 19., 20., 21., 22., 23., 24., &
    25.                                               /

! Table 424 (THIL, THETA to upper stratosphere)

data clrtab(424)%nvals/46/
data clrtab(424)%ifmt(1:2)/0,3/
data clrtab(424)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(424)%vals(1:46)/ &
     290., 295., 300., 305., 310., 315., 320., 325., 330., 335., &
     340., 345., 350., 355., 360., 365., 370., 380., 390., 400., &
     420., 440., 460., 480., 500., 520., 540., 560., 580., 600., &
     620., 640., 660., 680., 700., 720., 740., 760., 780., 800., &
     820., 840., 860., 880., 900., 920.                          /

! Table 425 (pcp for MJO demo [mm/day])

data clrtab(425)%nvals/20/
data clrtab(425)%ifmt(1:2)/1,6/
data clrtab(425)%ipal(1:20)/ &
   241,241,242,243,244,245,246,247,248,249, &
   250,251,252,253,254,221,222,223,224,225  /
data clrtab(425)%vals(1:20)/ &
     0.0,  0.1,  1.0,  3.0,  5.0, 10.0, 15.0, 20.0, 25.0, 30.0, &
    35.0, 40.0, 45.0, 50.0, 60.0, 70.0, 80.0, 90.0,100.0,110.0  /

! Table 426 (z_bedrock)

data clrtab(426)%nvals/26/  
data clrtab(426)%ifmt(1:2)/1,6/
data clrtab(426)%ipal(1:26)/ &
   141,140,139,138,137,136,135,134,133,132, &
   131,130,129,124,123,122,121,120,119,118, &
   117,116,111,110,109,108                  /
data clrtab(426)%vals(1:26)/ &
   -500.,-400.,-300.,-200.,-150.,-100., -80., -60., -50., -40., &
    -30., -20., -15., -10.,  -8.,  -6.,  -5.,  -4.,  -3.,  -2., &
     -1.,  -.5,  -.3,  -.2,  -.1,   0.                          /

! Table 427 (glhymps ksat)

data clrtab(427)%nvals/17/
data clrtab(427)%ifmt(1:2)/-1,-6/
data clrtab(427)%ipal(1:17)/ &
   141,108,109,110,111,116,117,118,119,120, &
   121,122,123,124,129,130,131              /
data clrtab(427)%vals(1:17)/ &
   1.e-14, 1.e-12, 1.e-10, 3.e-10, 1.e-9, 3.e-9, 1.e-8, 3.e-8, 1.e-7, 3.e-7, &
   1.e-6,  3.e-6,  1.e-5,  3.e-5,  1.e-4, 3.e-4, 1.e-3                       /

! Table 428 (glhymps poros)

data clrtab(428)%nvals/23/
data clrtab(428)%ifmt(1:2)/2,6/
data clrtab(428)%ipal(1:23)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133                              /
data clrtab(428)%vals(1:23)/ &
   .00, .01, .02, .03, .04, .05, .06, .07, .08, .09, &
   .10, .12, .14, .16, .18, .20, .22, .24, .26, .28, &
   .30, .32, .34                                     /

! Table 429 (gpp)

data clrtab(429)%nvals/24/
data clrtab(429)%ifmt(1:2)/0,5/
data clrtab(429)%ipal(1:24)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133,134                          /
data clrtab(429)%vals(1:24)/ &
      0., 100., 200., 300., 400., 500., 600., 700., 800., 900., &
   1000.,1200.,1400.,1600.,1800.,2000.,2200.,2400.,2600.,2800., &
   3000.,3200.,3400.,3600.                                      /

! Table 430 (clay, silt, sand fraction)

data clrtab(430)%nvals/28/
data clrtab(430)%ifmt(1:2)/2,6/
data clrtab(430)%ipal(1:28)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133,134,135,136,137,142          /
data clrtab(430)%vals(1:28)/ &
   .00, .02, .04, .06, .08, .10, .12, .14, .16, .18, &
   .20, .25, .30, .35, .40, .45, .50, .55, .60, .65, &
   .70, .75, .80, .85, .90, .95,1.00,1.05             /

! Table 431 (organic specific density)

data clrtab(431)%nvals/26/
data clrtab(431)%ifmt(1:2)/0,6/
data clrtab(431)%ipal(1:26)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133,134,135,136                  /
data clrtab(431)%vals(1:26)/ &
    0., 10., 20., 30., 40., 50., 60., 70., 80., 90., &
  100.,120.,140.,160.,180.,200.,250.,300.,350.,400., &
  450.,500.,550.,600.,650.,700.                      / 

! Table 432 (soil dry bulk density)

data clrtab(432)%nvals/23/
data clrtab(432)%ifmt(1:2)/0,6/
data clrtab(432)%ipal(1:23)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133                              /
data clrtab(432)%vals(1:23)/ &
      0., 100., 200., 300., 400., 500., 600., 700., 880., 980., &
   1080.,1100.,1200.,1300.,1400.,1500.,1600.,1700.,1800.,1900., &
   2000.,2100.,2200.                                            / 

! Table 433 (soil cation exchange capacity)

data clrtab(433)%nvals/24/
data clrtab(433)%ifmt(1:2)/0,6/
data clrtab(433)%ipal(1:24)/ &
   141,104,105,106,107,108,109,110,111,116, &
   117,118,119,120,121,122,123,124,129,130, &
   131,132,133,134                          /
data clrtab(433)%vals(1:24)/ &
     0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9., &
    10., 15., 20., 30., 40., 50., 60., 80.,100.,120., &
   140.,160.,180.,200.                                / 

! Table 434 (soil pH)

data clrtab(434)%nvals/17/
data clrtab(434)%ifmt(1:2)/1,6/
data clrtab(434)%ipal(1:17)/ &
   141,108,109,110,111,116,117,118,119,120, &
   121,122,123,124,129,130,131              /
data clrtab(434)%vals(1:17)/ &
    0.0, 3.0, 3.5, 4.0, 4.5,  5.0,  5.5, 6.0, 6.5, 7.0, &
    7.5, 8.0, 8.5, 9.0, 9.5, 10.0, 10.5                 / 

! Table 435 (CO2 concentration (ppmv of dry air)

data clrtab(435)%nvals/46/
data clrtab(435)%ifmt(1:2)/0,2/
data clrtab(435)%ipal(1:46)/ &
   100,101,102,103,104,105,106,107,108,109, &
   110,111,112,113,114,115,116,117,118,119, &
   120,121,122,123,124,125,126,127,128,129, &
   130,131,132,133,134,135,136,137,138,139, &
   140,141,142,143,144,145                  /
data clrtab(435)%vals(1:46)/ &
     
     300.,310.,320.,330.,340.,350.,360.,370.,380.,390., &
     391.,392.,393.,394.,395.,396.,397.,398.,399.,400., &
     401.,402.,403.,404.,405.,406.,407.,408.,409.,410., &
     420.,430.,440.,450.,460.,470.,480.,490.,500.,510., &
     520.,530.,540.,550.,560.,570.                      /

Contains

!===============================================================================

   subroutine gks_colors(iwk)

   implicit none
   integer, intent(in) :: iwk

   call o_hls(iwk, 1,120., 70.,100.)  ! red
   call o_hls(iwk, 2,180., 62., 90.)  ! yellow
   call o_hls(iwk, 3,265., 75., 70.)  ! green
   call o_hls(iwk, 4,290., 55., 70.)  ! cyan
   call o_hls(iwk, 5,188., 50., 75.)  ! yellow-green
   call o_hls(iwk, 6,350., 81., 75.)  ! blue
   call o_hls(iwk, 7, 00.,100.,  0.)  ! white
   call o_hls(iwk, 8,350., 60.,100.)  ! dark blue
   call o_hls(iwk, 9,265., 33., 90.)  ! dark green
   call o_hls(iwk,10, 00.,  0.,  0.)  ! black
   call o_hls(iwk,11, 50., 60.,100.)  ! purple
   call o_hls(iwk,12,135., 40., 70.)  ! dark red for roads
   call o_hls(iwk,13, 00., 43.,  0.)  ! gray for roads
   call o_hls(iwk,14,  0., 40., 80.)  ! test
   call o_hls(iwk,15,290., 90., 70.)  ! blue-green for canopy air
   call o_hls(iwk,16,150., 60.,100.)  ! orange
   call o_hls(iwk,17,150., 50., 50.)  ! brown
   call o_hls(iwk,18,340., 80.,100.)  ! light blue

! Finer shades

!   call o_hls (iwk, 20,350.,94., 99.) ! blues (classic3)
!   call o_hls (iwk, 21,350.,87., 99.)
!   call o_hls (iwk, 22,350.,79., 99.)
!   call o_hls (iwk, 23,350.,71., 99.)
!   call o_hls (iwk, 24,350.,63., 99.)
!   call o_hls (iwk, 25,350.,54., 99.)
!   call o_hls (iwk, 26,350.,45., 99.)
!   call o_hls (iwk, 27,350.,36., 99.)
!   call o_hls (iwk, 28,350.,28., 99.)

   call o_hls (iwk, 20,350.,96., 70.) ! blues
   call o_hls (iwk, 21,350.,92., 70.)
   call o_hls (iwk, 22,350.,88., 70.)
   call o_hls (iwk, 23,350.,83., 70.)
   call o_hls (iwk, 24,350.,77., 70.)
   call o_hls (iwk, 25,350.,71., 70.)
   call o_hls (iwk, 26,350.,64., 70.)
   call o_hls (iwk, 27,350.,56., 70.)
   call o_hls (iwk, 28,350.,46., 70.)
   call o_hls (iwk, 29,350.,38., 70.)

   call o_hls (iwk, 30,302.,92., 70.) ! blue greens
   call o_hls (iwk, 31,302.,88., 70.)
   call o_hls (iwk, 32,302.,83., 70.)
   call o_hls (iwk, 33,302.,77., 70.)
   call o_hls (iwk, 34,302.,71., 70.)
   call o_hls (iwk, 35,302.,64., 70.)
   call o_hls (iwk, 36,302.,56., 70.)
   call o_hls (iwk, 37,302.,46., 70.)
   call o_hls (iwk, 38,302.,38., 70.)
   call o_hls (iwk, 39,302.,30., 70.)

   call o_hls (iwk, 40,270.,96., 70.) ! greens
   call o_hls (iwk, 41,270.,92., 70.)
   call o_hls (iwk, 42,270.,88., 70.)
   call o_hls (iwk, 43,270.,83., 70.)
   call o_hls (iwk, 44,270.,77., 70.)
   call o_hls (iwk, 45,270.,71., 70.)
   call o_hls (iwk, 46,270.,64., 70.)
   call o_hls (iwk, 47,270.,56., 70.)
   call o_hls (iwk, 48,270.,46., 70.)
   call o_hls (iwk, 49,270.,38., 70.)

!   call o_hls (iwk, 50,120.,94., 99.) ! reds (classic 3)
!   call o_hls (iwk, 51,120.,86., 99.)
!   call o_hls (iwk, 52,120.,77., 99.)
!   call o_hls (iwk, 53,120.,69., 99.)
!   call o_hls (iwk, 54,120.,58., 99.)
!   call o_hls (iwk, 55,120.,46., 99.)
!   call o_hls (iwk, 56,120.,39., 99.)
!   call o_hls (iwk, 57,120.,32., 99.)
!   call o_hls (iwk, 58,120.,26., 80.)

   call o_hls (iwk, 50,135.,96., 70.) ! reds
   call o_hls (iwk, 51,135.,92., 70.)
   call o_hls (iwk, 52,135.,88., 70.)
   call o_hls (iwk, 53,135.,83., 70.)
   call o_hls (iwk, 54,135.,77., 70.)
   call o_hls (iwk, 55,135.,71., 70.)
   call o_hls (iwk, 56,135.,64., 70.)
   call o_hls (iwk, 57,135.,56., 70.)
   call o_hls (iwk, 58,135.,46., 70.)
   call o_hls (iwk, 59,135.,38., 70.)

   call o_hls (iwk, 60,170.,92., 70.) ! oranges
   call o_hls (iwk, 61,170.,88., 70.)
   call o_hls (iwk, 62,170.,83., 70.)
   call o_hls (iwk, 63,170.,77., 70.)
   call o_hls (iwk, 64,170.,71., 70.)
   call o_hls (iwk, 65,170.,64., 70.)
   call o_hls (iwk, 66,170.,56., 70.)
   call o_hls (iwk, 67,170.,46., 70.)
   call o_hls (iwk, 68,170.,38., 70.)
   call o_hls (iwk, 69,170.,30., 70.)

! set of 50 colors

  call o_hls (iwk,100,340.,72., 95.) ! blues
  call o_hls (iwk,101,340.,54., 95.)
  call o_hls (iwk,102,340.,40., 95.)
  call o_hls (iwk,103,340.,30., 95.)

  call o_hls (iwk,104,300.,72., 80.) ! green blues
  call o_hls (iwk,105,300.,49., 80.)
  call o_hls (iwk,106,300.,39., 80.)
  call o_hls (iwk,107,300.,30., 80.)

  call o_hls (iwk,108,270.,70., 85.) ! greens
  call o_hls (iwk,109,270.,51., 85.)
  call o_hls (iwk,110,270.,40., 85.)
  call o_hls (iwk,111,270.,30., 85.)

  call o_hls (iwk,112,194.,70., 85.) ! yellow greens
  call o_hls (iwk,113,194.,51., 85.)
  call o_hls (iwk,114,194.,39., 85.)
  call o_hls (iwk,115,194.,30., 85.)

  call o_hls (iwk,116,180.,62.,100.)  ! yellow

  call o_hls (iwk,117,155.,70.,100.) ! oranges
  call o_hls (iwk,118,155.,53.,100.)
  call o_hls (iwk,119,155.,42.,100.)
  call o_hls (iwk,120,155.,33.,100.)

  call o_hls (iwk,121,135.,70., 90.) ! reds
  call o_hls (iwk,122,135.,53., 90.)
  call o_hls (iwk,123,135.,40., 90.)
  call o_hls (iwk,124,135.,32., 90.)

  call o_hls (iwk,125, 85.,70., 90.) ! magentas
  call o_hls (iwk,126, 85.,53., 90.)
  call o_hls (iwk,127, 85.,40., 90.)
  call o_hls (iwk,128, 85.,32., 90.)

  call o_hls (iwk,129, 50.,70., 90.)  ! violets
  call o_hls (iwk,130, 50.,53., 90.)
  call o_hls (iwk,131, 50.,41., 90.)
  call o_hls (iwk,132, 50.,32., 90.)

  call o_hls (iwk,133, 20.,75., 90.)  ! violet blues
  call o_hls (iwk,134, 20.,60., 90.)
  call o_hls (iwk,135, 20.,45., 90.)
  call o_hls (iwk,136, 20.,32., 90.)

  call o_hls (iwk,137,150.,70., 40.)  ! browns
  call o_hls (iwk,138,150.,52., 40.)
  call o_hls (iwk,139,150.,40., 40.)
  call o_hls (iwk,140,150.,30., 40.)

  call o_hls (iwk,141, 00., 85.,  0.)
  call o_hls (iwk,142, 00., 70.,  0.)  ! gray shades
  call o_hls (iwk,143, 00., 52.,  0.)
  call o_hls (iwk,144, 00., 40.,  0.)
  call o_hls (iwk,145, 00., 30.,  0.)

  call o_hls (iwk,146, 00.,100.,  0.)  ! white

! Special for topo table 28

   call o_hls (iwk,151,290., 33., 75.)  ! Blue greens - below sea level
   call o_hls (iwk,152,290., 41., 75.)
   call o_hls (iwk,153,290., 55., 75.)
   call o_hls (iwk,154,290., 77., 75.)

   call o_hls (iwk,155,350., 70., 75.)  ! Blue - at sea level

   call o_hls (iwk,156,270., 33., 75.)  ! Greens
   call o_hls (iwk,157,270., 41., 75.)
   call o_hls (iwk,158,270., 50., 75.)
   call o_hls (iwk,159,270., 70., 75.)

   call o_hls (iwk,160,190., 33., 75.)  ! Yellow-greens
   call o_hls (iwk,161,190., 41., 75.)
   call o_hls (iwk,162,190., 50., 75.)
   call o_hls (iwk,163,190., 70., 75.)

   call o_hls (iwk,164,180., 62., 90.)  ! Yellow

   call o_hls (iwk,165,165., 73.,100.)  ! Oranges
   call o_hls (iwk,166,165., 60.,100.)
   call o_hls (iwk,167,165., 47.,100.)
   call o_hls (iwk,168,165., 42.,100.)

   call o_hls (iwk,169,150., 67., 50.)  ! Browns
   call o_hls (iwk,170,150., 57., 50.)
   call o_hls (iwk,171,150., 44., 50.)
   call o_hls (iwk,172,150., 40., 50.)

   call o_hls (iwk,173,135., 77., 70.)  ! Reds
   call o_hls (iwk,174,135., 66., 70.)
   call o_hls (iwk,175,135., 48., 70.)
   call o_hls (iwk,176,135., 40., 70.)
   
   call o_hls (iwk,177, 85., 80., 70.)  ! Magentas
   call o_hls (iwk,178, 85., 68., 70.)
   call o_hls (iwk,179, 85., 49., 70.)
   call o_hls (iwk,180, 85., 40., 70.)

   call o_hls (iwk,181, 50., 80., 70.)  ! Violets
   call o_hls (iwk,182, 50., 68., 70.)
   call o_hls (iwk,183, 50., 49., 70.)
   call o_hls (iwk,184, 50., 40., 70.)

   call o_hls (iwk,185, 20., 80., 70.)  ! Violet blues
   call o_hls (iwk,186, 20., 68., 70.)
   call o_hls (iwk,187, 20., 49., 70.)
   call o_hls (iwk,188, 20., 40., 70.)

   call o_hls (iwk,189, 00., 60.,  0.)  ! Grays
   call o_hls (iwk,190, 00., 80.,  0.)  

! Special for topo table 29

   call o_hls (iwk,191,290., 33., 95.)  ! Blue greens - below sea level
   call o_hls (iwk,192,290., 41., 95.)
   call o_hls (iwk,193,290., 55., 95.)
   call o_hls (iwk,194,290., 77., 95.)

   call o_hls (iwk,195,350., 70., 95.)  ! Blue - at sea level

   call o_hls (iwk,196,270., 33., 95.)  ! Greens
   call o_hls (iwk,197,270., 41., 95.)
   call o_hls (iwk,198,270., 50., 95.)
   call o_hls (iwk,199,270., 70., 95.)

   call o_hls (iwk,200,190., 33., 95.)  ! Yellow-greens
   call o_hls (iwk,201,190., 41., 95.)
   call o_hls (iwk,202,190., 50., 95.)
   call o_hls (iwk,203,190., 70., 95.)

   call o_hls (iwk,204,180., 62., 90.)  ! Yellow

   call o_hls (iwk,205,165., 73.,100.)  ! Browns
   call o_hls (iwk,206,165., 60.,100.)
   call o_hls (iwk,207,165., 47.,100.)
   call o_hls (iwk,208,165., 42.,100.)

   call o_hls (iwk,209,150., 67., 90.)  ! Oranges
   call o_hls (iwk,210,150., 57., 90.)
   call o_hls (iwk,211,150., 44., 90.)
   call o_hls (iwk,212,150., 40., 90.)

   call o_hls (iwk,213,135., 77., 90.)  ! Reds
   call o_hls (iwk,214,135., 66., 90.)
   call o_hls (iwk,215,135., 48., 90.)
   call o_hls (iwk,216,135., 40., 90.)
   
   call o_hls (iwk,217, 85., 80., 90.)  ! Magentas
   call o_hls (iwk,218, 85., 68., 90.)
   call o_hls (iwk,219, 85., 49., 90.)
   call o_hls (iwk,220, 85., 40., 90.)

   call o_hls (iwk,221, 50., 80., 90.)  ! Violets
   call o_hls (iwk,222, 50., 68., 90.)
   call o_hls (iwk,223, 50., 49., 90.)
   call o_hls (iwk,224, 50., 40., 90.)

   call o_hls (iwk,225, 20., 80., 90.)  ! Violet blues
   call o_hls (iwk,226, 20., 68., 90.)
   call o_hls (iwk,227, 20., 49., 90.)
   call o_hls (iwk,228, 20., 40., 90.)

   call o_hls (iwk,229, 00., 60.,  0.)  ! Grays
   call o_hls (iwk,230, 00., 80.,  0.)

! Extras - for precip (frances) 

!   call o_hls (iwk,241,350.,90., 95.)  ! Blue
!   call o_hls (iwk,242,350.,65., 95.)  ! Blue
!   call o_hls (iwk,243,270.,65., 90.)  ! Green
!   call o_hls (iwk,244,270.,45., 90.)  ! Green
!   call o_hls (iwk,245,180.,62., 90.)  ! Yellow
!   call o_hls (iwk,246,160.,70., 90.)  ! Orange
!   call o_hls (iwk,247,160.,50., 90.)  ! Orange
!   call o_hls (iwk,248,120.,70., 90.)  ! Red
!   call o_hls (iwk,249,120.,50., 90.)  ! Red
!   call o_hls (iwk,250, 50.,70., 90.)  ! Violet
!   call o_hls (iwk,251, 50.,45., 90.)  ! Violet
!   call o_hls (iwk,252,  0.,80.,  0.)  ! Gray
!   call o_hls (iwk,253,  0.,60.,  0.)  ! Gray
!   call o_hls (iwk,254,  0.,45.,  0.)  ! Gray
!   call o_hls (iwk,255,  0.,99.,  0.)  ! White

! Extras - for precip (Chapman - OLAM & WRF) 

!   call o_hls (iwk,241,  0.,99.,  0.)  ! White
!   call o_hls (iwk,242,  0.,81.,100.)  ! Blue
!   call o_hls (iwk,243,  0.,66.,100.)  ! Blue
!   call o_hls (iwk,244,  0.,53.,100.)  ! Blue
!   call o_hls (iwk,245,  0.,44.,100.)  ! Blue
!   call o_hls (iwk,246,295.,25.,100.)  ! Blue Green
!   call o_hls (iwk,247,240.,30.,100.)  ! Green
!   call o_hls (iwk,248,240.,40.,100.)  ! Green
!   call o_hls (iwk,249,210.,45.,100.)  ! Yellow Green
!   call o_hls (iwk,250,190.,57.,100.)  ! Green Yellow
!   call o_hls (iwk,251,180.,50.,100.)  ! Yellow
!   call o_hls (iwk,252,160.,50.,100.)  ! Orange
!   call o_hls (iwk,253,140.,50.,100.)  ! Red Orange
!   call o_hls (iwk,254,120.,50.,100.)  ! Red

! Extras - for dcmip 2016 Terminator 

   call o_hls (iwk,241,356.,54.,95.)
   call o_hls (iwk,242,337.,63.,99.)
   call o_hls (iwk,243,309.,63.,99.)
   call o_hls (iwk,244,290.,52.,80.)
   call o_hls (iwk,245,269.,36.,92.)
   call o_hls (iwk,246,240.,36.,92.)
   call o_hls (iwk,247,188.,46.,99.)
   call o_hls (iwk,248,170.,62.,100.)
   call o_hls (iwk,249,161.,57.,100.)
   call o_hls (iwk,250,150.,54.,100.)
   call o_hls (iwk,251,140.,54.,93.)
   call o_hls (iwk,252,120.,50.,93.)
   call o_hls (iwk,253,100.,44.,93.)
   call o_hls (iwk,254, 91.,30.,93.)
   call o_hls (iwk,255, 39.,26.,93.)

! Extras - for precip (GPCP v22) 

   call o_hls (iwk,71,  0.,99.,  0.)  ! White
   call o_hls (iwk,72,305.,90.,100.)  ! Light Blue
   call o_hls (iwk,73,255.,80., 90.)  ! Light Green
   call o_hls (iwk,74,265.,50., 65.)  ! Green
   call o_hls (iwk,75,180.,50.,100.)  ! Yellow
   call o_hls (iwk,76,150.,50.,100.)  ! Orange
   call o_hls (iwk,77,100.,80.,100.)  ! Darkish Pink
   call o_hls (iwk,78,120.,50.,100.)  ! Red
   call o_hls (iwk,79, 60.,80., 90.)  ! Light violet
   call o_hls (iwk,80, 40.,42., 95.)  ! Violet
   call o_hls (iwk,81,140.,30.,100.)  ! Reddish brown
   call o_hls (iwk,82,  0., 0.,  0.)  ! Black

! NCAR

!   call o_hls (iwk,230, 50.,45., 90.)  ! Violet
!   call o_hls (iwk,231, 50.,65., 90.)  ! Violet
!   call o_hls (iwk,232, 50.,80., 90.)  ! Violet
!   call o_hls (iwk,233,350.,40., 75.)  ! blue
!   call o_hls (iwk,234,350.,50., 95.)  ! blue
!   call o_hls (iwk,235,350.,65., 95.)  ! Blue
!   call o_hls (iwk,236,350.,80., 95.)  ! Blue
!   call o_hls (iwk,237,270.,35., 97.)  ! Green
!   call o_hls (iwk,238,270.,50., 97.)  ! Green
!   call o_hls (iwk,239,270.,70., 97.)  ! Green
!   call o_hls (iwk,240,193.,35., 90.)  ! Yellow Green
!   call o_hls (iwk,241,193.,55., 90.)  ! Yellow Green
!   call o_hls (iwk,242,193.,70., 90.)  ! Yellow Green

!   call o_hls (iwk,243,180.,72., 95.)  ! Yellow
!   call o_hls (iwk,244,150.,80., 95.)  ! Orange
!   call o_hls (iwk,245,150.,65., 95.)  ! Orange
!   call o_hls (iwk,246,150.,50., 95.)  ! Orange
!   call o_hls (iwk,247,150.,40., 95.)  ! Orange
!   call o_hls (iwk,248,120.,78., 95.)  ! Red
!   call o_hls (iwk,249,120.,62., 95.)  ! Red
!   call o_hls (iwk,250,120.,48., 95.)  ! Red
!   call o_hls (iwk,251,120.,38., 95.)  ! Red
!   call o_hls (iwk,252,120.,80.,  0.)  ! Gray
!   call o_hls (iwk,253,120.,65.,  0.)  ! Gray
!   call o_hls (iwk,254,120.,50.,  0.)  ! Gray
!   call o_hls (iwk,255,120.,40.,  0.)  ! Gray

! H*Wind color palette
!  call gscr (iwk, ic, r, g, b)
!   call gscr (iwk, 201, 0.      , 0.078431, 1.      )
!   call gscr (iwk, 202, 0.      , 0.121569, 1.      )
!   call gscr (iwk, 203, 0.      , 0.290196, 1.      )
!   call gscr (iwk, 204, 0.      , 0.450980, 1.      )
!   call gscr (iwk, 205, 0.      , 0.611765, 1.      )
!   call gscr (iwk, 206, 0.      , 0.772549, 1.      )
!   call gscr (iwk, 207, 0.      , 0.901961, 1.      )
!   call gscr (iwk, 208, 0.      , 1.      , 1.      )
!   call gscr (iwk, 209, 0.      , 1.      , 0.749020)
!   call gscr (iwk, 210, 0.      , 1.      , 0.529412)
!   call gscr (iwk, 211, 0.      , 1.      , 0.      )
!   call gscr (iwk, 212, 0.654902, 1.      , 0.      )
!   call gscr (iwk, 213, 0.811765, 1.      , 0.      )
!   call gscr (iwk, 214, 1.      , 1.      , 0.      )
!   call gscr (iwk, 215, 1.      , 0.529412, 0.      )
!   call gscr (iwk, 216, 1.      , 0.372549, 0.      )
!   call gscr (iwk, 217, 1.      , 0.152941, 0.      )
!   call gscr (iwk, 218, 1.      , 0.      , 0.466667)
!   call gscr (iwk, 219, 1.      , 0.      , 0.623529)
!   call gscr (iwk, 220, 1.      , 0.      , 0.780392)
!   call gscr (iwk, 221, 1.      , 0.      , 0.972549)
!   call gscr (iwk, 222, 0.968627, 0.      , 1.      )
!   call gscr (iwk, 223, 0.925490, 0.      , 1.      )
!   call gscr (iwk, 224, 0.878431, 0.      , 1.      )
!   call gscr (iwk, 225, 0.831373, 0.      , 1.      )
!   call gscr (iwk, 226, 0.788235, 0.      , 1.      )
!   call gscr (iwk, 227, 0.745098, 0.      , 1.      )
!   call gscr (iwk, 228, 0.701961, 0.      , 1.      )
!   call gscr (iwk, 229, 0.658824, 0.      , 1.      )

   end subroutine gks_colors

End Module plotcolors

